--[[ Copyright (c) 2020 Dimitris Iliadis

Permission is hereby granted, free of charge, to any person obtaining a copy of
this software and associated documentation files (the "Software"), to deal in
the Software without restriction, including without limitation the rights to
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
of the Software, and to permit persons to whom the Software is furnished to do
so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE. --]]

Font("unicode")
Language("Ελληνικά", "Greek", "el", "gre", "ell")
Inherit("English")
Encoding(utf8)
IsArabicNumerals(true)

menu_file = {
  quit = "  (%1%) ΕΞΟΔΟΣ   ",
  save = "  (%1%) ΑΠΟΘΗΚΕΥΣΗ   ",
  load = "  (%1%) ΦΟΡΤΩΣΗ  ",
  restart = "  (%1%) ΕΠΑΝΕΚΚΙΝΗΣΗ",
}
staff_descriptions = {
  misc = {
    [1] = "Παίζει γκολφ. ",
    [2] = "Πηγαίνει scuba diving. ",
    [3] = "Φτιάχνει γλυπτά από πάγο. ",
    [4] = "Πίνει κρασί. ",
    [5] = "Οδηγεί αγωνιστικά αυτοκίνητα. ",
    [6] = "Κάνει bungee jumping. ",
    [7] = "Συλλέγει σουβέρ μπύρας. ",
    [8] = "Γουστάρει το stage-diving. ",
    [9] = "Απολαμβάνει το fart-surfing. ",
    [10] = "Απολαμβάνει το river-widening. ",
    [11] = "Διυλίζει ουίσκι. ",
    [12] = "Ειδικός στις DIY κατασκευές. ",
    [13] = "Απολαμβάνει καλλιτεχνικές ταινίες του Γαλλικού κινηματογράφου. ",
    [14] = "Παίζει πολύ Theme Park. ",
    [15] = "Έχει άδεια φορτηγατζή. ",
    [16] = "Πάει για αγώνες με μοτοσυκλέτες. ",
    [17] = "Παίζει κλασικό βιολί και τσέλο. ",
    [18] = "Ενθουσιώδης καταστροφέας. ",
    [19] = "Λάτρεις των σκύλων. ",
    [20] = "Ακούει ραδιόφωνο. ",
    [21] = "Κάνει συχνά μπάνιο. ",
    [22] = "Raffia-work instructor. ",
    [23] = "Ανοίγει λαχανικά για να τα χρησιμοποιεί σαν σαπωνοθήκες. ",
    [24] = "Ειδικός Αστυφύλαξ μερικής απασχόλησης. ",
    [25] = "Παρουσιαστής τηλεπαιχνιδιών γνώσεων. ",
    [26] = "Συλλέγει θραύσματα βλημάτων του ΒΠΠ. ",
    [27] = "Αναδιατάσσει έπιπλα. ",
    [28] = "Ακούει rave και trip-hop. ",
    [29] = "Εκνευρίζει έντομα με αποσμητικό σπρέι. ",
    [30] = "Διακόπτει και εκνευρίζει κακούς κωμικούς. ",
    [31] = "Κατασκοπεύει για το συμβούλιο. ",
    [32] = "Μουλωχτός/ή κηπουρός. ",
    [33] = "Κάνει λαθρεμπόριο ρολογιών μαϊμού. ",
    [34] = "Τραγουδάει σε συγκρότημα rock and roll. ",
    [35] = "Λατρεύει την πρωινή τηλεόραση. ",
    [36] = "Γαργαλάει πέστροφες. ",
    [37] = "Ξεγελάει τουρίστες σε μουσεία. ",
  },
  good = {
    [1] = "Πολύ γρήγορος/η και επιμελής εργάτης. ",
    [2] = "Πολύ ευσυνείδητος/η. Νοιάζεται βαθύτατα. ",
    [3] = "Έχει ευρεία γκάμα δεξιοτήτων. ",
    [4] = "Πολύ φιλικός/ή και του/της αρέσει να γελάει. ",
    [5] = "Εξαιρετικές αντοχές. Απλά συνεχίζει δίχως σταματημό. ",
    [6] = "Αξιοσημείωτα ευγενικός/ή με εγκάρδια συμπεριφορά. ",
    [7] = "Απίστευτα ταλαντούχους/α και ικανός/ή. ",
    [8] = "Νοιάζεται πολύ για την δουλειά που έχει αναλάβει. ",
    [9] = "Είναι τελειομανής που δεν τα παρατάει ποτέ. ",
    [10] = "Αφοσειωμένος/η στο να βοηθάει τους ανθρώπους, πάντα με ένα χαμόγελο. ",
    [11] = "Γοητευτικός/ή, ευγενικός/ή και βοηθητικός/ή. ",
    [12] = "Γενικά αφοσειωμένός/η με κίνητρα. ",
    [13] = "Καλοσυνάτος/η και εργάζεται πολύ σκληρά. ",
    [14] = "Πιστός/ή και φιλικός/ή. ",
    [15] = "Προσεκτικός/ή και αξιόπιστος/η σε μια ανάγκη. ",
  },
  bad = {
    [1] = "Αργός/ή και γκρινιάρης/α. ",
    [2] = "Τεμπέλης/α δίχως κίνητρα. ",
    [3] = "Ανεπαρκώς εκπαιδευμένος/η και άσκοπος/η. ",
    [4] = "Αγενής και τραχύς/τραχιά. Εκνευρίζει τους άλλους. ",
    [5] = "Άθλιες αντοχές - έχει κακή συμπεριφορά. ",
    [6] = "Στου κουφού την πόρτα... Τα ρούχα του/της μυρίζουν λάχανο. ",
    [7] = "Χάλια στην δουλειά. Αναξιόπιστος/η. ",
    [8] = "Απερίσκεπτος/η και η προσοχή του/της αποσπάται εύκολα. ",
    [9] = "Στρεσσαρισμένος/η και επιρρεπής σε λάθη. ",
    [10] = "Διεφθαρμένος/η και μνησίκακος/η - καίγεται με μίσος. ",
    [11] = "Απρόσεκτος/η και επιρρεπής σε ατυχήματα. ",
    [12] = "Δεν ασχολείται με την δουλειά. Τεμπέλης/α ως το κόκαλο. ",
    [13] = "Παίρνει ανόητα ρίσκα και δεν θα μπορούσε να νοιάζεται λιγότερο. ",
    [14] = "Πονηρός/ή, ύπουλος/η και ανατρεπτικός/ή. ",
    [15] = "Αγέρωχος/η και ψωνισμένος/η. ",
  },
}
handyman_window = {
  all_parcels = "Όλα τα plots",
  parcel = "Plot",
}
months = {
  [1] = "Ιαν",
  [2] = "Φεβ",
  [3] = "Μαρ",
  [4] = "Απρ",
  [5] = "Μαϊ",
  [6] = "Ιουν",
  [7] = "Ιουλ",
  [8] = "Αυγ",
  [9] = "Σεπ",
  [10] = "Οκτ",
  [11] = "Νοε",
  [12] = "Δεκ",
}
adviser = {
  research = {
    new_drug_researched = "Ένα νέο φάρμακο για την θεραπεία τ@ %s έχει ανακαλυφθεί.",
    drug_improved = "Το φάρμακο %s έχει βελτιωθεί από το Τμήμα Ερευνών σας.",
    drug_improved_1 = "Το φάρμακο %s έχει βελτιωθεί από το Τμήμα Ερευνών σας.",
    machine_improved = "Το %s έχει αναβαθμιστεί από το Τμήμα Ερευνών σας.",
    autopsy_discovered_rep_loss = "Βρέθηκε το Μηχάνημα Αυτο-Αυτοψίας σας. Να περιμένετε αρνητικές αντιδράσεις από την κοινή γνώμη.",
    new_available = "Ένα νέο %s έγινε διαθέσιμο.",
    new_machine_researched = "Ένα νέο %s μόλις ανακαλύφθηκε επιτυχώς.",
    drug_fully_researched = "Η έρευνα για το %s είναι στο 100%.",
  },
  room_requirements = {
    research_room_need_researcher = "Πρέπει να προσλάβετε έναν Γιατρό με δεξιότητες Έρευνας για να χρησιμοποιήσετε το δωμάτιο Ερευνών.",
    op_need_another_surgeon = "Χρειάζεται να προσλάβετε έναν ακόμη Χειρουργό για να μπορεί να χρησιμοποιηθεί το Χειρουργείο.",
    gps_office_need_doctor = "Πρέπει να προσλάβετε έναν Γιατρό για να δουλέψει στο Γραφείο Γενικού Ιατρού.",
    pharmacy_need_nurse = "Θα πρέπει να προσλάβετε μία Νοσοκόμα για εκείνο το Φαρμακείο.",
    training_room_need_consultant = "Πρέπει να προσλάβετε έναν Σύμβουλο για να διδάξει σε εκείνο το δωμάτιο Εκπαίδευσης.",
    op_need_ward = "Πρέπει να χτίσετε μια Πτέρυγα για να στεγάζει ασθενείς πριν την διαδικασία της εγχείρησης.",
    op_need_two_surgeons = "Προσλάβετε δύο Χειρουργούς για να κάνουν εγχειρήσεις στο Χειρουργείο.",
    reception_need_receptionist = "Πρέπει να προσλάβετε μία Ρεσεψιονίστ για να δείτε τους ασθενείς σας.",
    psychiatry_need_psychiatrist = "Θα πρέπει να προσλάβετε έναν Ψυχίατρο, τώρα που χτίσατε έναν Ψυχιατρείο.",
    ward_need_nurse = "Θα πρέπει να προσλάβετε μία Νοσοκόμα για να δουλέψει σε αυτήν την Πτέρυγα.",
  },
  cheats = {
    th_cheat = "Συγχαρητήρια, μόλις ξεκλειδώσατε τα cheats!",
    roujin_on_cheat = "Το Roujin's challenge ενεργοποιήθηκε! Καλή τύχη...",
    roujin_off_cheat = "Το Roujin's challenge απενεργοποιήθηκε.",
  },
  tutorial = {
    place_door = "Μετακινείστε τον κέρσορα γύρω από τα τοιχώματα του σχεδίου για να τοποθετήσετε την πόρτα εκεί όπου επιθυμείτε.",
    select_diagnosis_rooms = "Κάντε αριστερό κλικ στο εικονάκι που αναβοσβήνει για να εμφανίσετε την λίστα των διαγνωστικών δωματίων.",
    rotate_and_place_reception = "Κάντε δεξί κλικ για να περιστρέψετε το Γραφείο και τοποθετήστε το στο νοσοκομείο σας με το αριστερό πλήκτρο.",
    next_receptionist = "Αυτή είναι η πρώτη Ρεσεψιονίστ στην λίστα. Κάντε αριστερό κλικ στο εικονάκι που αναβοσβήνει για να δείτε την αμέσως επόμενη διαθέσιμη.",
    hire_receptionist = "Θα χρειαστείτε ακόμα μία Ρεσεψιονίστ για να αντιμετωπίσετε τους ασθενείς σας.",
    select_doctors = "Κάντε αριστερό κλικ στο εικονάκι που αναβοσβήνει για να δείτε όλους τους Γιατρούς που είναι διαθέσιμοι προς πρόσληψη.",
    receptionist_invalid_position = "Δεν μπορείτε να τοποθετήσετε την Ρεσεψιονίστ σας εκεί.",
    room_in_invalid_position = "Αμάν! Αυτό το σχεδιάγραμμα είναι εσφαλμένο - οι κόκκινες περιοχές αναδεικνύουν τα σημεία που υπάρχει επικάλυψη με άλλα δωμάτιά ή τους τοίχους του νοσοκομείου.",
    choose_receptionist = "Αποφασίστε ποια Ρεσεψιονίστ είναι ικανή και ζητάει δίκαιες αποδοχές και έπειτα κάντε αριστερό κλικ στο εικονάκι που αναβοσβήνει για να την προσλάβετε.",
    build_pharmacy = "Συγχαρητήρια! Τώρα χτίστε ένα Φαρμακείο και προσλάβετε μία Νοσοκόμα για να έχετε ένα λειτουργικό νοσοκομείο.",
    information_window = "Το πλαίσιο Βοήθειας σας ενημερώνει για εκείνο το ωραίο Γραφείο Γενικού Ιατρού που μόλις χτίσατε.",
    room_big_enough = "Το σχεδιάγραμμα τώρα είναι επαρκώς μεγάλο. Όταν απελευθερώσετε το χέρι σας από το πλήκτρο του ποντικιού θα το τοποθετήσετε. Εξακολουθείτε να μπορείτε να το μετακινήσετε ή να αλλάξετε το μέγεθός του αν επιθυμείτε.",
    room_too_small_and_invalid = "Το Σχεδιάγραμμα είναι πολύ μικρό και σε λανθασμένη θέση. Κάντε τις απαραίτητες αλλαγές.",
    reception_invalid_position = "Το Γραφείο Ρεσεψιόν είναι γκριζαρισμένο επειδή είναι σε εσφαλμένη θέση. Δοκιμάστε να το μετακινήσετε ή να το περιστρέψετε.",
    build_reception = "Χαίρετε. Αρχικά, το νοσοκομείο σας χρειάζεται ένα Γραφείο Ρεσεψιόν. Διαλέξτε ένα από το μενού Εξοπλισμού Διαδρόμου.",
    order_one_reception = "Κάντε αριστερό κλικ μία φορά στην γραμμή που αναβοσβήνει για να παραγγείλετε ένα Γραφείο Ρεσεψιόν.",
    accept_purchase = "Κάντε αριστερό κλικ στο εικονάκι που αναβοσβήνει για να το αγοράσετε.",
    room_too_small = "Αυτό το σχεδιάγραμμα δωματίου είναι κόκκινο επειδή είναι πολύ μικρό. Τραβήξτε το παραπέρα για να το μεγαλώσετε.",
    place_receptionist = "Μετακινείστε την Ρεσεψιονίστ και τοποθετήστε την οπουδήποτε. Πολύ έξυπνα, θα βρει τον δρόμο για το γραφείο.",
    start_tutorial = "Διαβάστε την Ενημέρωση Αποστολής και κάντε αριστερό κλικ για να ξεκινήσετε το tutorial.",
    place_doctor = "Τοποθετήστε τον Γιατρό οπουδήποτε στο νοσοκομείο. Θα κατευθυνθεί προς το Γραφείο Γενικού Ιατρού όταν θα υπάρχει άτομο προς διάγνωση.",
    click_and_drag_to_build = "Για να χτίσετε ένα Γραφείο Γενικού Ιατρού πρέπει πρώτα να αποφασίσετε πόσο μεγάλο θα είναι. Κάντε αριστερό κλικ και κρατήστε το παρατεταμένα ενώ σέρνετε το δωμάτιο στο επιθυμητό μέγεθος.",
    object_in_invalid_position = "Αυτό το αντικείμενο είναι σε εσφαλμένη θέση. Παρακαλώ είτε τοποθετήστε το κάπου αλλού, ή περιστρέψτε το για να το κάνετε να χωρέσει.",
    confirm_room = "Κάντε αριστερό κλικ στο εικονάκι που αναβοσβήνει για να θέσετε σε ετοιμότητα το δωμάτιό σας ή κάντε κλικ στο εικονάκι με το X για να κάνετε πίσω ένα βήμα.",
    doctor_in_invalid_position = "Έι! Δεν μπορείτε να τοποθετήσετε τον Γιατρό εκεί.",
    build_gps_office = "Για να αρχίσετε να κάνετε διάγνωση των ασθενειών των ασθενών σας πρέπει να χτίσετε ένα Γραφείο Γενικού Ιατρού.",
    door_in_invalid_position = "Ωχ! Επιχειρήσατε να τοποθετήσετε την πόρτα σε εσφαλμένη θέση. Δοκιμάστε κάπου αλλού στα τοιχώματα του σχεδιαγράμματος.",
    place_objects = "Κάντε δεξί κλικ για να περιστρέψετε τα αντικείμενα του δωματίου, και αριστερό κλικ για να τα τοποθετήσετε.",
    place_windows = "Τοποθετήστε παράθυρα με τον ίδιο τρόπο όπως την πόρτα. Δεν χρειάζεστε παράθυρα, αλλά το προσωπικό σας είναι πιο ευτυχισμένο όταν έχει κάπου να δει.",
    select_receptionists = "Κάντε αριστερό κλικ στο εικονάκι που αναβοσβήνει για να δείτε τις Ρεσεψιονίστ που είναι αυτή την στιγμή διαθέσιμες για εργασία. Εκείνος ο αριθμός στο εικονάκι αναδεικνύει μεταξύ πόσων μπορείτε να επιλέξετε.",
    click_gps_office = "Κάντε αριστερό κλικ στην γραμμή που αναβοσβήνει για να διαλέξετε ένα Γραφείο Γενικού Ιατρού.",
    window_in_invalid_position = "Αυτό το παράθυρο είναι σε εσφαλμένη θέση. Δοκιμάστε να το τοποθετήσετε κάπου αλλού στα τοιχώματα, αν θα είχατε την καλοσύνη.",
    prev_receptionist = "Κάντε αριστερό κλικ στο εικονάκι που αναβοσβήνει για να δείτε την προηγούμενη διαθέσιμη Ρεσεψιονίστ.",
    hire_doctor = "Χρειάζεστε έναν Γιατρό για να διαγνώσει και να θεραπεύσει τις νοσούσες μάζες.",
    choose_doctor = "Ρίξτε μια καλή ματιά στις ικανότητες του κάθε Γιατρού προτού επιλέξετε έναν.",
  },
  surgery_requirements = {
    need_surgeons_ward_op = "Πρέπει να προσλάβετε δύο Χειρουργούς και να χτίσετε μία Πτέρυγα καθώς και ένα Χειρουργείο για να μπορείτε να πραγματοποιήσετε μία Εγχείρηση.",
    need_surgeon_ward = "Πρέπει να προσλάβετε ακόμη έναν Χειρουργό και να χτίσετε μία Πτέρυγα για να μπορείτε να πραγματοποιήσετε μία Εγχείρηση.",
  },
  praise = {
    plenty_of_benches = "Υπάρχουν αρκετά καθίσματα, οπότε αυτό δεν αποτελεί πρόβλημα.",
    patients_cured = "%d Ασθενείς έχουν θεραπευτεί.",
    many_benches = "Οι Ασθενείς τώρα έχουν αρκετές θέσεις. Μπράβο.",
    plants_are_well = "Αυτό είναι ωραίο. Φροντίζετε καλά τα φυτά σας. Εξαιρετικά.",
    plants_thriving = "Πολύ καλά. Τα φυτά σας είναι εξαιρετικά. Δείχνουν τέλεια. Συνεχίστε έτσι και μπορεί να κερδίσετε κάποιο βραβείο για αυτά.",
    many_plants = "Εξαιρετικά. Έχετε αρκετά φυτά. Οι Ασθενείς σας θα το εκτιμήσουν αυτό.",
    few_have_to_stand = "Σχεδόν κανένας δεν πρέπει να στέκεται όρθιος στο νοσοκομείο σας. Οι ασθενείς θα ικανοποιηθούν με αυτό.",
  },
  boiler_issue = {
    resolved = "Καλά νέα. Ο λέβητας και τα καλοριφέρ τώρα δουλεύουν σωστά ξανά. Η θερμοκρασία είναι μια χαρά για τους ασθενείς και το προσωπικό σας.",
    minimum_heat = "Α, να 'μαστε. Ο λέβητας του υπογείου χάλασε. Οι άνθρωποι στο νοσοκομείο σας θα την δαγκώσουν σε λίγο.",
    maximum_heat = "Ο λέβητας του υπογείου τρελάθηκε. Η θέρμανση των καλοριφέρ έχει κολλήσει στο μέγιστο. Ο κόσμος θα αρχίσει να λιώνει! Θα βοήθαγε να τοποθετούσατε περισσότερα Αναψυκτήρια.",
  },
    information = {
    promotion_to_doctor = "Ένας από τους ΑΡΧΑΡΙΟΥΣ σας μόλις έγινε ΓΙΑΤΡΟΣ.",
    first_cure = "Συγχαρητήρια! Μόλις θεραπεύσατε τον πρώτο σας ασθενή.",
    emergency = "Υπάρχει μια κρίση! Κουνηθείτε! Κουνηθείτε! ΚΟΥΝΗΘΕΙΤΕ!",
    fax_received = "Το εικονίδιο που μόλις εμφανίστηκε κάτω αριστερά στην οθόνη σας, σας ενημερώνει σχετικά με μία σημαντική πληροφορία ή απόφαση που πρέπει να πάρετε.",
    place_windows = "Τοποθετώντας παράθυρα κάνετε τα δωμάτιο φωτεινότερα και βελτιώνετε το κέφι του προσωπικού σας.",
    extra_items = "Επιπλέον αντικείμενα σε δωμάτια κάνουν το προσωπικό σας πιο άνετο και βελτιώνεται η απόδοσή τους.",
    promotion_to_specialist = "Ένας από τους Γιατρούς σας έχει προαχθεί στην ειδικότητα %s.",
    initial_general_advice = {
      research_now_available = "Έχετε χτίσει το πρώτο σας Δωμάτιο Ερευνών. Τώρα μπορείτε να έχετε πρόσβαση στην οθόνη Ερευνών.",
      autopsy_available = "Το Μηχάνημα Αυτο-Αυτοψίας έχει ανακαλυφθεί. Με αυτό, μπορείτε να ξεφορτώνεστε προβληματικούς ή ανεπιθύμητους ασθενείς, και να κάνετε έρευνα πάνω στα απομεινάρια τους. Προσοχή - η χρήση αυτού του μηχανήματος είναι εξαιρετικά αμφιλεγόμενη.",
      epidemic_spreading = "Υπάρχει μια μεταδοτική ασθένεια στο νοσοκομείο σας. Προσπαθήστε να θεραπεύσετε τους μολυσμένους ανθρώπους, πριν μπορέσουν να φύγουν.",
      taking_your_staff = "Κάποιος λεηλατεί το προσωπικό σας. Θα πρέπει να παλέψετε για να τους κρατήσετε.",
      psychiatric_symbol = "Οι γιατροί που είναι εξειδικευμένοι σε ζητήματα Ψυχιατρικής φέρουν το σύμβολο: |",
      research_symbol = "Οι γιατροί που μπορούν να κάνουν έρευνα φέρουν το σύμβολο: }",
      first_epidemic = "Υπάρχει μια επιδημία στο νοσοκομείο σας! Αποφασίστε εάν θέλετε να το κουκουλώσετε ή να το παραδεχθείτε.",
      surgeon_symbol = "Οι γιατροί που μπορούν να χειρουργήσουν φέρουν το σύμβολο: {",
      machine_needs_repair = "Διαθέτετε μια μηχανή, η οποία χρειάζεται επισκευή. Εντοπίστε τη μηχανή, η οποία θα βγάζει καπνό, και κάντε «κλικ» πάνω της. Στη συνέχεια, κάντε «κλικ» πάνω στο κουμπί του Πολυτεχνίτη.",
      first_VIP = "Είστε έτοιμοι να υποδεχθείτε τον πρώτο σας VIP. Προσπαθήστε να βεβαιωθείτε ότι ο VIP δεν θα δει τίποτε ανθυγιεινό, ή δυστυχισμένους ασθενείς.",
      first_patients_thirsty = "Υπάρχουν διψασμένοι άνθρωποι στο νοσοκομείο σας. Τοποθετήστε αυτόματους πωλητές ροφημάτων γι' αυτούς.",
      first_emergency = "Οι Ασθενείς σε Κατάσταση Εκτάκτου Ανάγκης έχουν ένα παλλόμενο μπλε φως πάνω από τα κεφάλια τους. Θεραπεύστε τους πριν πεθάνουν ή λήξει το χρονικό όριο.",
      rats_have_arrived = "Οι αρουραίοι έχουν μολύνει το νοσοκομείο σας. Προσπαθήστε να τους συντρίψετε με το ποντίκι σας.",
      place_radiators = "Οι Άνθρωποι στο νοσοκομείο σας κρυώνουν - μπορείτε να τοποθετήσετε περισσότερα καλοριφέρ, επιλέγοντάς τα από το Μενού των Αντικειμένων Διαδρόμου.",
      decrease_heating = "Οι Άνθρωποι στο νοσοκομείο σας ζεσταίνονται πολύ. Κατεβάστε το επίπεδο θέρμανσης. Αυτό γίνεται στην Οθόνη Πόλεως.",
      increase_heating = "Οι Άνθρωποι κρυώνουν. Ανεβάστε την θερμοκρασία πηγαίνοντας στην Οθόνη Πόλεως.",
    },
    promotion_to_consultant = "Ένας από τους ΙΑΤΡΟΥΣ σας μόλις έγινε ΣΥΜΒΟΥΛΟΣ.",
    patient_abducted = "Ένας από τους ασθενείς σας απαγάγεται από εξωγήινους.",
    pay_rise = "Ένα μέλος του προσωπικού απειλεί να παραιτηθεί. Επιλέξτε εάν συμφωνείτε με τις απαιτήσεις του ή αν θα τον απολύσετε. Κάντε «κλικ» στο εικονίδιο κάτω και αριστερά, για να δείτε ποιο μέλος του προσωπικού απειλεί να παραιτηθεί.",
    first_death = "Μόλις σκοτώσατε τον πρώτο σας ασθενή. Πώς αισθάνεστε;",
    epidemic_health_inspector = "Η Είδηση της επιδημίας σας έχει φτάσει στο Υπουργείο Υγείας. Ετοιμαστείτε για μια επίσκεψη ενός Επιθεωρητή Υγείας σύντομα.",
    larger_rooms = "Τα μεγαλύτερα δωμάτια κάνουν το προσωπικό να νιώθει πιο σημαντικό και βελτιώνει την επίδοσή τους.",
    vip_arrived = "Προσοχή! Ο %s μόλις έφτασε για να περιηγηθεί στο νοσοκομείο σας! Προσπαθήστε να κυλήσουν όλα ομ, ώστε να τον ευαρεστήσετε.",
    patient_leaving_too_expensive = "Ένας ασθενής φεύγει, δίχως να πληρώσει για  %s. Είναι πανάκριβα.",
    handyman_adjust = "Μπορείτε να κάνετε τους Πολυτεχνίτες να καθαρίσουν πιο αποτελεσματικά, με το να ρυθμίσετε τις προτεραιότητές τους.",
    epidemic = "Υπάρχει μια μεταδοτική ασθένεια στο νοσοκομείο σας. Πρέπει να την αντιμετωπίσετε αμέσως!",
  },
  epidemic = {
    hurry_up = "Αν αυτή η επιδημία δεν αντιμετωπιστεί αμέσως, θα βυθιστείτε στα προβλήματα. Κουνηθείτε!",
    serious_warning = "Αυτή η Μεταδοτική Ασθένεια σοβαρεύει. Πρέπει να κάνετε κάτι σύντομα!",
    multiple_epidemies = "Φαίνεται πως υπάρχει πάνω από μία επιδημία η οποία συμβαίνει ταυτόχρονα. Αυτό μπορεί να μετατραπεί σε μια τεράστια καταστροφή, γι' αυτό κινηθείτε γρήγορα.",
  },
  room_forbidden_non_reachable_parts = "Το να τοποθετήσετε το δωμάτιο σε αυτήν την τοποθεσία θα οδηγούσε σε μη προσπελάσιμα σημεία του νοσοκομείου σας.",
  warnings = {
    place_plants_to_keep_people = "Ο κόσμος φεύγει. Τοποθετήστε φυτά και μπορεί να τους πείσετε να μείνουν.",
    no_desk_3 = "Τέλεια, έχει περάσει σχεδόν ένας χρόνος και ακόμη να προσλάβετε Ρεσεψιονίστ! Πώς περιμένετε να έχετε ασθενείς, τακτοποιήστε το τώρα και σταματήστε τις χαζομάρες!",
    staff_unhappy = "Το προσωπικό σας είναι δυσαρεστημένο. Δοκιμάστε να τους δώσετε ένα μπόνους ή, ακόμα καλύτερα, να χτίσετε ένα Δωμάτιο Προσωπικού. Θα μπορούσατε ακόμα να αλλάξετε την πολιτική ξεκούρασης προσωπικού στην οθόνη Πολιτικής.",
    doctors_tired = "Οι Ιατροί σας είναι πολύ κουρασμένοι. Ξεκουράστε τους.",
    patients_thirsty = "Ο κόσμος αρχίζει να διψάει. Ίσως θα μπορούσατε να τους προμηθεύσετε με αναψυκτήρια.",
    researcher_needs_desk_3 = "Ο κάθε Ερευνητής χρειάζεται το δικό του γραφείο για να εργάζεται.",
    plants_thirsty = "Πρέπει να φροντίσετε τα φυτά σας. Έχουν αρχίσει και διψάνε.",
    patients_annoyed = "Ο κόσμος είναι τρομερά ενοχλημένος από τον τρόπο με τον οποίο «τρέχετε» το νοσοκομείο σας. Και δεν μπορώ να πω ότι τους κατηγορώ. Συγκεντρωθείτε ή αντιμετωπίστε τις συνέπειες!",
    too_many_plants = "Έχετε υπερβολικά πολλά φυτά. Είναι μια ζούγκλα εκεί μέσα.",
    queue_too_long_send_doctor = "Οι ουρές σας για %s είναι πολύ μεγάλες. Σιγουρευτείτε ότι υπάρχει ένας Γιατρός στο δωμάτιο.",
    hospital_is_rubbish = "Ο κόσμος λέει ανοιχτά ότι το νοσοκομείο σας είναι για τα σκουπίδια. Μετά έχει σειρά να πάρουν τις ασθένεις τους... και σ' άλλη παραλία.",
    receptionists_tired2 = "Οι Ρεσεψιονίστ σας είναι πολύ κουρασμένες. Αφήστε τες να ξεκουραστούν αυτή τη στιγμή.",
    no_desk_7 = "Έχετε χτίσει το γραφείο ρεσεψιόν, οπότε τι λέτε να προσλάβετε μια ρεσεψιονίστ; Δεν θα δείτε κανέναν ασθενή μέχρι να το τακτοποιήσετε αυτό, ξέρετε!",
    no_desk_2 = "Συγχαρητήρια, αυτό πρέπει να είναι παγκόσμιο ρεκόρ: κοντά ένας χρόνος και κανένας ασθενής! Αν θέλετε να συνεχίσετε ως Διευθυντής αυτού του νοσοκομείου, θα χρειαστεί να προσλάβετε μία ρεσεψιονίστ και να χτίσετε ένα γραφείο ρεσεψιόν για να μπορεί να εργαστεί!",
    many_epidemics = "Φαίνεται ότι έχετε περισσότερες από μία τρέχουσες επιδημίες. Αυτή θα μπορούσε να είναι μία τεράστια καταστροφή, οπότε κινηθείτε γρήγορα.",
    staff_too_hot = "Το προσωπικό σας υπερθερμαίνεται. Χαμηλώστε την θερμοκρασία ή αφαιρέστε καλοριφέρ από τα δωμάτιά τους.",
    cash_low_consider_loan = "Η κατάσταση με τα οικονομικά σας είναι πολύ άσχημη. Έχετε σκεφτεί να πάρετε κάποιο δάνειο;",
    people_have_to_stand = "Ο κόσμος υποφέρει και πρέπει να στέκεται όρθιος. Τοποθετήστε περισσότερα παγκάκια τώρα.",
    pay_back_loan = "Έχετε αρκετά χρήματα. Τι λέτε να πληρώσετε αυτό το νοίκι;",
    finanical_trouble2 = "Αυξήστε τα έσοδά σας ή ξοφλήσατε. Θα αποτύχετε το επίπεδο αν χάσετε ακόμα %d.",
    staff_overworked = "Το προσωπικό σας είναι απίστευτα εξουθενωμένο. Η απόδοσή τους πέφτει και θα αρχίσουν να κάνουν καταστροφικά λάθη.",
    build_toilets = "Χτίστε μια Τουαλέτα εδώ και τώρα ή θα δείτε κάτι πολύ δυσάρεστο. Και φανταστείτε πως θα μυρίζει το νοσοκομείο σας.",
    researcher_needs_desk_2 = "Ο Ερευνητής σας είναι ικανοποιημένος που του επιτρέψατε να κάνει ένα διάλειμμα. Αν έχετε την πρόθεση να έχετε περισσότερο προσωπικό στον τομέα της έρευνας, τότε πρέπει να παρέχετε στον καθένα το δικό του γραφείο για να εργαστεί.",
    no_desk_1 = "Αν θέλετε να έρθουν ασθενείς στο νοσοκομείο σας, θα χρειαστεί να προσλάβετε μία ρεσεψιονίστ και να της χτίσετε ένα γραφείο!",
    place_plants3 = "Οι ασθενείς αρχίζουν και γίνονται δυσαρεστημένοι. Τοποθετήστε περισσότερα φυτά για να τους ανεβάσετε το ηθικό.",
    patients_too_hot = "Οι ασθενείς έχουν αρχίσει και ζεσταίνονται πολύ. Είτε αφαιρέστε μερικά καλοριφέρ, χαμηλώστε την θερμοκρασία ή τοποθετήστε περισσότερα αναψυκτήρια.",
    falling_3 = "Άουτς, αυτό πρέπει να πόνεσε, κάποιος να καλέσει έναν Γιατρό!",
    place_plants2 = "Ο κόσμος αρχίζει και φεύγει. Αν υπήρχαν περισσότερα φυτά ίσως να έμεναν εδώ κι άλλο.",
    change_priorities_to_plants = "Πρέπει να αλλάξετε τις προτεραιότητες των Πολυτεχνιτών σας έτσι ώστε να περνούν περισσότερο χρόνο ποτίζοντας τα φυτά σας.",
    staff_tired = "Το προσωπικό σας είναι υπερβολικά κουρασμένο. Αν δεν τους αφήσετε λίγο χρόνο να ξεκουραστούν σε ένα Δωμάτιο Προσωπικού, κάποιοι μπορεί να λυγίσουν από την πίεση.",
    machinery_very_damaged = "Έκτακτο! Καλέστε έναν Πολυτεχνίτη να επιδιορθώσει τα μηχανήματά σας αμέσως! Πρόκειται να ανατιναχθούν!",
    research_screen_open_2 = "Η Έρευνα είναι απενεργοποιημένη για το τρέχον επίπεδο.",
    build_toilet_now = "Χτίστε μια τουαλέτα τώρα. Ο κόσμος δεν μπορεί να κρατηθεί άλλο. Μην χαμογελάτε - είναι σοβαρά τα πράγματα.",
    handymen_tired2 = "Οι Πολυτεχνίτες σας έχουν γίνει κομμάτια. Δώστε τους χρόνο για ξεκούραση αμέσως.",
    nurse_needs_desk_1 = "Κάθε Νοσοκόμα πρέπει να έχει το δικό της γραφείο για να εργαστεί.",
    cannot_afford = "Δεν έχετε αρκετά χρήματα για να προσλάβετε αυτό το άτομο!",
    doctors_tired2 = "Οι Γιατροί σας είναι υπερβολικά κουρασμένοι. Χρειάζονται άμεση ξεκούραση.",
    plants_dying = "Τα φυτά σας πεθαίνουν. Είναι απεγνωσμένα για νερό. Βάλτε περισσότερους Πολυτεχνίτες να τα φροντίσουν. Τα νεκρά φυτά δεν αρέσουν στους ασθενείς.",
    nurse_needs_desk_2 = "Η Νοσοκόμα σας είναι ευχαριστημένη που την αφήσατε να κάνει ένα διάλειμμα. Αν θέλατε να εργαζέται παραπάνω από μία σε μια Πτέρυγα, τότε πρέπει να παρέχετε στην καθεμία το δικό της γραφείο.",
    doctor_crazy_overwork = "Ωχ όχι! Ένας από τους Γιατρούς σας τρελάθηκε από την πολλή δουλειά. Μπορεί να αναρρώσει αν τον ξεκουράσετε αμέσως.",
    falling_6 = "Εδώ δεν είναι γήπεδο, δεν πρέπει να φέρεστε έτσι στον άρρωστο κόσμο!",
    many_killed = "Έχετε πλέον σκοτώσει %d ανθρώπους. Πρέπει να τους κάνετε να αισθάνονται καλύτερα, ξέρετε.",
    patient_stuck = "Κάποιος έχει κολλήσει. Σχεδιάστει καλύτερα την δομή του νοσοκομείου σας.",
    need_staffroom = "Χτίστε ένα Δωμάτιο Προσωπικού για να ξεκουράζονται οι υπάλληλοί σας.",
    queue_too_long_at_reception = "Υπάρχουν πάρα πολλοί ασθενείς που περιμένουν για κατανομή δωματίων στη ρεσεψιόν. Τοποθετήστε άλλον έναν Χώρο υποδοχής και προσλάβετε κι άλλη Ρεσεψιονίστ.",
    cannot_afford_2 = "Δεν έχετε αρκετά χρήματα στην τράπεζα, ώστε να πραγματοποιήσετε αυτήν την αγορά!",
    litter_catastrophy = "Η κατάσταση των απορριμμάτων είναι αβυσσαλέα. Βάλτε μια ομάδα Πολυτεχνιτών να εργαστούν πάνω σε αυτό αμέσως!",
    litter_everywhere = "Υπάρχουν απορρίμματα παντού. Βάλτε μερικούς Πολυτεχνίτες ν' ασχοληθούν με το πρόβλημα.",
    high_prices = "Οι αλλαγές σας για το %s είναι υψηλές. Αυτό θα αποφέρει υψηλά κέρδη βραχυπρόθεσμα, ωστόσο εν τέλει θ' αρχίσετε να απωθείτε τους ανθρώπους.",
    patient_not_paying = "Ένας ασθενής έφυγε δίχως να πληρώσει για τα %s επειδή είναι πολύ ακριβά!",
    patients_really_thirsty = "Οι Ασθενείς διψάνε πάρα πολύ. Τοποθετήστε περισσότερους αυτόματους πωλητές ποτών, ή μετακινήστε ήδη υπάρχουντες κοντά στις μεγαλύτερες ουρές.",
    reception_bottleneck = "Υπάρχει μποτιλιάρισμα στη Ρεσεψιόν. Προσλάβετε κι άλλη Ρεσεψιονίστ.",
    financial_trouble3 = "Ο τραπεζικός σας ισολογισμός μοιάζει κάπως ανησυχητικός. Σκεφτείτε να βγάλετε περισσότερα χρήματα. Βρίσκεστε %d μακριά από την καταστροφή.",
    machine_severely_damaged = "Η %s είναι πολύ κοντά στο να της προκληθεί ανεπανόρθωτη ζημιά.",
    charges_too_high = "Οι τιμές σας είναι υψηλές. Αυτό θα αποφέρει υψηλά κέρδη βραχυπρόθεσμα, ωστόσο εν τέλει θ' αρχίσετε να απωθείτε τους ανθρώπους.",
    reduce_staff_rest_threshold = "Προσπαθήστε να μειώσετε το Όριο Άδειας Προσωπικού στην οθόνη Πολιτικής, ώστε να ξεκουράζονται πιο συχνά. Απλώς μια ιδέα>",
    nurses_tired2 = "Οι Νοσοκόμες σας είναι πολύ κουρασμένες. Ξεκουράστε τες αμέσως.",
    no_desk_5 = "Λοιπόν, έφτασε σχεδόν η ώρα, σιγά-σιγά θα δείτε ασθενείς να φτάνουν σύντομα!",
    machinery_damaged = "Επισκευάστε τις μηχανές σας σύντομα. Δεν θ' αργήσουν ν' αρχίσουν να διαλύονται.",
    no_desk_6 = "Έχετε μία ρεσεψιονίστ, οπότε τι θα λέγατε να κατασκευάσετε έναν χώρο υποδοχής, ώστε να εργάζεται από εκεί;",
    patients_very_cold = "Οι ασθενείς κρυώνουν πολύ. Προσπαθήστε να ανεβάσετε τη θέρμανση ή να τοποθετήσετε περισσότερα καλοριφέρ στο νοσοκομείο σας.",
    falling_4 = "Εδώ είναι Νοσοκομείο, όχι Θεματικό Πάρκο!",
    patient_leaving = "Ένας Ασθενής φεύγει. Ο λόγος; Το νοσοκομείο σας έχει κακή διαχείριση και κακή στελέχωση.",
    staff_unhappy2 = "Το προσωπικό σας είναι γενικά ανικανοποίητο. Σύντομα όλοι θα θέλουν περισσότερα χρήματα.",
    researcher_needs_desk_1 = "Ένας Ερευνητής χρειάζεται ένα γραφείο για να εργάζεται.",
    patients_thirsty2 = "Οι άνθρωποι παραπονιούνται ότι διψούν. Καλό θα ήταν να τοποθετήσετε μερικούς ακόμη Αυτόματους Πωλητές Ποτών ή να μετακινήσετε ήδη υπάρχοντες πιο κοντά σε αυτούς.",
    research_screen_open_1 = "Πρέπει να χτίσετε ένα Τμήμα Ερευνών πριν αποκτήσετε πρόσβαση στην Οθόνη Έρευνας.",
    falling_1 = "Έι! Αυτό δεν είναι αστείο, πρόσεχετε πού κάνετε «κλικ»: μπορεί να χτυπήσει κανείς!",
    fair_prices = "Οι τιμές σας για τα %s φαίνονται λογικές και ισορροπημένες.",
    money_very_low_take_loan = "Ο Τραπεζικός σας Ισολογισμός είναι τρομερά χαμηλός. Ξέρετε, μπορείτε να πάρετε ένα δάνειο.",
    financial_trouble = "Έχετε σοβαρά οικονομικά προβλήματα. Τακτοποιήστε τα οικονομικά σας αυτήν τη στιγμή! Αν χάσετε άλλα %d, θα το χάσετε αυτό το επίπεδο!",
    more_toilets = "Χρειάζεστε περισσότερες τουαλέτες. Ο κόσμος κοντεύει να τα κάνει επάνω του.",
    no_desk = "Θα πρέπει να χτίσετε ένα γραφείο ρεσεψιόν και να προσλάβετε μια ρεσεψιονίστ κάποια στιγμή!",
    receptionists_tired = "Οι Ρεσεψιονίστ σας είναι πάρα πολύ κουρασμένες. Ξεκουράστε τες αυτήν τη στιγμή.",
    no_desk_4 = "Μια Ρεσεψιονίστ πρέπει να έχει τον δικό της χώρο εργασίας για να καλωσορίζει τους ασθενείς καθώς φτάνουν",
    patients_leaving = "Οι ασθενείς φεύγουν. Βελτιώστε το νοσοκομείο για τους επισκέπτες σας τοποθετώντας φυτά, παγκάκια, αναψυκτήρια και ούτω καθεξής.",
    build_staffroom = "Χτίστε ένα Δωμάτιο Προσωπικού αμέσως. Το προσωπικό σας εργάζεται πολύ σκληρά και είναι στα πρόθυρα κατάρρευσης. Ελάτε στα λογικά σας!",
    people_did_it_on_the_floor = "Κάποιοι από τους ασθενείς σας δεν μπόρεσαν να κρατηθούν. Το καθάρισμα θα είναι μεγάλη δουλειά για κάποιον.",
    queues_too_long = "Οι ουρές σας είναι πολύ μεγάλες.",
    patients_unhappy = "Στους ασθενείς δεν αρέσει το νοσοκομείο σας. Πρέπει να κάνετε κάτι για να βελτιώσετε το περιβάλλον τους.",
    need_toilets = "Οι ασθενείς χρειάζονται τουαλέτες. Χτίστε τες σε ένα εύκολα προσβάσιμο μέρος.",
    falling_2 = "Τι θα λέγατε να σταματήσετε να χαζολογάτε;",
    people_freezing = "Απίστευτο, στην εποχή της κεντρικής θέρμανσης, κάποιοι από τους ασθενείς σας έχουν γίνει παγάκια. Χτίστε καλοριφέρ για να τους ζεστάνετε και αυξήστε την θερμοκρασία.",
    patients_very_thirsty = "Οι ασθενείς υποφέρουν πραγματικά από την δίψα. Αν δεν βάλετε Αναψυκτήρια σύντομα, θα έρθετε αντιμέτωποι με μια μεγάλη αποχώρηση καθώς όλος ο κόσμος θα πάει σπίτι να πιει μια κόλα.",
    charges_too_low = "Χρεώνετε πολύ λίγα. Αυτό θα φέρει κόσμο στο νοσοκομείο σας, αλλά δεν θα βγάλετε πολλά κέρδη από τον καθένα τους.",
    handymen_tired = "Οι Πολυτεχνίτες σας είναι πολύ κουρασμένοι. Ξεκουράστε τους τώρα.",
    money_low = "Τα χρήματά σας λιγοστεύουν!",
    more_benches = "Σκεφτείτε να τοποθετήσετε περισσότερα παγκάκια. Στον άρρωστο κόσμο δεν αρέσει να στέκεται όρθιος.",
    machines_falling_apart = "Τα μηχανήματά σας διαλύονται. Οδηγήστε Πολυτεχνίτες σε αυτά τώρα!",
    machinery_deteriorating = "Τα μηχανήματά σας έχουν αρχίσει να φθείρονται λόγω της υπερχρησίας. Ρίχτε τους μια ματιά.",
    place_plants4 = "Βελτιώστε την διάθεση των Ασθενών τοποθετώντας περισσότερα φυτά τριγύρω.",
    nurses_tired = "Οι Νοσοκόμες σας είναι κουρασμένες. Ξεκουράστε τις τώρα.",
    no_patients_last_month = "Τον τελευταίο μήνα δεν είχατε κανέναν νέο ασθενή στο νοσοκομείο σας. Σοκαριστικό!",
    staff_very_cold = "Το προσωπικό παραπονιέται για το κρύο. Αυξήστε την θερμοκρασία ή τοποθετήστε περισσότερα καλοριφέρ.",
    machinery_slightly_damaged = "Ο εξοπλισμός του νοσοκομείου σας είναι ελαφρώς φθαρμένος. Μην ξεχάσετε να πραγματοποιήσετε εργασίες συντήρησης κάποια στιγμή.",
    falling_5 = "Αυτό δεν είναι μέρος για να σπρώχνετε ανθρώπους, είναι άρρωστοι ξέρετε!",
    low_prices = "Χρεώνετε πολύ λίγα για %s. Αυτό θα φέρει κόσμο στο νοσοκομείο σας, αλλά δεν θα βγάλετε πολύ κέρδος από τον καθένα τους.",
    nobody_cured_last_month = "Κυριολεκτικά κανείς δεν θεραπεύτηκε τον προηγούμενο μήνα.",
    patients_getting_hot = "Οι ασθενείς αρχίζουν και ζεσταίνονται πολύ. Δοκιμάστε να χαμηλώσετε την θερμοκρασία ή ακόμα και να αφαιρέσετε μερικά καλοριφέρ.",
    some_litter = "Οι Πολυτεχνίτες μπορούν να ξεφορτωθούν αυτά τα σκουπιδιά πριν αρχίσουν να αποτελούν σοβαρά πρόβλημα.",
    bankruptcy_imminent = "Έι! Οδεύετε προς την χρεοκοπία. Προσοχή!",
    too_much_litter = "Υπάρχει θέμα με τα σκουπίδια. Περισσότεροι Πολυτεχνίτες μπορεί να δώσουν λύση.",
    machinery_damaged2 = "Θα πρέπει σύντομα να προσλάβετε έναν Πολυτεχνίτη να επιδιορθώσει τα μηχανήματά σας.",
    epidemic_getting_serious = "Εκείνη η Μεταδοτική Ασθένεια αρχίζει και γίνεται σοβαρή. Πρέπει να κάνετε κάτι σύντομα!",
    deal_with_epidemic_now = "Αν αυτή η επιδημία δεν αντιμετωπιστεί τώρα, θα είστε βουτηγμένοι μέχρι το κεφάλι στα προβλήματα. Κουνηθείτε!",
    desperate_need_for_watering = "Χρειάζεστε απεγνωσμένα να προσλάβετε έναν Πολυτεχνίτη για να φροντίσει τα φυτά σας.",
  },
  goals = {
    win = {
      value = "Πρέπει να αυξήσετε την αξία του νοσοκομείου σας σε %d",
      reputation = "Αυξήστε την φήμη σας σε %d για να ικανοποιήσετε τα κριτήρια νίκης του επιπέδου",
      money = "Χρειάζεστε ακόμα %d για να εκπληρώσετε τα οικονομικά κριτήρια σε αυτό το επίπεδο.",
      cure = "Θεραπεύστε ακόμα %d ασθενείς και θα έχετε θεραπεύσει αρκετούς για να κερδίσετε αυτό το επίπεδο.",
    },
    lose = {
      kill = "Σκοτώστε ακόμα %d ασθενείς για να χάσετε το επίπεδο!",
    },
  },
  competitors = {
    land_purchased = "Ο ανταγωνιστής %s μόλις αγόρασε μερική γη.",
    hospital_opened = "Ένα ανταγωνιστικό νοσοκομείο μόλις άνοιξε στην περιοχή από %s.",
    staff_poached = "Ένα αντίπαλο Νοσοκομείο έκλεψε ένα μέλος από το προσωπικό σας.",
  },
  staff_place_advice = {
    only_researchers = "Οι Γιατροί μπορούν να εργαστούν σε ένα Τμήμα Ερευνών μόνο αν έχουν δεξιότητες Έρευνας.",
    only_surgeons = "Οι Γιατροί μπορούν να εργαστούν σε ένα Χειρουργείο μόνο αν έχουν δεξιότητες Εγχείρησης.",
    only_nurses_in_room = "Μόνο Νοσοκόμες μπορούν να εργαστούν στο %s",
    receptionists_only_at_desk = "Οι Ρεσεψιονίστ μπορούν να εργαστούν μόνο σε γραφείο Ρεσεψιόν.",
    doctors_cannot_work_in_room = "Οι Γιατροί δεν μπορούν να εργαστούν στο %s",
    nurses_cannot_work_in_room = "Οι Νοσοκόμες δεν μπορούν να εργαστούν στο %s",
    only_doctors_in_room = "Μόνο οι Γιατροί μπορούν να εργαστούν στο %s",
    only_psychiatrists = "Οι Γιατροί μπορούν να εργαστούν σε ένα Ψυχιατρείο μόνο αν έχουν Ψυχιατρικές δεξιότητες.",
  },
  build_advice = {
    door_not_reachable = "Ο κόσμος δεν θα μπορούσε να έχει πρόσβαση αυτήν την πόρτα. Σκεφτείτε το.",
    blueprint_would_block = "Αυτό το σχεδιάγραμμα θα μπλόκαρε άλλα δωμάτια. Δοκιμάστε να αλλάξετε το μέγεθός του, ή να το μετακινήσετε κάπου αλλού!",
    blueprint_invalid = "Αυτό το σχεδιάγραμμα δεν είναι έγκυρο.",
    placing_object_blocks_door = "Αν τοποθετήσετε αυτό το αντικείμενο εκεί θα αποτρέψετε τον κόσμο από το να έχει πρόσβαση σε μια πόρτα.",
  },
  placement_info = {
    door_cannot_place = "Συγγνώμη αλλά δεν μπορείτε να τοποθετήσετε αυτήν την πόρτα εδώ.",
    room_cannot_place_2 = "Δεν μπορείτε να τοποθετήσετε αυτό το δωμάτιο εδώ.",
    staff_cannot_place = "Δεν μπορείτε να τοποθετήσετε αυτό το προσωπικό εδώ. Συγγνώμη.",
    object_can_place = "Μπορείτε να τοποθετήσετε αυτό το αντικείμενο εδώ.",
    reception_cannot_place = "Δεν μπορείτε να τοποθετήσετε αυτό το γραφείο Ρεσεψιόν εδώ.",
    window_can_place = "Μπορείτε να τοποθετήσετε αυτό το παράθυρο εδώ. Είναι εντάξει.",
    window_cannot_place = "Α. Δεν μπορείτε να τοποθετήσετε αυτό το παράθυρο εδώ.",
    object_cannot_place = "Έι, δεν μπορείτε να τοποθετήσετε αυτό το αντικείμενο εδώ.",
    door_can_place = "Μπορείτε να τοποθετήσετε αυτήν την πόρτα εδώ αν θέλετε.",
    reception_can_place = "Μπορείτε να τοποθετήσετε αυτό το γραφείο Ρεσεψιόν εδώ.",
    staff_can_place = "Μπορείτε να τοποθετήσετε αυτό το προσωπικό εδώ. ",
    room_cannot_place = "Δεν μπορείτε να τοποθετήσετε αυτό το δωμάτιο εδώ.",
  },
  earthquake = {
    alert = "Προειδοποίηση σεισμού. Κατά την διάρκεια ενός σεισμού τα μηχανήματά σας θα υποστούν ζημιές. Αν δεν είναι καλά συντηρημμένα μπορεί να καταστραφούν.",
    damage = "Αυτός ο σεισμός έκανε ζημιές σε %d μηχανήματα και %d ανθρώπους στο νοσοκομείο.",
    ended = "Ουφ. Νόμιζα ότι ήταν ο «Big One» - ήταν %d στην Κλίμακα Ρίχτερ.",
  },
  level_progress = {
    another_patient_cured = "Συγχαρητήρια - ακόμη ένας θεραπευμένος ασθενής. Ανέρχονται σε %d.",
    close_to_win_increase_value = "Πλησιάζετε κοντά στην νίκη. Αυξήστε την αξία του νοσοκομείου σας κατά %d.",
    three_quarters_won = "Έχετε διανύσει τα τρία τέταρτα του δρόμου προς την νίκη αυτού του επιπέδου.",
    three_quarters_lost = "Έχετε διανύσει τα τρία τέταρτα του δρόμου προς την αποτυχία αυτού του επιπέδου.",
    another_patient_killed = "Ωχ όχι! Έχετε σκοτώσει ακόμη έναν ασθενή. Οι θάνατοι τώρα ανέρχονται σε %d.",
    halfway_lost = "Είστε περίπου στα μισά προς την αποτυχία αυτού του επιπέδου.",
    improve_reputation = "Πρέπει να αυξήσετε την φήμη σας κατά %d για να έχετε πιθανότητα να κερδίσετε αυτό το επίπεδο.",
    halfway_won = "Είστε περίπου στα μισά για την νίκη αυτού του επιπέδου.",
    cured_enough_patients = "Έχετε θεραπεύσει αρκετούς ασθενείς άλλα πρέπει να βελτιώσετε την λειτουργία του νοσοκομείου σας για να κερδίσετε αυτό το επίπεδο.",
    hospital_value_enough = "Κρατήστε την αξία του νοσοκομείου σας πάνω από %d και τακτοποιήστε τα υπόλοιπα προβλήματά σας για να κερδίσετε το επίπεδο.",
    reputation_good_enough = "Εντάξει, η φήμη σας είναι αρκετά καλή για να κερδίσετε αυτό το επίπεδο. Κρατήστε την πάνω από %d και τακτοποιήστε οποιαδήποτε άλλα προβλήματα για να το ολοκληρώσετε.",
    financial_criteria_met = "Έχετε ικανοποιήσει τα οικονομικά κριτήρια για αυτό το επίπεδο. Τώρα κρατήστε τον προϋπολογισμό σας πάνω από %d, ενώ σιγουρεύετε ότι το νοσοκομείο σας λειτουργεί αποτελεσματικά",
    dont_kill_more_patients = "Πραγματικά δεν έχετε το περιθώριο να πεθάνουν κι άλλοι ασθενείς!",
    nearly_won = "Είστε πολύ κοντά στο να κερδίσετε αυτό το επίπεδο.",
    nearly_lost = "Είστε πολύ κοντά στο να χάσετε αυτό το επίπεδο.",
  },
  staff_advice = {
    need_doctors = "Χρειάζεστε περισσότερους Γιατρούς. Δοκιμάστε να τοποθετήσετε τους καλύτερους Γιατρούς σας στα δωμάτια με τις μεγαλύτερες ουρές.",
    need_handyman_plants = "Θα χρειαστεί να προσλάβετε έναν Πολυτεχνίτη για να ποτίσει τα φυτά.",
    need_handyman_litter = "Ο κόσμος έχει αρχίσει να ρυπαίνει με σκουπίδια το νοσοκομείο σας. Προσλάβετε έναν Πολυτεχνίτη για να τα καθαρίσει.",
    too_many_doctors = "Έχετε πάρα πολλούς Γιατρούς. Κάποιοι από αυτούς δεν έχουν να απασχοληθούν με κάτι.",
    too_many_nurses = "Νομίζω ότι αυτήν την στιγμή έχετε πάρα πολλές Νοσοκόμες.",
    need_handyman_machines = "Θα χρειαστεί να προσλάβετε Πολυτεχνίτες αν θέλετε να συντηρήσετε τα μηχανήματα στο νοσοκομείο σας.",
    need_nurses = "Θα χρειαστεί να προσλάβετε περισσότερες Νοσοκόμες. Οι Πτέρυγες και τα Φαρμακεία χρειάζονται μια Νοσοκόμα.",
  },
  multiplayer = {
    players_failed = "Οι ακόλουθοι παίκτες απέτυχαν να ολοκληρώσουν τον τελευταίο στόχο : ",
    objective_completed = "Έχετε ολοκληρώσει τον στόχο. Συγχαρητήρια!",
    everyone_failed = "Κανείς δεν κατάφερε να ολοκληρώσει εκείνο τον τελευταίο στόχο. Οπότε όλοι συνεχίζουν το παιχνίδι !",
    objective_failed = "Αποτύχατε να ολοκληρώσετε τον στόχο.",
    poaching = {
      in_progress = "Θα σας ενημερώσω σχετικά με το αν αυτό το άτομο επιθυμεί να έρθει να εργαστεί για εσάς.",
      already_poached_by_someone = "Αποκλείεται! Κάποιος ήδη προσπαθεί να βουτήξει αυτό το άτομο.",
      not_interested = "Χα! Δεν ενδιαφέρονται να εργαστούν για εσάς - είναι χαρούμενοι εκεί που είναι.",
    },
  },
  vomit_wave = {
    started = "Φαίνεται πως έχετε έναν ιό εμετού στο νοσοκομείο σας, αν το είχατε κρατήσει καθαρότερο αυτό δεν θα είχε συμβεί. Ίσως πρέπει να προσλάβετε περισσότερους Πολυτεχνίτες.",
    ended = "Ουφ! Φαίνεται πως ο ιός που προκάλεσε εκείνον τον εμετό έχει λίγο-πολύ εξασθενήσει. Κρατήστε το νοσοκομείο σας καθαρό μελλοντικά.",
  },
}
menu_options_volume = {
  [80] = "  80%  ",
  [50] = "  50%  ",
  [100] = "  100%  ",
  [70] = "  70%  ",
  [40] = "  40%  ",
  [90] = "  90%  ",
  [60] = "  60%  ",
  [20] = "  20%  ",
  [30] = "  30%  ",
  [10] = "  10%  ",
}
newspaper = {
  [1] = {
    [1] = "ΓΙΑΤΡΟΣ ΔΗΜΙΟΥΡΓΕΙ ΚΟΙΝΩΝΙΚΗ ΚΑΤΑΚΡΑΥΓΗ",
    [2] = "ΤΡΕΛΟΣ ΓΙΑΤΡΟΣ ΠΑΡΙΣΤΑΝΕΙ ΤΟ ΘΕΟ",
    [3] = "ΣΟΚ ΦΑΡΣΕΡ-ΣΤΑΪΝ",
    [4] = "ΖΩΝΤΑΝΟ ΝΕΚΡΟΤΟΜΕΙΟ",
    [5] = "ΕΠΙΔΡΟΜΗ ΒΑΖΕΙ ΤΕΛΟΣ ΣΕ ΕΠΙΚΙΝΔΥΝΗ ΕΡΕΥΝΑ",
  },
  [2] = {
    [1] = "ΜΕΘ-ΥΣΜΕΝΟΣ ΓΙΑΤΡΟΣ",
    [2] = "ΧΕΙΡΟΥΡΓΟΣ ΝΤΕΦΙ",
    [3] = "ΞΕΦΑΝΤΩΜΕΝΟΣ ΣΥΜΒΟΥΛΟΣ",
    [4] = "ΧΕΙΡΟΥΡΓΙΚΟ ΣΦΗΝΑΚΙ",
    [5] = "ΧΕΙΡΟΥΡΓΟΣ ΝΕΡΟΧΥΤΗΣ",
    [6] = "ΧΕΙΡΟΥΡΓΙΚΑ ΠΝΕΥΜΑΤΑ",
  },
  [3] = {
    [1] = "ΧΕΙΡΟΥΡΓΟΣ ΠΟΥ ΣΚΙΖΕΙ",
    [2] = "ΔΟΚΤΩΡ ΤΣΙΤΣΙΔΗΣ",
    [3] = "ΔΟΚΤΩΡ ΚΑΤΡΑΚΥΛΑΕΙ",
    [4] = "ΑΧΟΡΤΑΓΟΣ ΧΕΙΡΟΥΡΓΟΣ",
  },
  [4] = {
    [1] = "ΓΙΑΤΡΕΥΟΝΤΑΣ ΤΑ ΝΟΥΜΕΡΑ",
    [2] = "ΟΡΓΑΝΟ-ΜΕΝΟ ΕΓΚΛΗΜΑ",
    [3] = "ΕΓΧΕΙΡΗΣΗ BYPASS ΤΡΑΠΕΖΗΣ",
    [4] = "ΓΙΑΤΡΟΣ ΚΑΤΟΧΟΣ FUND",
  },
  [5] = {
    [1] = "ΓΙΑΤΡΟΣ ΚΑΝΕΙ ΕΠΙΔΡΟΜΗ ΣΕ ΦΕΡΕΤΡΟ",
    [2] = "ΓΙΑΤΡΟΣ ΑΔΕΙΑΖΕΙ ΤΑΦΟΥΣ",
    [3] = "ΠΙΑΣΤΗΚΕ ΜΕ ΠΤΩΜΑ",
    [4] = "ΗΜΕΡΑ ΤΗΣ ΚΡΙΣΗΣ ΤΟΥ ΔΡ. ΘΑΝΑΤΟΥ",
    [5] = "ΤΕΡΜΑΤΙΚΗ ΑΜΕΛΕΙΑ",
    [6] = "ΣΚΑΨΙΜΟ ΓΙΑΤΡΟΥ ΚΑΤΑΔΙΚΑΖΕΤΑΙ",
  },
  [6] = {
    [1] = "ΓΙΑΤΡΟΣ ΜΕ ΡΑΜΜΑΤΑ!",
    [2] = "SLACK QUACK",
    [3] = "ΕΝΟΧΟΠΟΙΗΤΙΚΗ ΔΙΑΓΝΩΣΗ",
    [4] = "ΑΤΣΟΥΜΠΑΛΟΣ ΣΥΜΒΟΥΛΟΣ",
  },
  [7] = {
    [1] = "ΓΙΑΤΡΟΣ ΔΕΝ ΕΧΕΙ ΤΙΠΟΤΑ ΝΑ ΚΑΝΕΙ",
    [2] = "ΧΕΙΡΟΥΡΓΟΣ 'ΕΓΧΕΙΡΙΖΕΙ' ΤΟΝ ΕΑΥΤΟ ΤΟΥ",
    [3] = "BOG BISHOP-BASHING",
    [4] = "DOCTOR'S HANDLE SCANDAL",
    [5] = "ΓΙΑΤΡΟΣ ΤΑ ΚΑΝΕΙ ΑΝΩ ΚΑΤΩ",
  },
}
menu = {
  player_count = "ΑΡΙΘΜΟΣ ΠΑΙΚΤΩΝ",
  options = "  ΕΠΙΛΟΓΕΣ  ",
  file = "  ΑΡΧΕΙΟ  ",
  debug = "  DEBUG  ",
  display = "  ΕΜΦΑΝΙΣΗ  ",
  charts = "  ΓΡΑΦΗΜΑΤΑ  ",
}
misc = {
  no_heliport = "Είτε δεν έχουν ανακαλυφθεί ακόμη ασθένειες, είτε δεν υπάρχει ελικοδρόμιο σε αυτόν τον χάρτη. Μπορεί να χρειάζεται να χτίσετε έναν χώρο υποδοχής και να προσλάβετε μια ρεσεψιονίστ.",
  not_yet_implemented = "(δεν έχει υλοποιηθεί ακόμη)",
  save_failed = "ΣΦΑΛΜΑ: Το παιχνίδι δεν ήταν δυνατόν να αποθηκευτεί",
  done = "Όλα έτοιμα",
  low_res = "Χαμηλή ανάλυση",
  load_failed = "Το αποθηκευμένο παιχνίδι δεν φορτώθηκε",
  save_success = "Το παιχνίδι αποθηκεύτηκε",
  grade_adverb = {
    moderately = "μέτρια",
    extremely = "ακραία",
    mildly = "ήπια",
  },
  send_message_all = "Στείλτε Μήνυμα σε Όλους τους Παίκτες",
  mouse = "Ποντίκι",
  hospital_open = "Νοσοκομείο Ανοιχτό",
  out_of_sync = "Το παιχνίδι βρίσκεται εκτός συγχρονισμού",
  pause = "Παύση",
  balance = "Αρχείο Ισορροπίας:",
  force = "Δύναμη",
  send_message = "Στείλτε Μήνυμα στον Παίκτη %d",
}
menu_debug = {
  sprite_viewer = "  SPRITE VIEWER  ",
  display_pager = "  DISPLAY PAGER        ",
  remove_walls = "  REMOVE WALLS         ",
  connect_debugger = "  (%1%) CONNECT LUA DBGp SERVER  ",
  lose_game_anim = {
    [1] = "  LOSE GAME 1 ANIM     ",
    [2] = "  LOSE GAME 2 ANIM     ",
    [3] = "  LOSE GAME 3 ANIM     ",
    [4] = "  LOSE GAME 4 ANIM     ",
    [5] = "  LOSE GAME 5 ANIM     ",
    [6] = "  LOSE GAME 6 ANIM     ",
    [7] = "  LOSE GAME 7 ANIM     ",
  },
  make_debug_fax = "  MAKE DEBUG FAX  ",
  keep_clear_cells = "  KEEP CLEAR CELLS     ",
  make_debug_patient = "  MAKE DEBUG PATIENT  ",
  map_overlay = "  MAP OVERLAY  ",
  entry_cells = "  ENTRY CELLS          ",
  debug_script = "  (%1%) RUN DEBUG SCRIPT  ",
  show_help_hotspot = "  SHOW HELP HOTSPOTS   ",
  enter_nav_debug = "  ENTER NAV DEBUG      ",
  porter_pagers = "  PORTER PAGERS        ",
  limit_camera = "  LIMIT CAMERA  ",
  cheats = "  (%1%) CHEATS  ",
  jump_to_level = "  JUMP TO LEVEL  ",
  machine_pagers = "  MACHINE PAGERS       ",
  pixbuf_cells = "  PIXBUF CELLS         ",
  dump_strings = "  (%1%) DUMP STRINGS  ",
  mapwho_checking = "  MAPWHO CHECKING      ",
  nav_bits = "  NAV BITS             ",
  win_game_anim = "  WIN GAME ANIM        ",
  disable_salary_raise = "  DISABLE SALARY RAISE  ",
  dump_gamelog = "  (%1%) DUMP GAME LOG  ",
  lua_console = "  (%1%) LUA CONSOLE  ",
  transparent_walls = "  (%1%) TRANSPARENT WALLS  ",
  display_room_status = "  DISPLAY ROOM STATUS  ",
  calls_dispatcher = "  CALLS DISPATCHER  ",
  show_nav_cells = "  SHOW NAV CELLS       ",
  remove_objects = "  REMOVE OBJECTS       ",
  win_level_anim = "  WIN LEVEL ANIM       ",
  object_cells = "  OBJECT CELLS         ",
  display_big_cells = "  DISPLAY BIG CELLS    ",
  plant_pagers = "  PLANT PAGERS         ",
}
customise_window = {
  average_contents = "Μέσα περιεχόμενα",
  caption = "Προσαρμοσμένες Ρυθμίσεις",
  fractured_bones = "Σπασμένα Κόκαλα",
  intro = "Αναπαραγωγή Βίντεο Εισαγωγής",
  back = "Πίσω",
  option_on = "On",
  aliens = "Εξωγήινοι Ασθενείς",
  option_off = "Off",
  paused = "Χτίσιμο κατά την Παύση",
  movies = "Global Movie Control",
  volume = "Volume down hotkey",
}
rooms_long = {
  research_room = "Τμήμα Ερευνών",
  emergency = "Έκτακτη Ανάγκη",
  blood_machine = "Δωμάτιο Μηχανήματος Αίματος",
  scanner = "Δωμάτιο Σαρωτή",
  inflation = "Δωμάτιο Ξεφουσκώματος",
  x_ray = "Δωμάτιο Ακτινογραφίας",
  operating_theatre = "Χειρουργείο",
  staffroom = "Δωμάτιο Προσωπικού",
  jelly_vat = "Δεξαμενή Ζελέ",
  psychiatric = "Ψυχιατρείο",
  pharmacy = "Φαρμακείο",
  toilets = "Τουαλέτα",
  cardiogram = "Καρδιογράφημα",
  general = "Γενική",
  ward = "Πτέρυγα",
  dna_fixer = "Επιδιορθωτής DNA",
  ultrascan = "Δωμάτιο Ultrascan",
  tongue_clinic = "Κλινική Χαλαρής Γλώσσας",
  general_diag = "Γενική Διάγνωση",
  fracture_clinic = "Κλινική Καταγμάτων",
  corridors = "Διάδρομοι",
  training_room = "Δωμάτιο Εκπαίδευσης",
  gps_office = "Γραφείο Γενικού Ιατρού",
  decontamination = "Απολύμανση",
  hair_restoration = "Αποκατάσταση Μαλλιών",
  electrolysis = "Κλινική Ηλεκτρόλυσης",
}
menu_options_warmth_colors = {
  choice_1 = "  ΚΟΚΚΙΝΟ  ",
  choice_2 = "  ΜΠΛΕ ΠΡΑΣΙΝΟ ΚΟΚΚΙΝΟ  ",
  choice_3 = "  ΚΙΤΡΙΝΟ ΠΟΡΤΟΚΑΛΙ ΚΟΚΚΙΝΟ  ",
}
map_editor_window = {
  pages = {
    heliport_4 = "Ελικοδρόμιο 4",
    parcel_4 = "Οικόπεδο 4",
    delete_wall = "Διαγραφή τοίχων",
    hedgerow = "Θαμνοσειρά",
    inside = "Εσωτερικό",
    pond = "Λιμνούλα",
    road = "Δρόμος",
    foliage = "Βλάστηση",
    heliport_3 = "Ελικοδρόμιο 3",
    helipad = "Ελικοδρόμιο",
    camera_2 = "Κάμερα 2",
    camera_1 = "Κάμερα 1",
    paste = "Περιοχή επικόλλησης",
    north_wall = "Βόρειος τοίχος",
    heliport_1 = "Ελικοδρόμιο 1",
    parcel_8 = "Οικόπεδο 8",
    west_wall = "Δυτικός τοίχος",
    parcel_0 = "Οικόπεδο 0",
    parcel_1 = "Οικόπεδο 1",
    parcel_2 = "Οικόπεδο 2",
    camera_3 = "Κάμερα 3",
    parcel_5 = "Οικόπεδο 5",
    parcel_7 = "Οικόπεδο 7",
    heliport_2 = "Ελικοδρόμιο 2",
    camera_4 = "Κάμερα 4",
    parcel_9 = "Οικόπεδο 9",
    parcel_3 = "Οικόπεδο 3",
    outside = "Έξω",
    parcel_6 = "Οικόπεδο 6",
  },
}
menu_player_count = {
  players_4 = "  4 ΠΑΙΚΤΕΣ  ",
  players_1 = "  1 ΠΑΙΚΤΗΣ  ",
  players_2 = "  2 ΠΑΙΚΤΕΣ  ",
  players_3 = "  3 ΠΑΙΚΤΕΣ  ",
}
competitor_names = {
  [1] = "ORAC",
  [2] = "COLOSSUS",
  [3] = "HAL",
  [4] = "MULTIVAC",
  [5] = "HOLLY",
  [6] = "DEEP THOUGHT",
  [7] = "ZEN",
  [8] = "LEON",
  [9] = "MARVIN",
  [10] = "AKIRA",
  [11] = "MOTHER",
  [12] = "JAYNE",
  [13] = "CORSIX",
  [14] = "ROUJIN",
  [15] = "EDVIN",
  [16] = "SAM",
  [17] = "CHARLIE",
  [18] = "ARTHUR",
  [19] = "MAGNUS",
  [20] = "SAL",
  [21] = "JOSHUA",
  [22] = "DANEEL",
  [23] = "OLIVAW",
  [24] = "NIC",
}
town_map = {
  for_sale = "Προς Πώληση",
  chat = "Τσατ Λεπτομερειών Πόλης",
  number = "Αριθμός Οικοπέδου",
  area = "Περιοχή Οικοπέδου",
  price = "Τιμή Οικοπέδου",
  not_for_sale = "Δεν Πωλείται",
  owner = "Ιδιοκτήτης Οικοπέδου",
}
drug_companies = {
  [1] = "Drugs-U-Like",
  [2] = "Cures R Us",
  [3] = "Jagged Little Pill Co.",
  [4] = "Pootle Fenster Ltd.",
  [5] = "Omnidrug Corp.",
}
progress_report = {
  percentage_pop = "% πληθυσμού",
  more_drinks_machines = "Χτίστε περισσότερα αναψυκτήρια. ",
  win_criteria = "ΚΡΙΤΗΡΙΑ ΝΙΚΗΣ",
  header = "Αναφορά Προόδου",
  quite_unhappy = "Ο κόσμος σας είναι ιδιαίτερα δυσαρεστημένος. ",
  too_hot = "Τακτοποιήστε το σύστημα θέρμανσή σας. Έχει υπερβολική ζέστη. ",
  very_unhappy = "Ο κόσμος σας είναι πολύ δυσαρεστημένος. ",
  free_build = "ΕΛΕΥΘΕΡΟ ΧΤΙΣΙΜΟ",
  too_cold = "Παραείναι κρύα. Τοποθετήστε καλοριφέρ. ",
}
humanoid_name_starts = {
  [1] = "GOLD",
  [2] = "HIGH",
  [3] = "AND",
  [4] = "BEN",
  [5] = "BAN",
  [6] = "BILL",
  [7] = "WY",
  [8] = "WAT",
  [9] = "POD",
  [10] = "KING",
  [11] = "BAR",
  [12] = "PET",
  [13] = "MAN",
  [14] = "BOY",
  [15] = "WAR",
  [16] = "JACK",
  [17] = "CRAB",
  [18] = "FISH",
  [19] = "WATER",
  [20] = "MIL",
  [21] = "BY",
  [22] = "FEN",
  [23] = "RICH",
  [24] = "SCOT",
  [25] = "BUR",
  [26] = "PIKE",
  [27] = "CUR",
  [28] = "WHIT",
  [29] = "BINN",
  [30] = "BONN",
  [31] = "POG",
  [32] = "WRIGHT",
  [33] = "KER",
}
confirmation = {
  replace_machine = "Είστε σίγουροι ότι θέλετε να αντικαταστήσετε το %s για το $%d?",
  overwrite_save = "Ένα παιχνίδι είναι ήδη αποθηκευμένο σε αυτήν τη θέση. Είστε σίγουροι ότι θέλετε να το αντικαταστήσετε;",
  sack_staff = "Είστε σίγουροι ότι επιθυμείτε να απολύσετε αυτό το μέλος του προσωπικού;",
  maximum_screen_size = "Το μέγεθος οθόνης που έχετε καταχωρήσει είναι μεγαλύτερο από 3000 x 2000. Αναλύσεις μεγαλύτερες από αυτήν είναι πιθανές, ωστόσο θα απαιτήσουν καλύτερο hardware, ώστε να διατηρήσουν ένα μια αναπαράξιμη συχνότητα λήψεων (frame rate). Είστε σίγουροι ότι θέλετε να συνεχίσετε;",
  needs_restart = "Αλλάζοντας αυτήν την επιλογή απαιτείται η επανεκκίνηση του CorsixTH. Οποιαδήποτε μη-αποθηκευμένη πρόοδος θα χαθεί. Είστε σίγουροι ότι θέλετε να το κάνετε;",
  return_to_blueprint = "Είστε σίγουροι ότι θέλετε να επιστρέψετε στη λειτουργία Σχεδιαγράμματος;",
  delete_room = "Είστε σίγουροι ότι επιθυμείτε να διαγράψετε αυτό το δωμάτιο;",
  quit = "Επιλέξατε την Έξοδο. Είστε σίγουροι ότι θέλετε να φύγετε από το παιχνίδι;",
  restart_level = "Είστε σίγουροι ότι θέλετε να ξεκινήσετε το επίπεδο από την αρχή;",
  abort_edit_room = "Αυτή τη στιγμή χτίζετε ή επεξεργάζεστε ένα δωμάτιο. Αν όλα τα απαιτούμενα αντικείμενα τοποθετηθούν, τότε θα ολοκληρωθεί, διαφορετικά θα διαγραφεί. Θέλετε να συνεχίσετε;",
}
place_objects_window = {
  place_windows = "Τοποθετήστε μερικά παράθυρα, αν θέλετε. Στη συνέχεια, κάντε «κλικ» στην επιβεβαίωση.",
  place_door = "Τοποθετήστε την πόρτα",
  drag_blueprint = "Τραβήξτε προς τα έξω το σχεδιάγραμμα μέχρι να είστε ευχαριστημένοι με το μέγεθός του",
  confirm_or_buy_objects = "Μπορείτε να επιβεβαιώσετε το δωμάτιο, ή να αγοράσετε ή να μετακινήσετε μερικά αντικείμενα",
  place_objects = "Τοποθετήστε τα αντικείμενα και μετακινήστε τα μέχρι να είστε ευχαριστημένοι. Στη συνέχεια, επιβεβαιώστε",
  pick_up_object = "Κάντε «κλικ» στο αντικείμενο για να το σηκώσετε, ή επιλέξτε μια διαφορετική επιλογή από τον αιτούντα",
  place_objects_in_corridor = "Τοποθετήστε τα αντικείμενα κάτω σε έναν διάδρομο",
}
main_menu = {
  map_edit = "Επεξεργασία Χάρτη",
  custom_campaign = "Προσαρμοσμένη Καμπάνια",
  new_game = "Καμπάνια",
  savegame_version = "Έκδοση αποθηκευμένου παιχνιδιού: ",
  exit = "Έξοδος",
  options = "Ρυθμίσεις",
  custom_level = "Μοναδικό Σενάριο",
  continue = "Συνέχεια Παιχνιδιού",
  load_game = "Φόρτωση Παιχνιδιού",
  version = "Έκδοση: ",
}
pay_rise = {
  definite_quit = "Δεν μπορείτε να κάνετε τίποτα για να με κρατήσετε. Ως εδώ έχω φτάσει με αυτό το μέρος.",
  poached = "Μου έχουν προσφέρει %d από %s. Εκτός και αν με πληρώσετε τα ίδια, την έκανα.",
  regular = {
    [1] = "Είμαι τελείως εξουθενωμένος. Χρειάζομαι μια καλή ξεκούραση, και μια αύξηση των %d αν δεν θέλετε να παρατήσω αυτήν την παλιοδουλειά.",
    [2] = "Είμαι πολύ κουρασμένος. Χρειάζομαι ξεκούραση και μια αύξηση μισθού των %d, βάζοντάς με στα %d. Κάν'το τώρα, τύραννε.",
    [3] = "Έλα τώρα. Δουλεύω σαν το σκυλί εδώ. Δώστε μου μπόνους των %d και θα παραμείνω στο νοσοκομείο σας.",
    [4] = "Είμαι πολύ δυσαρεστημένος. Απαιτώ αύξηση των %d, τοποθετώντας τον μισθό μου στα %d, αλλιώς την έκανα από δω.",
    [5] = "Οι γονείς μου μου είπαν ότι τα ιατρικά επαγγέλματα πλήρωναν καλά. Οπότε δώστε μου αύξηση των %d ή θα γίνω παραγωγός βιντεοπαιχνιδιών.",
    [6] = "Νισάφι πια. Πληρώστε με έναν αξιοπρεπή μισθό. Θεωρώ πως μια αύξηση των %d θα κάνει την δουλειά.",
  },
}
menu_list_window = {
  back = "Πίσω",
  save_date = "Τροποποιημένο",
  name = "Όνομα",
}
policy = {
  diag_procedure = "διαδικασία διάγνωσης",
  diag_termination = "τερματισμός διάγνωσης",
  staff_leave_rooms = "το προσωπικό αποχωρεί από δωμάτια",
  header = "ΠΟΛΙΤΙΚΗ ΝΟΣΟΚΟΜΕΙΟΥ",
  sliders = {
    stop = "ΠΑΥΣΗ ΔΙΑΔΙΚΑΣΙΑΣ",
    send_home = "ΑΠΟΣΤΟΛΗ ΣΤΟ ΣΠΙΤΙ",
    staff_room = "ΜΕΤΑΒΑΣΗ ΣΤΟ ΔΩΜΑΤΙΟ ΠΡΟΣΩΠΙΚΟΥ",
    guess = "ΕΙΚΑΣΙΑ ΘΕΡΑΠΕΙΑΣ",
  },
  staff_rest = "αποστολή προσωπικού για ξεκούραση",
}
buy_objects_window = {
  price = "Τιμή: ",
  choose_items = "Επιλογή Αντικειμένων",
  total = "Σύνολο: ",
}
menu_file_load = {
  [1] = "  ΠΑΙΧΝΙΔΙ 1  ",
  [2] = "  ΠΑΙΧΝΙΔΙ 2  ",
  [3] = "  ΠΑΙΧΝΙΔΙ 3  ",
  [4] = "  ΠΑΙΧΝΙΔΙ 4  ",
  [5] = "  ΠΑΙΧΝΙΔΙ 5  ",
  [6] = "  ΠΑΙΧΝΙΔΙ 6  ",
  [7] = "  ΠΑΙΧΝΙΔΙ 7  ",
  [8] = "  ΠΑΙΧΝΙΔΙ 8  ",
}
menu_charts = {
  status = "  (%1%) ΚΑΤΑΣΤΑΣΗ  ",
  bank_manager = "  (%1%) ΜΑΝΑΤΖΕΡ ΤΡΑΠΕΖΑΣ  ",
  graphs = "  (%1%) ΓΡΑΦΗΜΑΤΑ  ",
  town_map = "  (%1%) ΧΑΡΤΗΣ ΠΟΛΗΣ  ",
  staff_listing = "  (%1%) ΛΙΣΤΑ ΠΡΟΣΩΠΙΚΟΥ  ",
  policy = "  (%1%) ΠΟΛΙΤΙΚΗ  ",
  briefing = "  ΕΝΗΜΕΡΩΣΗ  ",
  casebook = "  (%1%) ΑΡΧΕΙΟ ΥΠΟΘΕΣΕΩΝ  ",
  statement = "  (%1%) ΔΗΛΩΣΗ  ",
  research = "  (%1%) ΕΡΕΥΝΑ  ",
}
errors = {
  could_not_find_first_campaign_level = "Δεν ήταν δυνατή η εύρεση του πρώτου επιπέδου αυτή της καμπάνιας: %s",
  save_prefix = "Σφάλμα κατά την αποθήκευση του παιχνιδιού: ",
  no_games_to_contine = "Δεν υπάρχουν αποθηκευμένα παιχνίδια.",
  load_prefix = "Σφάλμα κατά την φόρτωση του παιχνιδιού: ",
  dialog_missing_graphics = "Συγγνώμη, τα αρχεία δεδομένων του demo δεν περιλαμβάνουν αυτόν τον διάλογο.",
  map_file_missing = "Δεν ήταν δυνατή η φόρτωση του αρχείου χάρτη %s για αυτό το επίπεδο!",
  load_quick_save = "Σφάλμα, δεν είναι δυνατή η φόρτωση του quicksave καθώς δεν υπάρχει, αλλά μην ανησυχείτε καθώς δημιουργήσαμε ένα τώρα για εσάς!",
  unavailable_screen_size = "Το μέγεθος οθόνης που απαιτήσατε δεν είναι διαθέσιμο στην λειτουργία πλήρης οθόνης.",
  minimum_screen_size = "Παρακαλώ εισάγετε ένα μέγεθος οθόνης τουλάχιστον 640x480.",
  alien_dna = "ΣΗΜΕΙΩΣΗ: Δεν υπάρχουν animations για Εξωγήινους ασθενείς για να κάθονται, να ανοίγουν ή να χτυπάνε πόρτες κτλ. Έτσι, όπως με το Theme Hospital, για να κάνουν αυτές τις ενέργειες θα φαίνονται να αλλάζουν σε κανονικούς ασθενείς και μετά να επανέρχονται. Οι ασθενείς με Εξωγήινο DNA θα εμφανίζονται μόνο αν έχουν ρυθμιστεί στο αρχείο επιπέδου",
  fractured_bones = "ΣΗΜΕΙΩΣΗ: Το animation για τους θηλυκούς ασθενείς με Σπασμένα Κόκαλα δεν είναι τέλειο",
  could_not_load_campaign = "Σφάλμα κατά την φόρτωση της καμπάνιας: %s",
}
vip_names = {
  [1] = "Δήμαρχος του Greater Trumpton",
  [2] = "Lawrence Nightingale",
  [3] = "Βασιλιάς Βερνάρδος της Ολλανδίας",
  [4] = "Aung Sang Su Kyi, ο Βιρμανός Ηγέτης της Δημοκρατικής Αντιπολίτευσης",
  [5] = "Sir Reginald Crumbly",
  [6] = "Sir Lancelot Spratt",
  [7] = "Σύμβουλος Crawford Purves",
  [8] = "Rocket Ronnie Jepson",
  [9] = "Ποδοσφαιριστής πρώτης εθνικής",
  [10] = "L. F. Probst, III",
  health_minister = "Υπουργός Υγείας",
}
save_game_window = {
  caption = "Αποθηκευμένο Παιχνίδι (%1%)",
  new_save_game = "Νέο Savegame",
}
fax = {
  debug_fax = {
    text11 = "ΑΝΘΡΩΠΟΙ ΑΥΤΟ ΤΟ ΜΗΝΑ %d",
    text3 = "ΝΟΥΜΕΡΑ  : ΓΙΑΤΡΟΙ %d  ΝΟΣΟΚΟΜΕΣ %d  ΠΕΡΙΟΧΗ %d  ΔΩΜΑΤΙΑ %d  ΤΙΜΗ %d",
    text10 = "ΘΑΝΑΤΟΙ %d ΕΠΙΤΡΕΠΤΟΙ (MTHS) %d (%d) ΜΕΙΩΣΗ %d",
    text5 = "ΣΥΝΕΙΣΦΟΡΑ : ΓΙΑΤΡΟΙ %d  ΝΟΣΟΚΟΜΕΣ %d  ΠΕΡΙΟΧΗ %d  ΔΩΜΑΤΙΑ %d  ΤΙΜΗ %d ΠΟΣΟΣΤΟ",
    text8 = "AMENITIES %d PEEPS HANDLED %d REDUCTION %d",
    text2 = "TOTAL PEOPLE IN HOSPITAL %d CHECKING AGAINST %d",
    text9 = "DISASTERS %d ALLOWED (MTHS) %d (%d)REDUCTION %d",
    text6 = "THE FOLLOWING FACTORS ARE ALSO APPLIED",
    close_text = "Yeah, yeah, yeah!",
    text7 = "REPUTATION: %d EXPECTED %d REDUCTION %d",
    text4 = "FACTORS  : DOCS %d  NURSES %d  AREA %d  ROOMS %d  PRICING %d",
    text1 = "BEST COUNT %d",
  },
  vip_visit_result = {
    rep_boost = "Η φήμη σας στην κοινότητα έχει ενισχυθεί.",
    telegram = "Τηλέγραμμα!",
    vip_remarked_name = "Αφού επισκέφτηκε το νοσοκομείο σας, ο %s σχολίασε:",
    close_text = "Ευχαριστούμε που επισκεφτήκατε το νοσοκομείο.",
    rep_loss = "Η φήμη σας έχει υποστεί ανάλογες μειώσεις.",
    cash_grant = "Σας έχει απονεμηθεί χρηματικό βραβείο των %d.",
    remarks = {
      free_build = {
        [1] = "Έχετε ένα πολύ ωραίο νοσοκομείο! Δεν είναι και τόσο δύσκολο να το διαχειρίζεστε δίχως περιορισμούς στα χρήματα όμως, ε;",
        [2] = "Δεν είμαι οικονομολόγος, αλλά πιστεύω ότι θα μπορούσα να τρέξω και εγώ αυτό το νοσοκομείο αν καταλαβαίνετε τι εννοώ...",
        [3] = "Ένα νοσοκομείο που λειτουργεί άψογα. Προσέχτε την άνοδο της οικονομικής κρίσης όμως! Σωστά... δεν χρειάζεται να ανησυχείτε για αυτό.",
      },
      [1] = "Τι φανταστικό νοσοκομείο. Όταν θα ξανααρρωστήσω βαριά, πηγαίντε με εκεί.",
      [2] = "ΑΥΤΟ είναι νοσοκομείο.",
      [3] = "Καταπληκτικό νοσοκομείο. Αν ξέρει κάποιος, αυτός είμαι εγώ, έχω πάει σε αρκετά.",
      [4] = "Βρίσκω πολύ καλή την λειτουργία του νοσοκομείου. Σας ευχαριστώ που με προσκαλέσατε.",
      [5] = "Χμμ. Καθόλου κακό ιατρικό ίδρυμα, αυτό είναι σίγουρο.",
      [6] = "Απόλαυσα το μαγευτικό νοσοκομείο σας. Τώρα, ψήνεται κανείς για curry στο Taj;",
      [7] = "Λοιπόν, έχω δει και χειρότερα. Αλλά πρέπει να κάνετε ορισμένες βελτιώσεις.",
      [8] = "Ω Θεέ μου. Όχι και τόσο ευχάριστο μέρος για να πάει κανείς αν δεν αισθάνεται καλά.",
      [9] = "Είναι ένα νορμάλ νοσοκομείο, για να είμαι ειλικρινής. Ειλικρινά, περίμενα περισσότερα.",
      [10] = "Γιατί ασχολούμαι καν; Ήταν χειρότερο από το να παω να δω τετράωρη όπερα!",
      [11] = "Είμαι αηδιασμένος από αυτό που είδα. Αυτό το λέτε νοσοκομείο; Πιο πολύ σε χοιροστάσιο κάνει!",
      [12] = "Έχω κουραστεί να είμαι στο φως της δημοσιότητας και να επισκέπτομαι βρωμερά αχούρια σαν και αυτό! Παραιτούμαι.",
      [13] = "Τι αχούρι. Θα προσπαθήσω να τερματίσω την λειτουργία του.",
      [14] = "Δεν έχω δει ξανά ένα τόσο αξιοθρήνητο νοσοκομείο. Τι απαίσιο!",
      [15] = "Είμαι σοκαρισμένος. Δεν μπορείτε να το αποκαλείτε αυτό νοσοκομείο! Έφυγα για μπύρα.",
    },
  },
  diagnosis_failed = {
    choices = {
      send_home = "Αποστόλη Ασθενή στο σπίτι.",
      take_chance = "Δοκιμάστε μια πιθανή θεραπεία.",
      wait = "Αφήστε τον Ασθενή να περιμένει, ενώ χτίζετε και άλλα διαγνωστικά δωμάτια.",
    },
    what_to_do_question = "Τι να κάνουμε με τον ασθενή;",
    situation = "Έχουμε εξαντλήσει όλα μας τα μηχανήματα διάγνωσης σε αυτόν τον ασθενή και ακόμα δεν είμαστε σίγουροι τι δεν πάει καλά.",
    partial_diagnosis_percentage_name = "Υπάρχει %d τοις εκάτο πιθανότητα να έχουμε αναγνωρίσει σωστά τι τύπο %s έχει ο ασθενής.",
  },
  choices = {
    return_to_main_menu = "Επιστροφή στο κύριο μενού",
    accept_new_level = "Μετάβαση στο επόμενο επίπεδο",
    decline_new_level = "Συνεχίστε να παίζετε λίγο παραπάνω",
  },
  disease_discovered_patient_choice = {
    guessed_percentage_name = "Η όμαδα σας έπρεπε να μαντέψει σχετικά με την ασθένεια ενός ασθενή. Υπάρχει %d τοις εκατό πιθανότητα να είναι %s",
    disease_name = "Η ομάδα σας έχει ανακαλύψει μια καινούρια ασθένεια. Ονομάζεται %s",
    need_to_build_and_employ = "Αν χτίσετε ένα %s και προσλάβετε %s μπορείτε να επιτύχετε.",
    need_to_build = "Πρέπει να χτίσετε ένα %s για να μπορέσετε να το αντιμετωπίσετε αυτό.",
    can_not_cure = "Δεν μπορείτε να θεραπεύσετε αυτήν την ασθένεια.",
    what_to_do_question = "Τι επιθυμείτε να κάνετε με τον ασθενή;",
    need_to_employ = "Προσλάβετε έναν/μία %s για να αντιμετώπισετε αυτήν την κατάσταση.",
    choices = {
      send_home = "Αποστολή Ασθενή στο σπίτι.",
      research = "Αποστολή ασθενή στο Τμήμα Ερευνών.",
      wait = "Αφήστε τον Ασθενή να περιμένει στο νοσοκομείο για λίγο.",
    },
  },
  vip_visit_query = {
    choices = {
      invite = "Αποστόλη Επίσης Πρόσκλησης στον V.I.P.",
      refuse = "Ξεφορτωθείτε τον V.I.P με δικαιολογίες.",
    },
    vip_name = "Ο %s εξέφρασε την επιθυμία να επισκεφτεί το νοσοκομείο σας",
  },
  disease_discovered = {
    need_to_build = "Πρέπει να χτίσετε ένα %s για να μπορέσετε να το αντιμετωπίσετε.",
    need_to_build_and_employ = "Αν χτίσετε ένα %s και προσλάβετε έναν %s μπορείτε να επιτύχετε.",
    close_text = "Μια νέα πάθηση έχει βρεθεί.",
    can_cure = "Μπορείτε να αντιμετωπίσετε αυτήν την ασθένεια.",
    need_to_employ = "Προσλάβετε έναν %s για να μπορέσετε να χειριστείτε την κρίση.",
    discovered_name = "Η ομάδα σας ανακάλυψε μια νέα πάθηση. Είναι η %s",
  },
  emergency_result = {
    close_text = "Κάντε «κλικ» για να πραγματοποιήσετε έξοδο.",
    earned_money = "Από το μέγισο μπόνους των %d, έχετε κερδίσει %d.",
    saved_people = "Σώσατε %d ανθρώπους από τους %d συνολικά.",
  },
  emergency = {
    choices = {
      accept = "Ναι. Είμαι έτοιμος/η να χειριστώ το επείγον περιστατικό. ",
      refuse = "Όχι. Αρνούμαι να χειριστώ το επείγον περιστατικό.",
    },
    num_disease = "Υπάρχουν %d άνθρωποι με %s και χρήζουν άμεσης παρακολούθησης.",
    cure_not_possible = "Δεν μπορείτε να θεραπεύσετε αυτήν την ασθένεια αυτήν τη στιγμή.",
    locations = {
      [1] = "Χημική Βιομηχανία Θάτσερ.",
      [2] = "Πανεπιστήμιο Σνάιντ.",
      [3] = "το Ανθοκομικό Κέντρο Παρακουάτ.",
      [4] = "το Ινστιτούτο Ερευνών Επικίνδυνων Ουσιών.",
      [5] = "το Συνέδριο Χορευτών Μόρρις.",
      [6] = "η παμπ «Βάτραχος και Ταύρος».",
      [7] = "Γραφείο Κηδειών και Κάβα του Μεγάλου Ντέιβ.",
      [8] = "το Ινδικό εστιατόριο Ταζ.",
      [9] = "Πολυκατάστημα Χρησιμοποιημένων Πετροχημικών του Μπερτ.",
    },
    free_build = "Εαν είστε επιτυχής, η φήμη σας θα αυξηθεί, ωστόσο αν αποτύχετε, η φήμη σας θα πληγεί δραματικά.",
    cure_possible = "Έχετε τα σωστά εργαλεία και δεξιότητες, οπότε ίσως να μπορέσετε να χειριστείτε αυτό το επείγον περιστατικό.",
    cure_possible_drug_name_efficiency = "Ήδη έχετε τα απαραίτητα εργαλεία και δεξιότητες. Έχετε το φάρμακο που χρειάζεστε. Είναι %s και το φάρμακο είναι το %d τοις εκατό αποτελεσματικό.",
    num_disease_singular = "Υπάρχει 1 άτομο με %s και χρήζει άμεσης παρακολούθησης.",
    location = "Υπάρχει ένα περιστατικό στην τοποθεσία %s",
    cure_not_possible_employ = "Θα χρειαστεί να προσλάβετε έναν/μία %s",
    cure_not_possible_build_and_employ = "Θα χρειαστεί να χτίσετε ένα/μία %s και να προσλάβετε έναν/μία %s",
    cure_not_possible_build = "Θα χρειαστεί να χτίσετε ένα/μία %s",
    bonus = "Υπάρχει ένα μέγιστο μπόνους από %d διαθέσιμο για να χειριστείτε αυτό το επείγον περιστατικό. Αν αποτύχετε, ωστόσο, η φήμη σας θα πληγεί δραματικά.",
  },
  epidemic = {
    choices = {
      cover_up = "Προσπαθήστε να θεραπεύσετε όλους τους μολυσμένους ασθενείς στον προκαθορισμένο χρόνο και προτού κάποιος φύγει από το νοσοκομείο σας.",
      declare = "Δηλώστε μία Επιδημία. Πληρώστε πρόστιμο με το ανάλογο πλήγμα στην φήμη σας.",
    },
    disease_name = "Οι Γιατροί σας ανακάλυψαν ένα μεταδοτικό στέλεχος από %s.",
    cover_up_explanation_1 = "Ωστόσο, αν προσπαθήσετε να το κουκουλώσετε, θα έχετε περιορισμένο χρόνο για να θεραπεύσετε όλους τους μολυσμένους ασθενείς, πριν το ανακαλύψουν οι Υγειονομικές Αρχές.",
    declare_explanation_fine = "Αν κηρύξετε επιδημία, θα πληρώσετε πρόστιμο των %d, η φήμη σας θα κλωνιστεί και όλοι οι ασθενείς σας θα εμβολιαστούν αυτομάτως.",
    cover_up_explanation_2 = "Αν ένας Επιθεωρητής Υγείας σας επισκεφτεί και ανακαλύψει ένα τρέχον κουκούλωμα επιδημίας, μπορεί να λάβει δραστικά μέτρα εναντίον σας.",
  },
  epidemic_result = {
    rep_loss_fine_amount = "Οι εφημερίδες θα έχουν την τιμητική τους πάνω σε αυτό. Η φήμη σας θα αμαυρωθεί βαριά. Συν τοις άλλοις, θα σας επιβληθεί πρόστιμο των %d.",
    failed = {
      part_2 = "το προσωπικό του νοσοκομείο σας έχει προκαλέσει την εξάπλωση της ασθένειας στο κοινό που ζει κοντά στο νοσοκομείο.",
      part_1_name = "Έχοντας προσπαθήσει να αποκρύψετε το γεγονός πως αντιμετώπιζαν μια μεταδοτική έξαρση του %s",
    },
    compensation_amount = "Η Κυβέρνηση αποφάσισε να σας απονείμει %d αποζημίωση για τη ζημιά που προκάλεσαν αυτά τα ψέματα στο καλό όνομα του νοσοκομείου σας.",
    close_text = "Ζήτω!",
    succeeded = {
      part_2 = "Ωστόσο, δεν κατάφερε να βρει αποδείξεις για να υποστηρίξει αυτές τις φήμες.",
      part_1_name = "Ο Επιθεωρητής Υγείας άκουσε φήμες ότι το ίδρυμά σας αντιμετώπιζε μια σοβαρή περίπτωση %s.",
    },
    fine_amount = "Η Κυβέρνηση κήρυξε εθνική έκτακτη ανάγκη και σας επιβάλλουν πρόστιμο %d.",
    hospital_evacuated = "Το Υπουργείο Υγείας δεν είχε άλλη επιλογή από το να εκκενώσει το νοσοκομείο σας.",
  },
}
menu_options_game_speed = {
  slower = "  (%1%) ΠΙΟ ΑΡΓΟ  ",
  slowest = "  (%1%) ΤΟ ΠΙΟ ΑΡΓΟ  ",
  pause = "  (%1%) ΠΑΥΣΗ  ",
  max_speed = "  (%1%) ΜΕΓΙΣΤΗ ΤΑΧΥΤΗΤΑ  ",
  normal = "  (%1%) ΚΑΝΟΝΙΚΟ  ",
  and_then_some_more = "  (%1%) ΚΑΙ ΛΙΓΟ ΠΑΡΑΠΑΝΩ  ",
}
high_score = {
  categories = {
    total_value = "ΣΥΝΟΛΙΚΗ ΑΞΙΑ",
    cures = "ΑΡΙΘΜΟΣ ΘΕΡΑΠΕΙΩΝ",
    staff_number = "ΠΕΡΙΣΣΟΤΕΡΟ ΠΡΟΣΩΠΙΚΟ",
    salary = "ΥΨΗΛΟΤΕΡΟΣ ΜΙΣΘΟΣ",
    deaths = "ΑΡΙΘΜΟΣ ΘΑΝΑΤΩΝ",
    patient_happiness = "ΙΚΑΝΟΠΟΙΗΣΗ ΠΕΛΑΤΩΝ",
    staff_happiness = "ΙΚΑΝΟΠΟΙΗΣΗ ΠΡΟΣΩΠΙΚΟΥ",
    cure_death_ratio = "ΛΟΓΟΣ ΘΕΡΑΠΕΙΩΝ-ΘΑΝΑΤΩΝ",
    visitors = "ΠΕΡΙΣΣΟΤΕΡΟΙ ΕΠΙΣΚΕΠΤΕΣ",
    money = "ΠΛΟΥΣΙΟΤΕΡΟΣ",
    clean = "ΚΑΘΑΡΙΟΤΗΤΑ",
  },
  pos = "ΘΕΣΗ",
  worst_scores = "ΤΟΙΧΟΣ ΤΗΣ ΝΤΡΟΠΗΣ",
  best_scores = "HALL OF FAME",
  killed = "Σκότωσε",
  player = "ΠΑΙΚΤΗΣ",
  score = "ΣΚΟΡ",
}
insurance_companies = {
  [1] = "SwanLeaf Ltd.",
  [2] = "Norfolk Onion",
  [3] = "Mutual Wavekrest",
  [4] = "Beagle Scar Ltd.",
  [5] = "Swim Bladder Ltd.",
  [6] = "Terry Honest plc.",
  [7] = "Sir Chubby Holdings",
  [8] = "Leon Katz Corp.",
  [9] = "Prigg and Bong Ltd.",
  [10] = "Happy Insurance Co.",
  [11] = "Syndicate Insurance",
  out_of_business = "ΧΡΕΟΚΟΠΗΣΕ",
}
hotkeys_file_err = {
  file_err_01 = "Δεν ήταν δυνατή η φόρτωση του αρχείου hotkeys.txt. Παρακαλώ βεβαιωθείτε ότι το CorsixTH έχει άδειες ανάγνωσης/εγγραφής",
  file_err_02 = ", ή χρησιμοποιήστε την παράμετρο γραμμής εντολών --hotkeys-file=filename για να επισημάνετε ένα εγγράψιμο αρχείο. Για αναφορά, το σφάλμα φόρτωσης του αρχείου hotkeys ήταν: ",
}
update_window = {
  caption = "Ενημέρωση Διαθέσιμη!",
  new_version = "Νέα Έκδοση:",
  ignore = "Παράβλεψη και επιστροφή στο κύριο μενού",
  current_version = "Τρέχουσα Έκδοση:",
  download = "Μετάβαση στην σελίδα λήψης",
}
information = {
  cheat_not_possible = "Δεν είναι δυνατή η χρήση του συγκεκριμένου cheat σε αυτό το επίπεδο. Αποτυγχάνετε ακόμα και στο να κλέψετε, όχι και τόσο αστείο ε;",
  no_custom_game_in_demo = "Συγγνώμη, αλλά στην demo έκδοση δεν μπορείτε να παίξετε προσαρμοσμένα επίπεδα.",
  very_old_save = "Έχουν γίνει πολλές ενημερώσεις στο παιχνίδι από την στιγμή που ξεκινήσατε να παίζετε αυτό το επίπεδο. Για να βεβαιωθείτε ότι όλες οι λειτουργίες δουλεύουν όπως πρέπει παρακαλώ σκεφτείτε να το ξεκινήσετε από την αρχή.",
  level_lost = {
    [1] = "Ατυχία! Αποτύχατε αυτό το επίπεδο. Καλή τύχη την επόμενη φορά!",
    [2] = "Ο λόγος που χάσατε:",
    reputation = "Η φήμη σας έπεσε κάτω από %d.",
    balance = "Το τραπεζικό σας υπόλοιπο έπεσε κάτω από %d.",
    percentage_killed = "Σκοτώσατε περισσότερο από το %d τοις εκατό των ασθενών.",
    cheat = "Αυτό ήταν επιλογή σας ή επιλέξατε το λάθος κουμπί; Ούτε να κλέψετε σωστά δεν μπορείτε, όχι και τόσο αστείο ε;",
  },
  custom_game = "Καλώς ήρθατε στο CorsixTH. Καλή διασκέδαση με αυτόν τον προσαρμοσμένο χάρτη!",
  cannot_restart = "Δυστυχώς αυτό το προσαρμοσμένο παιχνίδι είχε αποθηκευτεί πριν να υλοποιηθεί η λειτουργία επανεκκίνησης επιπέδου.",
}
calls_dispatcher = {
  staff = "%s - %s",
  close = "Κλείσιμο",
  watering = "Πότισμα @ %d,%d",
  summary = "%d κλήσεις; %d έχουν ανατεθεί",
  repair = "Επιδιόρθωση %s",
}
totd_window = {
  tips = {
    [1] = "Κάθε νοσοκομείο χρειάζεται ένα γραφείο ρεσεψιόν και ένα Γραφείου Γενικού Ιατρού για να ξεκινήσει. Μετά από αυτό, εξαρτάται από το είδος των ασθενών που επισκέπτονται το νοσκομείο σας. Ένα φαρμακείο είναι πάντα καλή επιλογή, πάντως.",
    [2] = "Μηχανήματα όπως ο Φουσκωτής χρειάζονται συντήρηση. Προσλάβετε κάνα δυο πολυτεχνίτες για να επιδιορθώσουν τα μηχανήματά σας, ή θα ρισκάρετε τον τραυματισμό του προσωπικού και των ασθενών σας.",
    [3] = "Μετά από λίγο, το προσωπικό σας θα αρχίσει να κουράζεται. Σιγουρευτείτε να χτίσετε ένα δωμάτιο προσωπικού, ώστε να μπορούν να χαλαρώσουν.",
    [4] = "Τοποθετήστε αρκετά καλοριφέρ για να κρατάτε το προσωπικό και τους ασθενείς σας ζεστούς, ή θα δυσαρεστηθούν. Χρησιμοποιήστε τον χάρτη πόλης για να εντοπίσετε οποιαδήποτε σημεία στο νοσοκομείο σας χρειάζονται περισσότερη θέρμανση.",
    [5] = "Το επίπεδο δεξιότητας ενός γιατρού επηρεάζει σημαντικά την ποιότητα και την ταχύτητα των διαγνώσεών του. Τοποθετήστε έναν ικανό γιατρό στο Γραφείο Γενικού Ιατρού σας, και δεν θα χρειαστείτε τόσα πολλά επιπλέον διαγνωστικά δωμάτια.",
    [6] = "Οι Μαθητευόμενοι και οι γιατροί μπορούν να βελτιώσουν τις δεξιότητές τους μαθαίνοντας από έναν σύμβουλο στο δωμάτιο εκπαίδευσης. Αν ο σύμβουλος έχει κάποια εξειδίκευση (χειρουργός, ψυχίατρος ή ερευνητής), θα μεταλαμπαδεύσει αυτήν την γνώση και στους μαθητές του.",
    [7] = "Δοκιμάσατε να εισάγετε τον Ευρωπαϊκό αριθμό έκτακτης ανάγκης (112) στην μηχανή του φαξ; Σιγουρευτείτε ότι ο ήχος σας είναι ενεργοποιημένος!",
    [8] = "Μπορείτε να τροποποιήσετε κάποιες ρυθμίσεις όπως την ανάλυση και την γλώσσα στο παράθυρο επιλογών που βρίσκεται τόσο στο κύριο μενού όσο και μέσα στο παιχνίδι.",
    [9] = "Έχετε επιλέξει μια γλώσσα διαφορετική των Αγγλικών, αλλά υπάρχει παντού Αγγλικό κείμενο; Βοηθήστε μας μεταφράζοντας τα κείμενα που λείπουν στην γλώσσα σας!",
    [10] = "Η ομάδα του CorsixTH ψάχνει για ενισχύσεις! Ενδιαφέρεστε να γράψετε κώδικα, να μεταφράσετε ή να δημιουργήσετε γραφικά για το CorsixTH; Επικοινωνήστε μαζί μας στο Forum μας, το Mailing List ή IRC Channel (corsix-th at freenode).",
    [11] = "Αν βρείτε κάποιο bug, παρακαλώ όπως αναφέρετε το στον bugtracker μας: th-issues.corsix.org",
    [12] = "Κάθε επίπεδο έχει ορισμένες απαιτήσεις που πρέπει να εκπληρωθούν πριν να μπορείτε να μεταβείτε στο επόμενο. Τσεκάρετε το παράθυρο κατάστασης για να δείτε την πρόοδο σας όσον αφορά τους στόχους του επιπέδου.",
    [13] = "Αν θέλετε να επεξεργαστείτε ή να αφαιρέσετε ένα υπάρχον δωμάτιο, μπορείτε να το κάνετε με το κουμπί επεξεργασίας δωματίου το οποίο βρίσκεται στην κάτω εργαλειοθήκη.",
    [14] = "Σε μια ορδή ασθενών που περιμένουν, μπορείτε γρήγορα να βρείτε ποιοι περιμένουν για κάποιο συγκεκριμένο δωμάτιο περνώντας τον κέρσορά σας πάνω από εκείνο το δωμάτιο.",
    [15] = "Κάντε κλικ στην πόρτα ενός δωματίου για να δείτε την ουρά του. Μπορείτε να κάνετε χρήσιμες τροποποιήσεις εδώ, όπως να αλλάξετε την σειρά ή να στείλετε έναν ασθενή σε άλλο δωμάτιο.",
    [16] = "Δυσαρεστημένο προσωπικό θα ζητάει αυξήσεις μισθού συχνά. Σιγουρευτείτε ότι το προσωπικό σας δουλεύει σε ένα άνετο περιβάλλον για να μην συμβεί αυτό.",
    [17] = "Οι ασθενείς σας θα διψάσουν καθώς περιμένουν στο νοσοκομείο σας, ακόμα περισσότερο αν αυξήσετε την θερμοκρασία! Τοποθετήστε αναψυκτήρια σε στρατηγικές θέσεις για λίγα παραπάνω έσοδα.",
    [18] = "Μπορείτε να ακυρώσετε την διαγνωστική διαδικασία για έναν ασθενή πρόωρα και να μαντέψετε την θεραπεία, αν έχετε ήδη αντιμετωπίσει την ασθένεια. Σημειώστε πως αυτό μπορεί να αυξήσει το ρίσκο λανθασμένης θεραπείας, με αποτέλεσμα τον θάνατο του ασθενή.",
    [19] = "Οι καταστάσεις έκτακτης ανάγκης μπορούν να αποτελέσουν καλή πηγή επιπλέον εισοδήματος, με την προϋπόθεση ότι έχετε τις υποδομές να χειριστείτε τους επείγοντες ασθενείς εγκαίρως.",
  },
  previous = "Προηγούμενη Συμβουλή",
  next = "Επόμενη Συμβουλή",
}
cheats_window = {
  caption = "Cheats",
  warning = "Προσοχή: Δεν θα λάβετε μπόνους πόντους στο τέλος του επιπέδου εάν κλέψετε!",
  cheated = {
    yes = "Χρησιμοποιήθηκαν cheats: Ναι",
    no = "Χρησιμοποιήθηκαν cheats: Όχι",
  },
  cheats = {
    all_research = "Ολοκληρωμένη Έρευνα",
    emergency = "Δημιουργία Κατάστασης Έκτακτης Ανάγκης",
    vip = "Δημιουργία VIP",
    win_level = "Κερδίστε το Επίπεδο",
    epidemic = "Δημιουργία μεταδοτικού ασθενή",
    toggle_infected = "Εμφάνιση εικονιδιών για μολυσμένους",
    end_year = "Τέλος Έτους",
    lose_level = "Χάστε το Επίπεδο",
    increase_prices = "Αύξηση τιμών",
    earthquake = "Δημιουργία Σεισμού",
    decrease_prices = "Μείωση τιμών",
    create_patient = "Δημιουργία Ασθενή",
    money = "Cheat Χρημάτων",
    end_month = "Τέλος Μήνα",
  },
  close = "Κλείσιμο",
}
bank_manager = {
  current_loan = "Τρέχον Δάνειο",
  hospital_value = "Αξία Νοσοκομείου",
  interest_payment = "Πληρωμή Τόκου",
  insurance_owed = "Οφειλόμενη Ασφάλιση",
  inflation_rate = "Ρυθμός Πληθωρισμού",
  balance = "Υπόλοιπο",
  statistics_page = {
    details = "Λεπτομέρειες",
    money_in = "Έσοδα",
    date = "Ημερομηνία",
    money_out = "Έξοδα",
    balance = "Υπόλοιπο",
    current_balance = "Τρέχον Υπόλοιπο",
  },
  interest_rate = "Επιτόκιο",
}
queue_window = {
  num_expected = "Προσδοκόμενο",
  num_in_queue = "Μέγεθος Ουράς",
  num_entered = "Πλήθος Επισκεπτών",
  max_queue_size = "Μέγιστο Μέγιθος",
}
warnings = {
  levelfile_variable_is_deprecated = "Σημείωση: Το επίπεδο '%s' περιέχει έναν απαρχαιωμένο ορισμό μεταβλητής στο αρχείο επιπέδου. Το αρχείο '%LevelFile' έχει μετονομαστεί σε '%MapFile'. Παρακαλώ συμβουλέψτε τον δημιουργό του χάρτη να ανανεώσει το επίπεδο.",
}
lua_console = {
  execute_code = "Εκτέλεση",
  close = "Κλείσμιο",
}
new_game_window = {
  hard = "Σύμβουλος (Δύσκολο)",
  caption = "Καμπάνια",
  option_on = "On",
  easy = "Μαθητευόμενος (Εύκολο)",
  medium = "Γιατρός (Μέτριο)",
  tutorial = "Tutorial",
  difficulty = "Δυσκολία",
  option_off = "Off",
  cancel = "Ακύρωση",
  player_name = "Όνομα παίκτη",
  start = "Εκκίνηση",
}
font_location_window = {
  caption = "Επιλογή γραμματοσειράς (%1%)",
}
humanoid_name_ends = {
  [1] = "SMITH",
  [2] = "WICK",
  [3] = "CLIFFE",
  [4] = "SON",
  [5] = "INGTON",
  [6] = "BURY",
  [7] = "TON",
  [8] = "SON",
  [9] = "LEY",
  [10] = "BERRY",
  [11] = "BAUM",
  [12] = "LAN",
  [13] = "HAM",
  [14] = "SILL",
  [15] = "WIN",
  [16] = "LET",
  [17] = "ERS",
  [18] = "TON",
  [19] = "MOND",
  [20] = "MAN",
  [21] = "ELTON",
  [22] = "E",
  [23] = "MORE",
  [24] = "MOOR",
  [25] = "LET",
  [26] = "LIN",
}
date_format = {
  daymonth = "%1% %2:months%",
}
build_room_window = {
  cost = "Κόστος: ",
  pick_department = "Διαλέξτε Τμήμα",
  pick_room_type = "Διαλέξτε Τύπο Δωματίου",
}
staff_class = {
  surgeon = "Χειρουργός",
  receptionist = "Ρεσεψιονίστ",
  doctor = "Γιατρός",
  nurse = "Νοσοκόμα",
  handyman = "Πολυτεχνίτης",
}
folders_window = {
  caption = "Τοποθεσίες Φακέλων",
  data_label = "Δεδομένα TH",
  music_label = "Μουσική",
  font_label = "Γραμματοσειρά",
  screenshots_location = "Επιλέξτε τον προορισμό που θέλετε να χρησιμοποιήσετε για τα Screenshots",
  screenshots_label = "Screenshots",
  savegames_label = "Αποθηκευμένα Παιχνίδια",
  back = "Πίσω",
  savegames_location = "Επιλέξτε τον προορισμό που θέλετε να χρησιμοποιήσετε για τα Αποθηκευμένα Παιχνίδια",
  music_location = "Επιλέξτε τον προορισμό που θέλετε να χρησιμοποιήσετε για την Μουσική σας",
  new_th_location = "Εδώ μπορείτε να ορίσετε έναν νέο προορισμό εγκατάστασης για το Theme Hospital. Όταν επιλέξετε τον καινούριο προορισμό το παιχνίδι θα επανεκκινηθεί.",
}
options_window = {
  capture_mouse = "Capture Mouse",
  caption = "Ρυθμίσεις",
  folder = "Φάκελοι",
  option_on = "On",
  option_off = "Off",
  customise = "Τροποποίηση",
  scrollspeed = "Ταχύτητα Κύλησης",
  fullscreen = "Πλήρης Οθόνη",
  shift_scrollspeed = "Ταχύτητα Κύλησης Shift",
  hotkey = "Hotkeys",
  audio = "Καθολικός Ήχος",
  change_resolution = "Αλλαγή ανάλυσης",
  height = "Ύψος",
  language = "Γλώσσα παιχνιδιού",
  width = "Πλάτος",
  zoom_speed = "Ταχύτητα Ζουμ",
  resolution = "Ανάλυση",
  apply = "Εφαρμογή",
  cancel = "Ακύρωση",
  custom_resolution = "Προσαρμοσμένη...",
  back = "Πίσω",
}
research = {
  categories = {
    specialisation = "Ειδίκευση",
    drugs = "Έρευνα Φαρμάκων",
    improvements = "Βελτιώσεις",
    diagnosis = "Διαγνωστικός Εξοπλισμός",
    cure = "Εξοπλισμός Θεραπείας",
  },
  allocated_amount = "Κατανεμημένο Ποσό",
  funds_allocation = "Κατανομή Χρημάτων",
}
introduction_texts = {
  level18 = "",
  level6 = " Χρησιμοποιήστε όλη σας την γνώση για να δημιουργήσετε ένα καλά οργανωμένο και λειτουργικό νοσοκομείο που έχει υγιή κέρδη και μπορεί να αντιμετωπίσει οτιδήποτε του ρίχνει ο άρρωστος ντουνιάς.  Θα πρέπει να γνωρίζετε ότι η ατμόσφαιρα εδώ πέρα είναι πολύ καλή στο να κουβαλάει μικρόβια και μολύνσεις.  Εκτός και αν κρατήσετε το νοσοκομείο σας σχολαστικά καθαρό θα έρθετε αντιμέτωποι με μια πληθώρα επιδημιών μεταξύ των ασθενών.  Σιγορευτείτε πως έχετε κέρδη της τάξεως των $150,000, καθώς και ότι η συνολική αξία του νοσοκομείου σας είναι πάνω από $140,000.",
  level3 = " Θα δημιουργήσετε το νοσοκομείο σας σε μια εύπορη περιοχή αυτήν την φορά. Το Υπουργείο Υγείας προσδοκεί να βγάλετε έναν καλό τζίρο.  Για αρχή θα πρέπει να αποκτήσετε καλή φήμη, αλλά μόλις το νοσοκομείο είναι πλήρως λειτουργικό, συγκεντρωθείτε στο να έχετε όσες περισσότερες εισφορές γίνεται.  Υπάρχει επίσης πιθανότητα να έχετε Επείγοντα Περιστατικά.  Αυτά συμβαίνουν είναι όταν έρχονται ένας μεγάλος αριθμός ατόμων ταυτόχρονα, όλοι με την ίδια ασθένεια. Το να τους θεραπεύσετε εντός του χρονικού ορίου σας εξασφαλίζει καλύτερη φήμη και ένα μεγάλο χρηματικό μπόνους.  Ασθένειες όπως το Σύνδρομο Βασιλιά μπορεί να εμφανιστούν, και θα πρέπει να αποταμιεύσετε χρήματα για το χτίσιμο ενός Χειρουργείου με μια Πτέρυγα κοντά του.  Κερδίστε $20,000 για να επιτύχετε.",
  level16 = " Όταν θα έχετε διαγνώσει μερικούς από τους ασθενείς θα χρειαστεί να φτιάξετε μονάδες θεραπείας και κλινικές για να τους θεραπεύσετε - μία καλή για να ξεκινήσετε είναι το Φαρμακείο. Θα χρειαστείτε ακόμα μία Νοσοκόμα για να χορηγεί τα διάφορα φάρμακα στο Φαρμακείο.",
  level11 = " Σας έχει δοθεί η ευκαρία να χτίσετε το απόλυτο νοσοκομείο.  Αυτή είναι μια περιοχή υπερβολικά υψηλού κύρους, και το Υπουργείο θα ήθελε να δει το καλύτερο δυνατό νοσοκομείο.  Αναμένουμε να έχετε μεγάλα κέρδη, εξαιρετικά υψηλή φήμη καθώς και να καλύψετε κάθε πιθανό ενδεχόμενο.  Αυτή η δουλειά είναι πολύ σημαντική.  Θα πρέπει να είστε άτομο ξεχωριστό για να τα καταφέρετε.  Σημειώστε, επίσης, ότι έχουν θεαθεί UFO στην περιοχή. Σιγουρευτείτε ότι το προσωπικό σας είναι προετοιμασμένο για μερικούς αναπάντεχους επισκέπτες.  Το νοσοκομείο σας θα πρέπει να έχει αξία $240,000, θα χρειαστείτε $500,000 στην τράπεζα και η φήμη σας θα πρέπει να είναι τουλάχιστον 700.",
  level12 = " Έρχεστε αντιμέτωποι με την μητέρα όλων των προκλήσεων τώρα.  Εντυπωσιασμένοι από την επιτυχία σας, το Υπουργείο σας έχει την πλέον κορυφαία δουλειά; θέλουν κάποιον να χτίσει και άλλο απόλυτο νοσοκομείο, να βγάλει ένα τεράστιο χρηματικό ποσό και να έχει απίστευτη φήμη.  Επίσης αναμένεται από εσάς να αγοράσετε όλη την γη που μπορείτε, να θεραπεύσετε τα πάντα (και εννοούμε τα πάντα) και να κερδίσετε όλα τα βραβεία.  Νομίζετε ότι μπορείτε να ανταπεξέλθετε; Κερδίστε $650,000, θεραπεύστε 750 ανθρώπους, και αποκτήστε σκορ φήμης 800 για να κερδίσετε.",
  level9 = " Έχοντας γεμίσει τον τραπεζικό λογαριασμό του Υπουργείου και πληρώσει για μια νέα λιμουζίνα για τον Υπουργό τον ίδιο, μπορείτε τώρα να επιστρέψετε στην δημιουργία ενός νοσοκομείου που λειτουργεί καλά και εξυπηρετεί τις ανάγκες των άπορων και των εχόντων ανάγκη.  Αναμένετε να μαζευτεί πληθώρα διαφορετικών προβλήματων. Εάν έχετε αρκετό καλά εκπαιδευμένο προσωπικό και επαρκή δωμάτια, τότε θα είστε καλυμμένοι από όλες τις πλευρές.  Το νοσοκομείο σας θα πρέπει να αξίζει $200,000, και επίσης θα χρειαστεί να έχετε $400,000 στην τράπεζα.  Οτιδήποτε λιγότερο και δεν θα μπορέσετε να ολοκληρώσετε το επίπεδο.",
  level17 = " Τελευταία προειδοποίηση - να δίνετε προσοχή στην Φήμη σας - αυτή είναι που θα προσελκύσει τους ασθενείς από μακριά στο ίδρυμά σας.  Αν δεν σκοτώσετε πάρα πολλούς ανθρώπους και τους κρατήσετε χαρούμενους σε λογικά πλαίσια δεν θα έχετε και τόσο θέμα σε αυτό το επίπεδο !// Είστε μόνοι σας τώρα. Καλή τύχη και τα λοιπά.",
  level7 = " Εδώ θα είστε σε στενό έλεγχο από το Υπουργείο Υγείας, οπότε σιγουρευτείτε ότι οι λογαριασμοί σας δείχνουν ότι βγάζετε πολύ χρήμα, καθώς και ότι η φήμη σας παραμένει υψηλή.  Δεν υπάρχει δυνατότητα για περιττούς θανάτους - κάνουν κακό στην δουλειά.  Σιγουρευτείτε ότι το προσωπικό σας βρίσκεται σε εξαιρετική φόρμα, καθώς και οτι διαθέτετε όλον τον εξοπλισμό που χρειάζεστε.  Αποκτήστε 600 φήμη, καθώς και $200,000 στην τράπεζα.",
  level15 = " Εντάξει αυτοί είναι οι βασικοί μηχανισμοί για την λειτουργία ενος νοσοκομείου.// Οι Γιατροί θα χρειαστούν την μέγιστη δυνατή βοήθεια για να διαγνώσουν κάποιους από αυτούς τους ασθενείς. Μπορείτε να τους βοηθήσετε χτίζοντας κι άλλες διαγνωστικές εγκαταστάσεις όπως το Δωμάτιο Γενικής Διάγνωσης.",
  level1 = " Καλώς ήρθατε στο πρώτο σας νοσοκομείο!// Εκκινήστε την λειτουργία του τοποθετώντας ένα Γραφείο Ρεσεψιόν, χτίζοντας ένα Γραφείο Γενικού Ιατρού και προσλαμβάνοντας μία Ρεσεψιονίστ και έναν Γιατρό.  Έπειτα περιμένετε την άφιξη κάποιων ασθενών. Θα ήταν καλή ιδέα να χτίσετε ένα Ψυχιατρείο και να προσλάβετε έναν Γιατρό με Ψυχιατρικές ικανότητες.  Ένα Φαρμακείο και μια Νοσοκόμα είναι επίσης απαραίτητα για την θεραπεία των ασθενών σας.  Να είστε προσεκτικοί για κατεργάρικες περιπτώσεις Φουσκωμένου Κεφαλιού - ένα δωμάτιο Φουσκώματος θα τις τακτοποιήσει.  Θα πρέπει να θεραπεύσετε 10 ανθρώπους και να σιγουρέψετε ότι η φήμη σας δεν θα πέσει κάτω από 200.",
  level10 = " Πέρα από το να καλύψετε όλες τις περιπτώσεις ασθενειών που συγκεντρώνονται στην περιοχή αυτή, το Υπουργείο ζητάει να αφιερώσετε κάποιο χρόνο στην αποτελεσματικότητα των φαρμάκων σας.  Υπήρξαν κάποια παράπονα από τον Ofsick, την Επιτροπή Δεοντολογίας Υγείας, έτσι για να έχετε μια καλή εικόνα πρέπει να σιγουρέψετε ότι όλα τα φάρμακά σας είναι εξαιρετικά αποτελεσματικά.  Επίσης, σιγουρευτείτε ότι το νοσοκομείο σας είναι υπεράνω μομφής. Κρατήστε τους θανάτους σε χαμηλό επίπεδο.  Σαν συμβουλή, μπορεί να θέλετε να κρατήσετε ελεύθερο χώρο για μια Κλινική Ζελέ.  Βελτιώστε όλα τα φάρμακά σας βαθμό αποτελεσματικότητας τουλάχιστον 80 τοις εκατό, αποκτήστε 650 φήμη και τζίρο $500,000 στην τράπεζα για να κερδίσετε. ",
  level4 = " Κρατήστε όλους τους ασθενείς σας χαρούμενους, αντιμετωπίστε τους όσο αποτελεσματικά μπορείτε και κρατήστε τους θανάτους στο απολύτως ελάχιστο. Διακυβεύβεται η φήμη σας, οπότε σιγουρέψτε ότι θα την φτάσετε όσο πιο ψηλά γίνεται.  Μην ανησυχείτε πάρα πολύ για τα χρήματα - αυτά θα έρθουν καθώς αυξάνεται η ζωτική σας φήμη.  Θα μπορείτε, επίσης, να εκπαιδεύσετε τους Γιατρούς σας για να διευρύνετε τις ικανότητές τους.  Θα μπορούσαν κάλλιστα να έρθουν αντιμέτωποι με ασθενείς που είναι περισσότερο διαφανείς από το συνηθισμένο.  Αποκτήστε φήμη πάνω από 500.",
  demo = "Καλώς ήρθατε στο demo νοσοκομείο!//Δυστυχώς η έκδοση του the demo περιέχει μόνο αυτό το επίπεδο. Ωστόσο, υπάρχουν πολλά που μπορείτε να κάνετε εδώ για να σας κρατήσουν απασχολημένους για αρκετή ώρα! Θα έρθετε αντιμέτωποι με διάφορες ασθένειες οι οποίες απαιτούν διαφορετικά δωμάτια για θεραπεία. Ανά διαστήματα, ενδέχεται να προκληθούν καταστάσεις έκτακτης ανάγκης. Επίσης θα πρέπει να ανακαλύψετε επιπλέον δωμάτια χρησιμοποιώντας ένα δωμάτιο ερευνών. Ο στόχος σας είναι να κερδίσετε $100,000, να έχετε αξία νοσοκομείου $70,000 και φήμη 700, έχοντας παράλληλα θεραπεύσει τουλάχιστον το 75% των ασθενών σας. Σιγουρευτείτε ότι η φήμη σας δεν θα πέσει κάτω από 300 και ότι δεν θα σκοτώσετε περισσότερο από το 40% των ασθενών σας, ειδάλλως θα χάσετε.//Καλή τύχη!",
  level5 = " Αυτό θα είναι ένα πολυάσχολο νοσοκομείο, το οποίο θα αντιμετωπίζει μια πληθώρα ασθενειών. Οι Γιατροί σας είναι όλοι φρέσκοι από την ιατρική σχολή, οπότε κρίνεται ζωτικής σημασίας να χτίσετε ένα δωμάτιο εκπαίδευσης και να τους εκπαιδεύσετε σε αποδεκτά επίπεδα.  Έχετε μόνο τρεις Συμβούλους για να βοηθήσουν στην εκπαίδευση του άπειρου προσωπικού σας, οπότε κρατήστε τους χαρούμενους.  Σημειώστε, επίσης, ότι τα θεμέλια του νοσοκομείου σας εκτείνονται πάνω από το σεισμογενές έδαφος του San Android.  Υπάρχει μόνιμο ενδεχόμενο σεισμών.  Θα προκαλέσουν σημαντικές υλικές ζημιές στα μηχανήματά σας, και θα αναταράξουν την ομαλή λειτουργία του νοσοκομείου σας.  Αποκτήστε φήμη 400, και κάντε καθαρό τζίρο $50,000 στην τράπεζα για να επιτύχετε. Επίσης, θεραπεύστε 200 ασθενείς.",
  level8 = " Είναι επάνω σας το να χτίσετε το πιο αποτελεσματικό και οικονομικά αποδοτικό νοσοκομείο δυνατόν.  Ο κόσμος εδώ τριγύρω είναι αρκετά καλά οικονομικά, οπότε αρμέξτε τους για όσα περισσότερα χρήματα μπορείτε.  Να θυμάστε, το να θεραπεύεις κόσμο είναι ωραίο, ναι, αλλά αυτό που πραγματικά ΧΡΕΙΑΖΕΣΤΕ είναι τα λεφτά που αυτό αποφέρει.  Πηγαίντε αυτούς τους αρρώστους στο καθαριστήριο.  Συγκεντρώστε το γιγάντιο ποσό των $300,000 για να ολοκληρώσετε αυτό το επίπεδο.",
  level13 = " Οι εκπληκτικές σας ικανότητες ως διαχειριστής νοσοκομείου έχουν τραβήξει την προσοχή του Ειδικού Μυστικού Τμήματος της Μυστικής Ειδικής Υπηρεσίας.  Έχουν ένα ειδικό μπόνους για εσάς: υπάρχει ένα κατακλυσμένο από αρουραίους νοσοκομείο σε ανάγκη για έναν αποτελεσματικό Εξολοθρευτή.  Πρέπει να πυροβολήσετε όσους περισσότερους αρουραίους γίνεται πριν οι Πολυτεχνίτες μαζέψουν όλα τα σκουπίδια.  Πιστεύετε ότι μπορείτε να το κάνετε;",
  level2 = "Υπάρχει μεγαλύτερη ποικιλία ασθενειών σε αυτήν την περιοχή. Σετάρετε το νοσοκομείο σας για να χειρίζεστε περισσότερους ασθενείς, και σχεδιάστε να χτίσετε ένα Τμήμα Ερευνών. Να θυμάστε να κρατάτε το ίδρυμά σας καθαρό, και προσπαθήστε να φτάσετε την φήμη σας όσο πιο ψηλά γίνεται - θα έρθετε αντιμέτωποι με ασθένειες όπως Χαλαρωμένη Γλώσσα, οπότε θα χρειαστείτε μια Κλινική Χαλαρωμένης Γλώσσας. Μπορείτε επίσης να χτίσετε ένα Δωμάτιο Καρδιογραφήματος για να σας βοηθήσει να διαγνώσετε νέες ασθένειες. Και τα δύο αυτά δωμάτια θα πρέπει πρώτα να ανακαλυφθούν πριν να μπορούν να χτιστούν. Τώρα μπορείτε επίσης να αγοράσετε επιπλέον κομμάτια γης για να επεικτείνετε το νοσοκομείο σας - χρησιμοποιήστε τον Χάρτη Πόλης για αυτό. Στοχεύστε για φήμη 300 και τραπεζικό υπόλοιπο $10,000 και θεραπεύστε 40 άτομα.",
  level14 = " Υπάρχει ακόμα μια πρόκληση - το εντελώς αναπάντεχο νοσοκομείο-έκπληξη.  Αν επιτύχετε σε αυτό, θα είστε ο νικητής πάνω από όλους τους άλλους νικητές.  Μην περιμένετε να είναι ευκολάκι, ωστόσο, επειδή θα είναι η δυσκολότερη δοκιμασία που αντιμετωπίσατε ποτέ σας.  Καλή τύχη!",
}
custom_campaign_window = {
  start_selected_campaign = "Έναρξη καμπάνιας",
  caption = "Προσαρμοσμένη Καμπάνια",
}
trophy_room = {
  gen_repairs = {
    awards = {
      [1] = "Σας απονέμεται ένα ειδικό βραβείο για την ετοιμότητα των Πολυτεχνιτών σας στο να διατηρούν σε άριστη κατάσταση τον εξοπλισμό του νοσοκομείου σας. Συγχαρητήρια. Πάρτε λίγο χρόνο για διακοπές.",
      [2] = "Οι Πολυτεχνίτες σας έχουν καλύτερες επιδόσεις συγκριτικά με αυτούς οποιουδήποτε άλλου νοσοκομείου. Αυτό είναι ένα μεγάλο κατόρθωμα για το οποίο πρέπει να σας συγχαρούμε.",
      [3] = "Τα μηχανήματά σας είναι συντηρημένα εξαιρετικά. Η αφοσίωση που δείχνουν οι Πολυτεχνίτες σας είναι εξωπραγματική. Όλοι σας αξίζετε αυτό το βραβείο υψηλού κύρους. Φανταστική δουλειά.",
    },
    penalty = {
      [1] = "Οι Πολυτεχνίτες σας δεν έχουν συντηρήσει επαρκώς τα μηχανήματά σας. Πρέπει να τους επιβλέπετε με μεγαλύτερη προσοχή ή να προσλάβετε περισσότερους για να αντιμετωπίσουν τον φόρτο εργασίας.",
      [2] = "Τα έχετε κάνει χάλια με την συντήρηση. Το προσωπικό σας θα πρέπει να συντηρεί τον εξοπλισμό σας με αμεσότητα και φροντίδα.",
    },
  },
  happy_vips = {
    trophies = {
      [1] = "Έχετε κερδίσει το βραβείο Νόμπελ για Εντυπωσιασμό VIPs. Όλοι που επισκέφτηκαν το νοσοκομείο σας τον τελευταίο χρόνο είχαν πολύ θετικές εντυπώσεις.",
      [2] = "Η Υπηρεσία Διάσημων Ατόμων θα ήθελε να σας απονείμει το Τρόπαιο Σελέμπριτι για την ικανοποίηση κάθε VIP που επισκέφτηκε τις εγκαταστάσεις σας. Θεωρείστε τον εαυτό σας ελαφρώς διάσημο, και (σχεδόν) έναν από εμάς.",
      [3] = "Συγχαρητήρια που κερδίσατε το βραβείο VIP TRIP για την βελτίωση της ζωής των σκληρά εργαζόμενων ατόμων στα μάτια της δημοσιότητας, ικανοποιώντας όλους όσους επισκέφτηκαν το νοσοκομείο σας. Εξαίσια.",
    },
  },
  research = {
    regional_good = {
      [1] = "Η έρευνά σας έχει κρατήσει το νοσοκομείο σας μπροστά από όλες τις τελευταίες τεχνολογικές εξελίξεις. Στο προσωπικό ερευνών σας αξίζει αυτό το βραβείο. Μπράβο.",
    },
    penalty = {
      [1] = "Η προσπάθειά σας στην έρευνα νέων θεραπειών, εξοπλισμού και φαρμάκων είναι ιδιαίτερα ανεπαρκής. Αυτό είναι πολύ κακό, καθώς η τεχνολογική πρόοδος είναι ζωτικής σημασίας.",
    },
    awards = {
      [1] = "Η έρευνά σας έχει κρατήσει το νοσοκομείο σας μπροστά από όλες τις τελευταίες τεχνολογικές εξελίξεις. Στο προσωπικό ερευνών σας αξίζει αυτό το βραβείο. Μπράβο.",
      [2] = "Κατά την διάρκεια του προηγούμενου χρόνου, ανακαλύψατε πολλά περισσότερα φάρμακα και εξοπλισμό απ' ό,τι θα ήλπιζε κανείς. Παρακαλούμε αποδεχτείτε αυτό το βραβείο από όλους εμάς στο Υπουργείο.",
    },
    regional_bad = {
      [1] = "Κάθε άλλο νοσοκομείο στην περιοχή είναι καλύτερο στην έρευνα από εσάς. Καθώς η έρευνα είναι υψίστης σημασίας για το νοσοκομείο σας, το Υπουργείο είναι έξω φρενών.",
    },
  },
  no_deaths = {
    trophies = {
      [1] = "Έχετε κερδίσει το βραβείο No-Croak επειδή κρατήσατε το 100 τοις εκατό των ασθενών σας ζωντανό τον τελευταίο χρόνο.",
      [2] = "Η Life Goes On Corporation σας απονέμει αυτό το τρόπαιο επειδή κατορθώσατε να έχετε μηδενικούς θανατούς τον τελευταίο χρόνο.",
      [3] = "Σας έχει απονεμηθεί το τρόπαιο Staying Alive λόγω του ότι κατορθώσατε να μην έχετε καθόλου θανάτους στο εξαιρετικό νοσοκομείο σας τον τελευταίο χρόνο. Έξοχα.",
    },
    penalty = {
      [1] = "Ο αριθμός των θανάτων στο νοσοκομείο σας τον τελευταίο χρόνο ήταν υψηλός σε αποδεκτά επίπεδα. Δώστε περισσότερη προσοχή στην δουλειά σας. Σιγουρέψτε ότι περισσότερος κόσμος θα επιβιώνει μελλοντικά.",
      [2] = "Το νοσοκομείο σας αποτελεί ρίσκο στην υγεία των ασθενών. Προσδοκείται από εσάς να θεραπεύετε πολύ κόσμο, όχι να τους αφήνετε να πεθαίνουν.",
    },
    awards = {
      [1] = "Κερδίσατε αυτό το βραβείο για να τιμηθεί ο χαμηλός αριθμός θανάτων στο νοσοκομείο σας, αυτή τη χρονιά. Είναι απίθανο!",
      [2] = "Η ιδιοφυΐα σας κράτησε τους θανάτους στο νοσοκομείο σας στο ελάχιστο. Πρέπει να είστε υπερβολικά ευχαριστημένος με αυτό το αποτέλεσμα!",
    },
    regional = {
      [1] = "Ο αριθμός θανάτων στο νοσοκομείο σας πέρσι ήταν χαμηλότερος από κάθε άλλο νοσοκομείο. Παρακαλώ, δεχθείτε αυτό το βραβείο.",
    },
  },
  cash = "CASH",
  emergencies = {
    award = {
      [1] = "Συγχαρητήρια: ο αποδοτικός κι αποτελεσματικός χειρισμός σας των επειγόντων περιστατικών, σας απέσπασαν αυτό το ξεχωριστό βραβείο. Καλή δουλειά!",
      [2] = "Ο χειρισμός σας των επειγόντων περιστατικών είναι εξαιρετικός! Αυτό το βραβείο είναι γιατί είστε ο απόλυτος χειριστής μεγάλων εισροών ασθενών και ταλαίπωρων.",
    },
    penalty = {
      [1] = "Ο χειρισμός σας των επειγόντων περιστατικών είναι ανεπαρκής. Οι εισερχόμενοι ασθενείς αξίζουν άμεση και σωστή προσοχή, την οποία αποτύχατε να τους δώσετε.",
    },
    regional_good = {
      [1] = "Το Υπουργείο Υγείας αναγνωρίζει ότι το νοσοκομείο σας χειρίστηκε επείγοντα περιστατικά καλύτερα από κάθε άλλο νοσοκομείο, πέρσι και σας παρουσιάζει αυτό το βραβείο.",
    },
    regional_bad = {
      [1] = "Το νοσοκομείο σας ήταν το χειρότερο στην περιοχή στον χειρισμό επειγόντων περιστατικών. Εσείς φταίτε που μαραζώνετε στον πάτο του τοπικού συνδέσμου πρώτων βοηθειών.",
    },
  },
  happy_staff = {
    awards = {
      [1] = "Το προσωπικό σας σάς παρουσιάζει αυτό το βραβείο. Λένε ότι υπάρχει χώρος για βελτίωση, αλλά η συμπεριφορά σας προς αυτούς ήταν γενικά καλή.",
      [2] = "Το προσωπικό σας είναι τόσο χαρούμενο που δουλεύει για εσάς, που δεν μπορούν να σταματήσουν να χαμογελάνε. Είστε ένας υπέροχος διευθυντής!",
    },
    regional_good = {
      [1] = "Το προσωπικό σας είναι πιο χαρούμενο από οποιουδήποτε άλλου νοσοκομείου. Χαρούμενο προσωπικό σημαίνει πιο πολύ κέρδος και λιγότεροι θάνατοι. Το Υπουργείο Υγείας είναι ευχαριστημένο.",
    },
    penalty = {
      [1] = "Το προσωπικό σας θέλει να σας γνωστοποιήσει ότι είναι πολύ δυσαρεστημένοι. Το καλό προσωπικό είναι προσόν. Κάντε το δικό σας πιο χαρούμενο, αλλιώς μια μέρα θα χάσετε τα πάντα μια μέρα.",
    },
    regional_bad = {
      [1] = "Ολόκληρο το προσωπικό σας είναι πολύ θλιμμένο το περασμένο έτος. Πρέπει να το έχετε παρατηρήσει. Όπως έχουν τα πράγματα, κάθε άλλο νοσοκομείο έχει πιο χαρούμενο προσωπικό από το δικό σας.",
    },
    trophies = {
      [1] = "Σας απονέμεται το Χαμογελαστό Βραβείο, καθώς κρατήσατε το σκληρά εργαζόμενο προσωπικό σας όσο πιο χαρούμενο γίνεται.",
      [2] = "Το Ινστιτούτο Χαράς Αδάμη σας ανταμείβει, καθώς δεν έχετε καθόλου λυπημένο προσωπικό στο νοσοκομείο σας πέρσι, δίνοντάς σας αυτό το βραβείο.",
      [3] = "Αυτό το Βραβείο, το Χαμογέλα-πιο-πλατιά Κύπελλο σας απονέμεται, καθώς κρατήσατε όλο σας το προσωπικό χαρούμενο με υπερβάλλοντα ζήλο πέρσι. Παντού γύρω σας υπάρχουν χαμόγελα!",
    },
  },
  happy_patients = {
    awards = {
      [1] = "Μπορείτε να καυχηθείτε για το γεγονός ότι ο κόσμος στο νοσοκομείο σας ήταν πολύ ευχαριστημένος τον τελευταίο χρόνο.",
      [2] = "Τα άτομα που επισκέπτονται το νοσοκομείο σας έμειναν κάτα μέσο όρο πιο ικανοποιημένα κατά την θεραπεία συγκριτικά με οποιοδήποτε άλλο νοσοκομείο στο παιχνίδι.",
    },
    penalty = {
      [1] = "Ο κόσμος που έρχεται στο νοσοκομείο σας βρίσκει τραγική την εμπειρία. Θα πρέπει να τα πάτε πολύ καλύτερα αν θέλετε να κερδίσετε τον σεβασμό του Υπουργείου.",
      [2] = "Τα άτομα που λαμβάνουν θεραπεία στο νοσοκομείο σας ήταν πολύ δυσαρεστημένοι για την κατάσταση του μέρους. Πρέπει να σκεφτείτε πιο διεξοδικά την πρόνοια των ασθενών.",
    },
  },
  curesvdeaths = {
    awards = {
      [1] = "Θερμά συγχαρητήρια που κατορθώσατε μια εντυπωσιακή αναλογία θεραπειών-θανάτων στο νοσοκομείο σας τον τελευταίο χρόνο.",
    },
    penalty = {
      [1] = "Έχετε πολύ κακή αναλογία θεραπειών προς θανάτους. Πρέπει να σιγουρέψετε οτι θεραπεύετε πολύ περισσότερο κόσμο απ' όσους αφήνετε να πεθαίνουν. Μην απογοητεύσετε τους συναδέλφους σας.",
    },
  },
  rats_killed = {
    trophies = {
      [1] = "Κερδίσατε το Βραβείο Κατά των Παρασίτων, επειδή πυροβολήσατε %d αρουραίους στο νοσοκομείο σας το επρασμένο έτος.",
      [2] = "Έχετε τα προσόντα να λάβετε αυτό το βραβείο από την Ομοσπονδία Εναντίον των Αρουραίων και των Ποντικιών, λόγω των εξαιρετικών ικανοτήτων σας να πυροβολείτε αρουραίους, σκοτώνοντας %d από αυτούς.",
      [3] = "Έχετε τα προσόντα να λάβετε το Βραβείο Ανατίναξης Αρουραίων, λόγω των εξαιρετικών ικανοτήτων σας να ξεκάνετε %d αρουραίους στο νοσοκομείο σας το περασμένο έτος.",
    },
  },
  best_value_hosp = {
    penalty = {
      [1] = "Κάθε νοσοκομείο στην γύρω περιοχή αξίζει περισσότερο από το δικό σας. Κάντε κάτι γι' αυτήν την ντροπιαστική κατάσταση. Εισάγετε μερικά πιο ακριβά πράγματα!",
    },
    trophies = {
      [1] = "Συγχαρητήρια για την παραλαβή του Βραβείου Πεντακαθαριότητας για το νοσοκομείο με την καλύτερη φήμη το περασμένο έτος. Το αξίζετε πραγματικά!",
    },
    regional = {
      [1] = "Συγχαρητήρια που έχετε το πιο πολύτιμο νοσοκομείο στο παιχνίδι. Καλή δουλειά. Προσπαθήστε παρακαλώ να συνεχίσετε έτσι.",
    },
  },
  healthy_plants = {
    awards = {
      [1] = "Συγχαρητήρια για τη νίκη του βραβείου Μεγάλωσε-Πιο-Πολύ Λίπασμα, επειδή κρατήσατε τα φυτά σας εξαιρετικά υγιή κατά τη διάρκεια του περασμένου έτους.",
    },
    trophies = {
      [1] = "Οι φίλοι των Φυτών Γλάστρας θα ήθελαν να σας απονέμουν το Πράσινο Βραβείο Υγείας, επειδή κρατήσατε όλα τα φυτά σας σε καλή κατάσταση κατά τη διάρκεια του τελευταίου δωδεκαμήνου.",
      [2] = "Τα Τρομερά Τριφίδια θα ήθελαν να σας προσφέρουν το Βραβείο Πρασινοδάχτυλου, επειδή κρατήσατε τα φυτά του νοσοκομείου σας στο καλύτερο επίπεδο υγείας το περασμένο έτος.",
    },
  },
  all_cured = {
    awards = {
      [1] = "Συγχαρητήρια για τη νίκη του Βραβείου Μαρί Κιουρί, λόγω της επιτυχούς θεραπείας όλων των ασθενών στο νοσοκομείο σας το περασμένο έτος.",
    },
    trophies = {
      [1] = "Το Διεθνές Ίδρυμα Θεραπείας θα ήθελε τιμήσει το γεγονός ότι θεραπεύσατε όλους στο νοσοκομείο σας το περασμένο έτος, απονέμοντάς σας το Βραβείο Θεραπείας των Πάντων.",
      [2] = "Σας απονέμεται το Βραβείο Μη-άρρωστου Γαργάρα, επειδή καταφέρατε να θεραπεύσετε κάθε ασθενή στο νοσοκομείο σας το περασμένο έτος.",
    },
  },
  sold_drinks = {
    trophies = {
      [1] = "Η Παγκόσμια Ένωση Οδοντιάτρων είναι περήφανη να σας απονείμει αυτό το τρόπαιο και την τιμητική πλακέτα επειδή πουλήσατε πολλά αναψυκτικά στο νοσοκομείο σας.",
      [2] = "Έχει απονεμηθεί στο νοσοκομείο σας το τρόπαιο Fizzy-Bizz για λιανική πώληση αναψυκτικών ως αναγνώριση του αριθμόυ των αναψυκτικών που πουλήθηκαν στο νοσοκομείο σας τον τελευταίο χρόνο.",
      [3] = "Εκ μέρους της DK Fillings Inc, σας απονέμεται αυτό το επικαλυμμένο από σοκολάτα τρόπαιο για να τιμήσει τον εξαιρετικό αριθμό αναψυκτικών που πουλήθηκαν στο νοσοκομείο σας αυτόν τον χρόνο.",
    },
  },
  consistant_rep = {
    trophies = {
      [1] = "Δια του παρόντος σας απονέμεται το βραβείο του Υπουργού της Κυβερνήσεως για τα πιο εξαιρετικά standards και την υψηλότερη δυνατή φήμη φέτος. Ωραία.",
      [2] = "Συγχαρητήρια για την παραλαβή του τρόπαιου SqueakiKlean για το νοσοκομείο με την καλύτερη φήμη τον τελευταίο χρόνο. Και το αξίζετε με το παραπάνω.",
    },
  },
  reputation = "ΦΗΜΗ",
  cleanliness = {
    award = {
      [1] = "Οι επιθεωρητές επισημαίνουν ότι το νοσοκομείο σας είναι πολύ καθαρό. Τα καθαρά νοσοκομεία είναι ασφαλή νοσοκομεία. Συνεχίστε την καλή δουλειά.",
    },
    regional_good = {
      [1] = "Το νοσοκομείο σας έχει σημειωθεί ως ένα από τα λιγότερο καθαρά στην περιοχή. Ένα βρώμικο νοσοκομείο είναι βρωμερό και επικίνδυνο. Πρέπει να δίνετε περισσότερη προσοχή στο χάλι.",
    },
    regional_bad = {
      [1] = "Το νοσοκομείο σας είναι το πιο βρώμικο στην περιοχή. Κάθε άλλος Διαχειριστής έχει καταφέρει να έχει καθαρότερους διαδρόμους από εσάς. Είστε ντροπή για το ιατρικό επάγγελμα.",
    },
  },
  hosp_value = {
    penalty = {
      [1] = "Το νοσοκομείο σας έχει αποτύχει να αποκτήσει καλή αξία. Έχετε λάβει κακές αποφάσεις όσον αφορά την διαχείριση των χρημάτων. Να θυμάστε, ένα καλό νοσοκομείο είναι επίσης ένα ακριβό νοσοκομείο.",
    },
    awards = {
      [1] = "Το Υπουργείο Υγείας θα ήθελε να πάρει την ευκαρία να σας συγχαρεί για την εντυπωσιακή γενική αξία του νοσοκομείου σας.",
    },
    regional = {
      [1] = "Είστε μια οικονομική ιδιοφυία. Το νοσοκομείο σας έχει μεγαλύτερη αξία από όλα τα υπόλοιπα νοσοκομεία της περιοχής μαζί.",
    },
  },
  pop_percentage = {
    awards = {
      [1] = "Σημειώστε πως κερδίσατε ένα μεγάλο μερίδιο του πληθυσμού της πόλης ως επισκέπτες στο νοσοκομείο σας τον τελευταίο χρόνο. Μπράβο σας.",
      [2] = "Συγχαρητήρια. Ένα μεγαλύτερο μέρος του τοπικού πληθυσμού επισκέπτεται το νοσοκομείο σας συγκριτικά με οποιαδήποτε από τα υπόλοιπα νοσοκομεία.",
      [3] = "Έξοχα. Έχετε προσελκύσει περισσότερο κόσμο στον νοσοκομείο σας από ό,τι έχουν επισκεφτεί όλα τα άλλα νοσοκομεία μαζί.",
    },
    penalty = {
      [1] = "Πολύ λίγος κόσμος επισκέφθηκε το νοσοκομείο σας φέτος. Για να βγάλετε χρήματα, η επιχείρησή σας χρειάζεται παθόντες να τα ακουμπήσουν χοντρά.",
      [2] = "Κάθε νοσοκομείο στην περιοχή κατόρθωσε να έχει μεγαλύτερη μερίδα του πληθυσμού να το επισκέπτεται, συγκριτικά με εσάς. Θα έπρεπε να ντρέπεστε πολύ.",
    },
  },
  rats_accuracy = {
    trophies = {
      [1] = "Σας απονέμεται το βραβείο Syndicate Wars Accurate Shooting Trophy επειδή είχατε %d%% ακρίβεια στο να πυροβολείτε αρουραίους.",
      [2] = "Αυτό το τρόπαιο έχει σκοπό να τιμήσει την απίστευτη ακρίβειά σας, καθώς σκοτώσατε το %d%% των αρουραίων που σημαδέψατε στο νοσοκομείο σας τον περασμένο χρόνο.",
      [3] = "Για να τιμηθεί η περίσταση της θανάτωσης του %d%% όλων των αρουραίων στο νοσοκομείο σας, σας απονέμεται το βραβείο Dungeon Keeper Vermin-Free. CongRATulations!",
    },
  },
  many_cured = {
    trophies = {
      [1] = "Συγχαρητήρια για τη βράβευσή σας με το Βραβείο Μαρί Κιουρί, λόγω της επιτυχούς θεραπείας σχεδόν όλων των ασθενών του νοσοκομείου σας πέρσι.",
      [2] = "Το Διεθνές Ίδρυμα Θεραπείας θα ήθελε τιμήσει το γεγονός ότι θεραπεύσατε έναν μεγάλο αριθμό ανθρώπων στο νοσοκομείο σας πέρσι, απονέμοντάς σας το Βραβείο Θεραπείας των Πάντων..",
      [3] = "Σας απονέμεται το Βραβείο Μη-άρρωστου Γαργάρα, επειδή καταφέρατε να θεραπεύσετε ένα τεράστιο ποσοστό των ασθενών του νοσοκομείου σας το περασμένο έτος.",
    },
    penalty = {
      [1] = "Το νοσοκομείο σας αποτυγχάνει να δώσει αποτελεσματικές θεραπείες στους ασθενείς που τις χρειάζονται. Συγκεντρωθείτε στις θεραπείες σας, ώστε να τις κάνετε πιο αποτελεσματικές.",
      [2] = "Το νοσοκομείο σας είναι λιγότερο αποτελεσματικό στην θεραπεία ασθενών από κάθε άλλο. Απογοητεύσατε το Υπουργείο Υγείας και τον εαυτό σας. Δεν θα πούμε τίποτε άλλο.",
    },
    awards = {
      [1] = "Συγχαρητήρια που θεραπεύσατε ένα σωρό ανθρώπους το περασμένο έτος. Πολλοί άνθρωποι νιώθουν πολύ καλύτερα, χάρη στη δουλειά σας.",
      [2] = "Παρακαλώ δεχθείτε αυτό το βραβείο, επειδή θεραπεύσατε περισσότερους ασθενείς από κάθε άλλο νοσοκομείο. Άψογη επίδοση!",
    },
    regional = {
      [1] = "Διά του παρόντος σας απονέμεται το Βραβείο Απόλυτης Θεραπείας, επειδή θεραπεύσατε περισσότερους ανθρώπους  από όλα τα υπόλοιπα νοσοκομεία μαζί.",
    },
  },
  high_rep = {
    penalty = {
      [1] = "Θα επιπλαγείτε, επειδή διατηρήσατε πολύ κακή φήμη κατά το περασμένο έτος. Βεβαιωθείτε ότι θα βελτιωθεί στο μέλλον.",
      [2] = "Η φήμη του νοσοκομείου σας είναι η χειρότερη στην περιοχή. Ντροπή σας! Βελτιωθείτε ή αλλάξτε επάγγελμα.",
    },
    awards = {
      [1] = "Μπράβο! Κερδίσατε ένα μικρούλι βραβείο, επειδή η φήμη σας πέρσι ήταν κάπως εντυπωσιακή.",
      [2] = "Θαυμάσια! Το νοσοκομείο σας κέρδισε ένα βραβείο, επειδή χτίσατε την καλύτερη φήμη κατά το περασμένο έτος.",
    },
    regional = {
      [1] = "Παρακαλώ αποδεχθείτε το Βραβείο Βουβαλοβατράχου για την υψηλότερη φήμη το περασμένο έτος για κάθε νοσοκομείο. Απολαύστε το - το δικαιούστε!",
      [2] = "Αυτό το έτος, η φήμη του νοσοκομείου σας έχει ξεπεράσει αυτήν όλων των άλλων νοσοκομείων μαζί. Ένα σημαντικό επίτευγμα!",
    },
  },
  wait_times = {
    award = {
      [1] = "Συγχαρητήρια! Το νοσοκομείο σας έχει συστηματικά πολύ χαμηλούς χρόνους αναμονής. Αυτό είναι ένα σημαντικό βραβείο!",
    },
    penalty = {
      [1] = "Οι άνθρωποι στο νοσοκομείο σας πρέπει να περιμένουν πολλή ώρα. Πάντα υπάρχουν απαράδεκτες ουρές. Θα μπορούσατε να διαχειριστείτε τους ασθενείς σας πιο αποδοτικά, αν θέλατε.",
    },
  },
}
transactions = {
  epidemy_fine = "Πρόστιμο Επιδημίας",
  general_bonus = "Γενική Πληρωμή Μπόνους",
  wages = "Μισθοί",
  compensation = "Κρατική Αποζημείωση",
  jukebox = "Έσοδα: Jukebox",
  heating = "Έξοδα Θέρμανσης",
  severance = "Αποζημείωση Λόγω Απόλυσης",
  treat_colon = "Θεραπεία:",
  drug_cost = "Κόστος Φαρμάκου",
  eoy_trophy_bonus = "End of year Trophy Bonus",
  vip_award = "Χρηματικό Βραβείο VIP",
  drinks = "Έσοδα: Αναψυκτήρια",
  machine_replacement = "Κόστος Αντικατάστασης Μηχανήματος",
  research = "Έξοδα Έρευνας",
  bank_loan = "Δάνειο Τράπεζας",
  buy_object = "Αγορά Αντικειμένου",
  vaccination = "Εμβολιασμός",
  build_room = "Χτίσιμο Δωματίου",
  sell_object = "Πώληση Αντικειμένου",
  cure_colon = "Θεραπεία:",
  personal_bonus = "Προσωπική Πληρωμή Μπόνους",
  research_bonus = "Μπόνους Έρευνας",
  epidemy_coverup_fine = "Πρόστιμο Συγκάληψης Επιδημίας",
  cheat = "Cheat Money",
  loan_repayment = "Αποπληρωμή Δανείου",
  eoy_bonus_penalty = "End of year Μπόνους/Ποινή",
  final_treat_colon = "Τελική Θεραπεία:",
  emergency_bonus = "Μπόνους Πληρωμή Έκτακτης Ανάγκης",
  loan_interest = "Τόκος Δανείου",
  insurance_colon = "Ασφάλιση:",
  advance_colon = "Προκαταβολή:",
  deposit = "Κατάθεση Αγωγής",
  hire_staff = "Πρόσληψη Προσωπικού",
  buy_land = "Αγορά Γης",
  overdraft = "Τόκος Υπερανάληψης",
  cure = "Θεραπεία",
}
custom_game_window = {
  free_build = "Free Build",
  caption = "Προσαρμοσμένο Παιχνίδι",
  load_selected_level = "Εκκίνηση",
}
load_game_window = {
  caption = "Φόρτωση Παιχνιδιού (%1%)",
}
install = {
  title = "--------------------------------- CorsixTH Setup ---------------------------------",
  ok = "OK",
  th_directory = "Το CorsixTH χρειάζεται ένα αντίγραφο των δεδομένων από το αυθεντικό Theme Hospital (ή το demo) για να τρέξει. Παρακαλώ χρησιμοποιείστε τον παρακάτω selector για να εντοπίσετε την διαδρομή εγκατάστασης του Theme Hospital.",
  cancel = "Ακύρωση",
  exit = "Έξοδος",
}
hotkey_window = {
  panel_recallPosKeys = "Πλήκτρα Επαναφοράς Θέσης",
  ingame_panel_staffManage = "Διαχείριση Προσωπικού",
  ingame_recallPosition_3 = "3",
  button_defaults = "Επαναφορά Προεπιλογών",
  ingame_quickLoad = "Γρήγορη Φόρτωση",
  ingame_recallPosition_2 = "2",
  ingame_showmenubar = "Εμφάνισης Μπάρας Μενού",
  ingame_recallPosition_6 = "6",
  ingame_patient_gohome = "Αποστολή στο Σπίτι",
  ingame_storePosition_2 = "2",
  ingame_panel_buildRoom = "Χτίσιμο Δωματίου",
  global_connectDebugger = "Debugger",
  ingame_panel_furnishCorridor = "Επίπλωση Διαδρόμου",
  global_resetApp = "Επαναφορά Εφαρμογής",
  panel_debugKeys = "Debug Keys",
  ingame_panel_research = "Έρευνα",
  global_fullscreen_toggle = "Πλήρης οθόνη",
  ingame_panel_townMap = "Χάρτης Πόλης",
  ingame_recallPosition_8 = "8",
  ingame_zoom_in_more = "Περισσότερο Zoom In",
  global_screenshot = "Screenshot",
  ingame_recallPosition_1 = "1",
  ingame_loadMenu = "Φόρτωση Παιχνιδιού",
  ingame_scroll_right = "Κύλιση Δεξιά",
  ingame_setTransparent = "Διαφάνεια",
  ingame_gamespeed_normal = "Normal",
  ingame_toggleAdvisor = "Βοηθός",
  ingame_storePosition_0 = "10",
  ingame_storePosition_3 = "3",
  panel_gameSpeedKeys = "Πλήκτρα Ταχύτητας Παιχνιδιού",
  panel_globalKeys = "Global Keys",
  global_runDebugScript = "Debug Script",
  global_releaseMouse = "Απελευθέρωση Ποντικιού",
  ingame_recallPosition_7 = "7",
  button_accept = "Αποδοχή",
  ingame_gamespeed_slowest = "Το πιο αργό",
  ingame_panel_map_alt = "Χάρτης Πόλης 2",
  global_cancel = "Ακύρωση",
  ingame_zoom_in = "Zoom In",
  button_toggleKeys = "Toggle Keys",
  panel_altPanelKeys = "Alternate Panel Keys",
  global_stop_movie_alt = "Stop Movie",
  ingame_scroll_left = "Κύλιση Αριστερά",
  ingame_rotateobject = "Περιστροφή Αντικειμένου",
  global_confirm = "Επιβεβαίωση",
  ingame_zoom_out = "Zoom Out",
  ingame_showCheatWindow = "Μενού Cheat",
  ingame_jukebox = "Jukebox",
  ingame_saveMenu = "Αποθήκευση Παιχνιδιού",
  ingame_scroll_shift = "Αλλαγή Ταχύτητας",
  ingame_openFirstMessage = "Μήνυμα Επιπέδου",
  ingame_recallPosition_9 = "9",
  global_cancel_alt = "Cancel Alt",
  ingame_storePosition_6 = "6",
  ingame_panel_research_alt = "Έρευνα 2",
  ingame_recallPosition_4 = "4",
  ingame_pause = "Pause",
  ingame_scroll_down = "Κύλιση Κάτω",
  ingame_gamespeed_slower = "Πιο αργά",
  ingame_storePosition_9 = "9",
  ingame_gamespeed_max = "Max",
  ingame_scroll_up = "Κύλιση Πάνω",
  ingame_restartLevel = "Επανεκκίνηση Επιπέδου",
  ingame_quickSave = "Γρήγορη Αποθήκευση",
  ingame_panel_casebook = "Casebook",
  global_window_close_alt = "Κλείσιμο Παραθύρου",
  ingame_panel_policy = "Πολιτική",
  ingame_panel_bankManager = "Διευθυντής Τραπέζης",
  ingame_zoom_out_more = "Περισσότερο Zoom Out",
  ingame_panel_bankStats = "Κατάσταση Τραπέζης",
  ingame_panel_status = "Κατάσταση",
  ingame_panel_charts = "Γραφήματα",
  ingame_panel_editRoom = "Επεξεργασία Δωματίου",
  ingame_toggleSounds = "Ήχοι",
  panel_toggleKeys = "Πλήκτρα Toggle",
  panel_generalInGameKeys = "Γενικά In-Game Πλήκτρα",
  global_showLuaConsole = "Κονσόλα Lua",
  ingame_poopLog = "Dump Log",
  global_confirm_alt = "Confirm Alt",
  ingame_poopStrings = "Dump Strings",
  ingame_gamespeed_speedup = "Επιτάχυνση",
  global_exitApp = "Έξοδος Εφαρμογής",
  ingame_toggleInfo = "Πληροφορίας",
  caption_main = "Ανάθεση Hotkey",
  panel_storePosKey = "Πλήκτρα Αποθήκευσης Θέσης",
  ingame_panel_hireStaff = "Πρόσληψη Προσωπικού",
  ingame_panel_casebook_alt = "Casebook 2",
  button_cancel = "Ακύρωση",
  button_back = "Πίσω",
  caption_panels = "Πλήκτρα Panel",
  ingame_quitLevel = "Έξοδος Επιπέδου",
  ingame_storePosition_1 = "1",
  ingame_toggleAnnouncements = "Ανακοινώσεις",
  ingame_toggleMusic = "Μουσική",
  ingame_panel_casebook_alt02 = "Casebook 3",
  panel_scrollKeys = "Πλήκτρα Κύλισης",
  ingame_storePosition_8 = "8",
  ingame_recallPosition_0 = "10",
  ingame_gamespeed_thensome = "Κι Άλλο Τόσο",
  ingame_recallPosition_5 = "5",
  ingame_storePosition_4 = "4",
  panel_zoomKeys = "Πλήκτρα Zoom",
  ingame_storePosition_7 = "7",
  button_recallPosKeys = "Πλήκτρα Ανάκλησης Θέσης",
  ingame_storePosition_5 = "5",
}
menu_file_save = {
  [1] = "  ΠΑΙΧΝΙΔΙ 1  ",
  [2] = "  ΠΑΙΧΝΙΔΙ 2  ",
  [3] = "  ΠΑΙΧΝΙΔΙ 3  ",
  [4] = "  ΠΑΙΧΝΙΔΙ 4  ",
  [5] = "  ΠΑΙΧΝΙΔΙ 5  ",
  [6] = "  ΠΑΙΧΝΙΔΙ 6  ",
  [7] = "  ΠΑΙΧΝΙΔΙ 7  ",
  [8] = "  ΠΑΙΧΝΙΔΙ 8  ",
}
level_names = {
  [1] = "ToxiCity",
  [2] = "Sleepy Hollow",
  [3] = "Largechester",
  [4] = "Frimpton-on-Sea",
  [5] = "Simpleton",
  [6] = "Festering-on-the-Wold",
  [7] = "Greenpool",
  [8] = "Manquay",
  [9] = "Eastville",
  [10] = "Eggsenham",
  [11] = "Croaking",
  [12] = "Battenburg",
  [13] = "Chumleigh",
  [14] = "Little Drubbing",
  [15] = "Bury Bury",
}
casebook = {
  cure_desc = {
    hire_psychiatrists = "Πρέπει να προσλάβετε Ψυχίατρους.",
    no_cure_known = "Καμία γνωστή Θεραπεία.",
    hire_surgeons = "Πρέπει να προσλάβετε Χειρουργούς.",
    build_ward = "Πρέπει ακόμα να χτίσετε μια Πτέρυγα.",
    cure_known = "Θεραπεία.",
    improve_cure = "Βελτίωση Θεραπείας.",
    build_room = "Συνιστώ να χτίσετε %s",
    hire_doctors = "Πρέπει να προσλάβετε μερικούς Γιατρούς.",
    hire_nurses = "Πρέπει να προσλάβετε μερικές Νοσοκόμες.",
  },
  research = "συγκέντρωση έρευνας",
  earned_money = "κέρδη",
  reputation = "φήμη",
  deaths = "θάνατοι",
  treatment_charge = "χρέωση αγωγής",
  sent_home = "απορρίφθηκαν",
  cured = "αναρρώσεις",
  cure = "θεραπεία",
}
menu_debug_overlay = {
  byte_floor = "  BYTE FLOOR  ",
  none = "  NONE  ",
  byte_n_wall = "  BYTE N WALL  ",
  parcel = "  PARCEL  ",
  flags = "  FLAGS  ",
  positions = "  POSITIONS  ",
  byte_5 = "  BYTE 5  ",
  byte_0_1 = "  BYTE 0 & 1  ",
  byte_7 = "  BYTE 7  ",
  heat = "  TEMPERATURE  ",
  byte_w_wall = "  BYTE W WALL  ",
  byte_6 = "  BYTE 6  ",
}
menu_options_wage_increase = {
  deny = "    ΑΠΟΡΡΙΨΗ ",
  grant = "    ΧΟΡΗΓΗΣΗ ",
}
tooltip = {
  save_game_window = {
    save_game = "Αντικατάσταση αποθηκευμένου παιχνιδιού %s",
    new_save_game = "Εισάγετε όνομα για νέο αποθηκευμένο παιχνίδι",
  },
  fax = {
    close = "Κλείστε αυτό το παράθυρο χωρίς να διαγραφεί το μήνυμα",
  },
  watch = {
    epidemic = "Επιδημία: ο χρόνος που απομένει για να συγκαλύψετε την επιδημία. Όταν αυτός ο χρόνος λήξει Ή ένας μεταδοτικός ασθενής αποχωρήσει από το νοσοκομείο σας, ένας Επιθεωρητής Υγείας θα σας επισκεφτεί. Το κουμπί κάνει toggle την λειτουργία εμβολιασμού σε on ή off. Κάντε κλικ σε έναν ασθενή για να εμβολιαστεί από μια Νοσοκόμα.",
    emergency = "Επείγον Περιστατικό: ο χρόνος που απομένει για να θεραπεύσετε όλους τους ασθενείς έκτακτης ανάγκης.",
    hospital_opening = "Χρόνος χτισίματος: αυτός είναι ο χρόνος που απομένει πριν το νοσοκομείο σας δηλωθεί ως ανοιχτό. Κάνοντας κλικ στο κουμπί GO θα έχει ως αποτέλεσμα το νοσοκομείο σας να ανοίξει αμέσως.",
  },
  jukebox = {
    rewind = "Rewind jukebox",
    stop = "Παύση jukebox",
    current_title = "Jukebox",
    play = "Αναπαραγωγή jukebox",
    close = "Κλείσιμο jukebox",
    loop = "Loop play jukebox",
    fast_forward = "Fast Forward jukebox",
  },
  handyman_window = {
    face = "Πρόσωπο Πολυτεχνίτη",
    close = "Ακύρωση αιτούντα",
    prio_machines = "Αύξηση προτεραιότητας Πολυτεχνίτη για επιδιόρθωση εξοπλισμού",
    salary = "Μισθός",
    happiness = "Ευτυχία",
    parcel_select = "Το οικόπεδο (περιοχή) στο οποίο ο πολυτεχνίτης αναλαμβάνει δουλειές, κλικ για αλλαγή ρυθμίσεων",
    ability = "Ικανότητα",
    center_view = "Εστίαση σε Πολυτεχνίτη",
    name = "Όνομα Πολυτεχνίτη",
    prio_plants = "Αύξηση προτεραιότητας Πολυτεχνίτη για πότισμα φυτών",
    pick_up = "Αρπάξτε",
    sack = "Απολύστε",
    tiredness = "Κούραση",
    prio_litter = "Αυξήση προτεραιότητας Πολυτεχνίτη για καθαρισμό σκουπιδιών",
  },
  hire_staff_window = {
    nurses = "Προβολή Νοσοκόμων διαθέσιμων προς πρόσληψη",
    hire = "Πρόσληψη ατόμου",
    psychiatrist = "Ψυχίατρος",
    qualifications = "Επιπλέον Προσόντα Γιατρού",
    researcher = "Ερευνητής",
    next_person = "Προβολή επόμενου ατόμου",
    doctor_seniority = "Παλαιότητα Γιατρού (Μαθητευόμενος, Γιατρός, Σύμβουλος)",
    handymen = "Προβολή Πολυτεχνιτών διαθέσιμων προς πρόσληψη",
    surgeon = "Χειρουργός",
    prev_person = "Προβολή προηγούμενου ατόμου",
    salary = "Μισθός",
    staff_ability = "Ικανότητα προσωπικού",
    cancel = "Ακύρωση",
    doctors = "Προβολή Γιατρών διαθέσιμων προς πρόσληψη",
    receptionists = "Προβολή Ρεσεψιονίστ διαθέσιμων προς πρόσληψη",
  },
  information = {
    close = "Κλείσιμο διαλόγου πληροφοριών",
  },
  totd_window = {
    previous = "Εμφάνιση προηγούμενης συμβουλής",
    next = "Εμφάνιση επόμενης συμβουλής",
  },
  bank_manager = {
    close = "Κλείσιμο οθόνης Διευθυντή Τράπεζας",
    graph_return = "Επιστροφή στην προηγούμενη οθόνγ",
    balance = "Το τραπεζικό σας υπόλοιπο",
    inflation_rate = "Ετήσιος ρυθμός πληθωρισμού",
    interest_payment = "Μηνιαίες πληρωμές τόκων",
    hospital_value = "Τρέχουσα αξία του νοσοκομείου σας",
    current_loan = "Τρέχοντα δάνεια που εκκρεμούν",
    repay_5000 = "Αποπληρωμή $5000 στην τράπεζα",
    insurance_owed = "Χρήματα που σας χρωστούνται από %s",
    graph = "Γράφημα αναμενόμενων πληρωμών από %s",
    borrow_5000 = "Δανειστείτε $5000 από την τράπεζα",
    interest_rate = "Ετήσιο επιτόκιο",
    show_graph = "Προβολή γραφήματος αναμενόμενων πληρωμών από %s",
  },
  rooms = {
    research_room = "Οι Γιατροί με ικανότητες Έρευνας μπορούν να ανακαλύψουν νέα φάρμακα και μηχανήματα στο Δωμάτιο Ερευνών",
    hair_restoration = "Ένας Γιατρός χρησιμοποιεί το Δωμάτιο Αποκατάστασης Μαλλιών για να θεραπεύσει την Φαλάκρα",
    blood_machine = "Ένας Γιατρός χρησιμοποιεί το Δωμάτιο Μηχανήματος Αίματος για να διαγνώσει Ασθενείς",
    scanner = "Ένας Γιατρός χρησιμοποιεί το Δωμάτιο Σάρωσης για να διαγνώσει Ασθενείς",
    inflation = "Ένας Γιατρός χρησιμοποιεί το Δωμάτιο Φουσκώματος για να θεραπεύει Ασθενείς με Φουσκωτά Κεφάλια",
    x_ray = "Ένας Γιατρός χρησιμοποιεί το Δωμάτιο Ακτινογραφίας για να διαγνώσει Ασθενείς",
    operating_theatre = "Το Χειρουργείο χρειάζεται δύο Γιατρούς με χειρουργικές δεξιότητες",
    staffroom = "Οι Γιατροί και οι Νοσοκόμες χαλαρώνουν στο δωμάτιο προσωπικού και ξεκουράζονται",
    jelly_vat = "Ένας Γιατρός χρησιμοποιεί την Δεξαμενή Ζελέ για να θεραπεύσει την Ζελετίτιδα",
    psychiatry = "Το Ψυχιατρείο θεραπεύει τρελούς ασθενείς και βοηθάει στην διάγνωση άλλων ασθενών, αλλά χρειάζεται έναν Γιατρό με Ψυχιατρικές δεξιότητες",
    toilets = "Χτίστε Τουαλέτες για να αποτρέψετε τους Ασθενείς από το να τα κάνουν χάλια στο νοσοκομείο σας!",
    cardiogram = "Ένας Γιατρός χρησιμοποιεί το Δωμάτιο Καρδιογραφήματος για να διαγνώσει Ασθενείς",
    ward = "Οι Πτέρυγες είναι χρήσιμες τόσο για διαγνώσεις όσο και για αγωγές. Οι ασθενείς στέλνονται εδώ για ιατρική παρακολούθηση και επίσης για να αναρρώσουν σε μια κλινή από εγχειρήσεις, με την φροντίδα μιας Νοσοκόμας",
    ultrascan = "Ένας Γιατρός χρησιμοποιεί το Δωμάτιο Ultrascan για να διαγνώσει Ασθενείς",
    tongue_clinic = "Ένας Γιατρός χρησιμοποιεί το Δωμάτιο Τεμαχίσματος Γλώσσας για να θεραπεύσει Χαλαρές Γλώσσες",
    pharmacy = "Η Νοσοκόμα χορηγεί φάρμακα για να θεραπεύσει ασθενείς σε ένα Φαρμακείο",
    fracture_clinic = "Μια Νοσοκόμα χρησιμοποιεί την Κλινική Καταγμάτων για να επιδιορθώσει Σπασμένα Κόκκαλα",
    dna_fixer = "Ένας Γιατρός χρησιμοποιεί το Δωμάτιο Επιδιορθωτή DNA για να θεραπεύσει ασθενείς με Εξωγήινο DNA",
    general_diag = "Ένας Γιατρός χρησιμοποιεί το Δωμάτιο Γενικής Διάγνωσης για βασική διάγνωση των ασθενών. Φθηνή και πολλές φορές πολύ αποτελεσματική",
    gps_office = "Οι Ασθενείς λαμβάνουν την αρχική τους γνωμάτευση και τα αποτελέσματα στο Γραφείο του Γενικού Ιατρού",
    decontamination = "Ένας Γιατρός χρησιμοποιεί το Ντουζ Απολύμανσης για να θεραπεύσει την Ραδιενέργεια",
    training_room = "Ένα δωμάτιο Εκπαίδευσης μαζί με έναν Σύμβουλο Ιατρό μπορεί να χρησιμοποιηθεί για την εκπαίδευση περαιτέρω Γιατρών",
    electrolysis = "Ένας Γιατρός χρησιμοποιεί το δωμάτιο Ηλεκτρόλυσης για να θεραπεύσει την Μαλλίτιδα",
  },
  queue_window = {
    patient_dropdown = {
      hospital_2 = "Παραπομπή ασθενούς σε άλλο νοσοκομείο",
      reception = "Αποστολή ασθενή στην Ρεσεψιονίστ",
      send_home = "Δώστε στον ασθενή εξιτήριο από το νοσοκομείο",
      hospital_1 = "Παραπομπή ασθενούς σε άλλο νοσοκομείο",
      hospital_3 = "Παραπομπή ασθενούς σε άλλο νοσοκομείο",
    },
    num_entered = "Αριθμός ασθενών οι οποίοι έχουν λάβει ιατρική αγωγή σε αυτό το δωμάτιο μέχρι τώρα",
    max_queue_size = "Μέγιστο μέγεθος ουράς που μπορεί η Ρεσεψιονίστ να αποπειραθεί να επιτρέψει",
    num_in_queue = "Αριθμός ασθενών στην ουρά",
    dec_queue_size = "Μείωση μέγιστου μεγέθους ουράς",
    inc_queue_size = "Αύξηση μέγιστου μεγέθους ουράς",
    front_of_queue = "Σύρετε έναν ασθενή σε αυτό το εικονίδιο για να τους μεταφέρετε στην αρχή της ουράς",
    num_expected = "Αριθμός ασθενών που η Ρεσεψιονίστ αναμένει να έρθουν στην ουρά προσεχώς",
    patient = "Σύρετε έναν ασθενή για να αλλάξετε την θέση του στην ουρά. Κάντε δεξί κλικ σε έναν ασθενή για να τον στείλετε στο σπίτι ή να τον παραπέμψετε σε ένα αντίπαλο νοσοκομείο",
    end_of_queue = "Σύρετε έναν ασθενή σε αυτό το εικονίδιο για να τον μετακινήσετε στο τέλος της ουράς",
    close = "Κλείσιμο αυτού του παραθύρου",
  },
  update_window = {
    download = "Πηγαίντε στην σελίδα λήψης για την τελευταία έκδοση του CorsixTH",
    ignore = "Αγνοήστε αυτήν την ενημέρωση για τώρα. Θα ειδοποιηθείτε ξανά όταν ξανανοίξετε το CorsixTH",
  },
  staff_list = {
    tiredness = "Δείχνει πόσο κουρασμένοι είναι αυτοί οι υπάλληλοι",
    salary = "Τρέχοντας μισθός αυτού του ατόμου",
    researcher_train = "Ο %d%% εκπαιδεύτηκε για να γίνει Ερευνητής",
    receptionists = "Προβολή λίστας Ρεσεψιονίστ που εργάζονται στο νοσοκομείο",
    surgeon = "Πιστοποιημένος Χειρουργός",
    psychiatrist = "Πιστοποιημένος Ψυχίατρος",
    pay_rise = "Αύξηση του μισθού αυτού του ατόμου κατά 10%",
    prev_person = "Προβολή προηγούμενης σελίδας",
    next_person = "Προβολή επόμενης σελίδας",
    tiredness_2 = "Κούραση Υπαλλήλου",
    bonus = "Πληρώστε αυτό το άτομο μπόνους 10%",
    nurses = "Προβολή λίστας Νοσοκόμων που εργάζονται στο νοσοκομείο",
    close = "Έξοδος και επιστροφή στο παιχνίδι",
    skills = "Επιπλέον δεξιότητες",
    researcher = "Πιστοποιημένος Ερευνητής",
    ability_2 = "Επίπεδο Ικανότητας Υπαλλήλου",
    happiness = "Δείχνει πόσο ικανοποιημένοι είναι αυτοί οι υπάλληλοι",
    psychiatrist_train = "Ο %d%% εκπαιδεύτηκε για να γίνει Ψυχίατρος",
    doctor_seniority = "Παλαιότητα Γιατρού",
    view_staff = "Προβολή ατόμου εν ώρα εργασίας",
    handymen = "Προβολή λίστας Πολυτεχνιτών που εργάζονται στο νοσοκομείο",
    happiness_2 = "Ηθικό Υπαλλήλου",
    surgeon_train = "Ο %d%% εκπαιδεύτηκε για να γίνει Χειρουργός",
    doctors = "Προβολή λίστας Γιατρών που εργάζονται στο νοσοκομείο",
    sack = "Απολύστε αυτό το άτομο",
    detail = "Προσοχή στις Λεπτομέρειες",
    ability = "Προβολή του επιπέδου ικανοτήτων αυτών των υπαλλήλων",
  },
  objects = {
    desk = "Γραφείο: απαραίτητο για να τοποθετούν επάνω του οι Γιατροί τον υπολογιστή τους.",
    toilet_sink = "Νιπτήρας: οι ασθενείς σας που νοιάζονται παραπάνω για την προσωπική τους υγιεινή μπορούν να πλύνουν εδώ τα λιγδιασμένα χέρια τους. Αν δεν υπάρχουν αρκετοί νιπτήρες, οι ασθενείς γίνονται δυσαρεστημένοι.",
    fire_extinguisher = "Πυροσβεστήρας: ελαχιστοποιεί τον κίνδυνο από τυχόν δυσλειτουργικό εξοπλισμό.",
    skeleton = "Σκελετός: χρησιμοποιείται για εκπαιδευτικούς σκοπούς για το Halloween.",
    scanner = "14 OB_SCANNER",
    x_ray = "27 OB_XRAY",
    litter_bomb = "Βόμβα Σκουπιδιών: σαμποτόρατε τα νοσοκομεία των αντιπάλων",
    rathole = "Σπίτι μιας οικογένειας αρουραίων που βρήκε το νοσοκομείο σας αρκετά βρώμικο ώστε να ζήσει εδώ.",
    video_game = "Βιντεοπαιχνίδι: αφήστε το προσωπικό σας να ξεκουραστεί με το Hi-Octane.",
    electrolyser = "46 OB_ELECTRO",
    litter = "Σκουπίδι: το πέταξε στο πάτωμα ασθενής γιατί δεν βρήκε έναν κάδο να το πετάξει.",
    swing_door2 = "53 OB_DOUBLE_DOOR2",
    bookcase = "Βιβλιοθήκη: βιβλιογραφική αναφορά για Γιατρούς.",
    couch = "18 OB_COUCH",
    lamp = "Λάμπα: εκπέμπει φως και βοηθάει το προσωπικό να δει.",
    x_ray_viewer = "29 OB_XRAY_VIEWER",
    bed3 = "δεν χρησιμοποιείται",
    chair = "Καρέκλα: οι ασθενείς κάθονται εδώ και συζητάνε σχετικά με τις ασθένειές τους.",
    bench = "Πάγκος: προσφέρει κάθισμα για έναν ασθενή, καθιστώντας πια άνετη την αναμονή.",
    operating_table = "30 OB_OP_TABLE",
    crash_trolley = "20 OB_CRASH",
    entrance_right = "59 OB_ENT_RDOOR",
    inflator = "Φουσκωτής: θεραπεύει άτομα με Φουσκωτό Κεφάλι.",
    op_sink1 = "33 OB_OP_SINK_1",
    radiator = "Καλοριφέρ: κρατάει το νοσοκομείο σας ζεστό.",
    drinks_machine = "Αναψυκτήριο: αποτρέπει τους ασθενείς από το να διψάσουν και προσφέρει εισόδημα.",
    cast_remover = "24 OB_CAST_REMOVE",
    ultrascanner = "22 OB_ULTRASCAN",
    hair_restorer = "25 OB_HAIR_RESTORER",
    dna_fixer = "23 OB_DNA_FIXER",
    plant = "Φυτό: κρατάει τους ασθενείς χαρούμενος και εξαγνίζει τον αέρα.",
    slicer = "26 OB_SLICER",
    auto_autopsy = "Αυτοψία: βοηθάει εξαιρετικά στην ανακάλυψη νέων θεραπειών",
    computer = "Υπολογιστής: απαραίτητο εργαλείο έρευνας",
    projector = "37 OB_PROJECTOR",
    radiation_shield = "28 OB_RAD_SHIELD",
    op_sink2 = "34 OB_OP_SINK_2",
    table2 = "12 OB_BTABLE",
    jelly_moulder = "47 OB_JELLY_VAT",
    cardio = "13 OB_CARDIO",
    blood_machine = "42 OB_BLOOD_MC",
    door = "Πόρτα: ο κόσμος τις ανοίγει και τις κλείνει πολύ συχνά.",
    pool_table = "Τραπέζι μπιλιάρδου: βοηθάει το προσωπικό σας να χαλαρώσει.",
    cabinet = "Αρχειοθήκη: περιέχει τα αρχεία των ασθενών, σημειώσεις και έγγραφα ερευνών.",
    shower = "54 OB_DECON_SHOWER",
    surgeon_screen = "35 OB_SURGEON_SCREEN",
    gates_of_hell = "48 OB_HELL",
    table1 = "Τραπέζει (ΕΧΕΙ ΑΦΑΙΡΕΘΕΙ): φιλοξενεί συλλογές περιοδικών για να κρατήσουν χαρούμενους τους ασθενείς που περιμένουν.",
    bed2 = "δεν χρησιμοποιείται",
    sofa = "Καναπές: Οι Υπάλληλοι που ξεκουράζονται στο Δωμάτιο Προσωπικού θα κάτσουν ήσυχα σε έναν καναπέ εκτός και αν υπάρχει κάποιος καλύτερος τρόπος χαλάρωσης.",
    comfortable_chair = "61 OB_COMFY_CHAIR",
    console = "15 OB_SCANNER_CONSOLE",
    atom_analyser = "Χημικό μίξερ: αυτό το μηχάνημα που βρίσκεται στο Τμήμα Ερευνών, επιταχύνει την όλη διαδικασία της έρευνας.",
    tv = "TV: σιγουρέψτε ότι το προσωπικό σας δεν θα χάσει τα αγαπημένα του προγράμματα.",
    bed = "Κρεβάτι: οι πολύ άρρωστοι άνθρωποι ξαπλώνουν σε αυτά.",
    pharmacy_cabinet = "Φαρμακείο: εκεί όπου φυλάσσονται τα φάρμακα",
    toilet = "Τουαλέτα: ασθενείς.. χρησιμοποιήστε την.",
    lecture_chair = "Καρέκλα διάλεξης: οι μαθητευόμενοι Γιατροί σας κάθονται εδώ για να πάρουν σημειώσεις, να βαρεθούν και να κάνουν μουτζούρες. Όσο περισσότερες καρέκλες τοποθετείτε, τόσο μεγαλύτερο μπορεί να είναι το μέγεθος της τάξης σας.",
    screen = "16 OB_SCREEN",
    bin = "Κάδος: οι ασθενείς πετάνε τα σκουπίδια τους εδώ.",
    swing_door1 = "52 OB_DOUBLE_DOOR1",
    entrance_left = "58 OB_ENT_LDOOR",
    reception_desk = "Γραφείο Ρεσεψιόν: απαιτεί μια Ρεσεψιονίστ που κατευθύνει τους ασθενείς στους Γιατρούς σας.",
  },
  calls_dispatcher = {
    close = "Κλείσιμο διαλόγου αποστολέα κλήσεων",
    assigned = "Αυτό το κουτί μαρκάρεται εάν σε κάποιον έχει ανατεθεί η εργασία που αντιστοιχεί.",
    task = "Λίστα εργασιών - κάντε κλικ σε μια εργασία για να ανοίξετε το παράθυρο του προσωπικού στο οποίο έχει ανατεθεί και να κάνετε μετακύλιση στην τοποθεσία της εργασίας",
  },
  customise_window = {
    average_contents = "Αν επιθυμείτε το παιχνίδι να θυμάται ποια επιπλέον αντικείμενα προσθέτετε συνήθως όταν χτίζετε δωμάτια, τότε ενεργοποιήστε αυτήν την επιλογή",
    intro = "Απενεργοποιήστε ή ενεργοποιήστε το εισαγωγικό βίντεο, η ρύθμιση global movies θα πρέπει να είναι on αν θέλετε το εισαγωγικό βίντεο να παίζει κάθε φορά που φορτώνετε το CorsixTH",
    paused = "Στο Theme Hospital ο παίκτης μπορούσε να χρησιμοποιήσει το top menu μόνο αν το παιχνίδι ήταν σε παύση. Αυτή είναι και η προεπιλεγμένη ρύθμιση στο CorsixTH, αλλά ενεργοποιώντας το τα πάντα επιτρέπονται ακόμα και εάν το παιχνίδι βρίσκεται σε παύση",
    aliens = "Λόγω της έλλειψης των κατάλληλων animations έχουμε ορίσει από προεπιλογή ότι οι ασθενείς με Εξωγήινο DNA θα έρχονται μόνο στα επείγοντα περιστατικά. Για να επιτρέψετε τους ασθενείς με Εξωγήινο DNA να επισκεφτούν το νοσοκομείο σας, σε περιπτώσεις εκτός των περιστατικών έκτακτης ανάγκης, απενεργοποιήστε αυτήν την ρύθμιση",
    fractured_bones = "Λόγω ενός poor animation έχουμε ορίσει από προεπιλογή το να μην υπάρχουν θηλυκοί ασθενείς με Σπασμένα Κόκκαλα. Για να επιτρέψεται σε θηλυκούς ασθενείς με Σπασμένη Κόκκαλα να επισκέπτονται το νοσοκομείο σας, απενεργοποιήστε αυτήν την ρύθμιση",
    back = "Κλείσιμο αυτού του μενού και επιστροφή στο Μενού Ρυθμίσεων",
    movies = "Καθολικός ελεγχτής βίντεο (global movie control), αυτή η ρύθμιση θα σας επιτρέψει να απενεργοποιήσετε όλα τα βίντεo",
    volume = "Αν το πλήκτρο volume down ανοίγει επίσης και το casebook, ενεργοποιήστε αυτήν την ρύθμιση για να αλλάξετε το hotkey για το casebook σε Shift + C",
  },
  status = {
    close = "Κλείσιμο οθόνης Επισκόπησης",
    win_progress_other = "Εμφάνιση προόδου αναφορικά με τα κριτήρια νίκης για %s",
    thirst = "Γενικό επίπεδο δίψας των ανθρώπων στο νοσοκομείο σας",
    happiness = "Γενικό επίπεδο ευτυχίας των ανθρώπων στο νοσοκομείο σας",
    percentage_cured = "Θα πρέπει να έχετε θεραπεύσει το %d%% των επισκεπτών στο νοσοκομείο σας. Μέχρι στιγμής έχετε θεραπεύσει το %d%%",
    win_progress_own = "Εμφάνιση προόδου αναφορικά με τα κριτήρια νίκης για το δικό σας νοσοκομείο",
    reputation = "Η Φήμη σας δεν πρέπει να είναι λιγότερο από %d. Προς το παρόν είναι %d",
    balance = "Το τραπεζικό σας υπόλοιπο δεν πρέπει να είναι κάτω από %d. Προς το παρόν είναι %d",
    warmth = "Γενικό επίπεδο ζεστασιάς των ανθρώπων στο νοσοκομείο σας",
    population_chart = "Γράφημα το οποίο δείχνει τι μερίδα του τοπικού πληθυσμού προσελκύει το κάθε νοσοκομείο",
    value = "Το νοσοκομείο σας πρέπει να αξίζει $%d. Προς το παρόν αξίζει $%d",
    percentage_killed = "Η αποστολή είναι να έχετε σκοτώσει λιγότερο από το %d%% των επισκεπτών. Προς το παρόν έχετε σκοτώσει το %d%%",
    num_cured = "Η αποστολή είναι να θεραπεύσετε %d ανθρώπους. Μέχρι στιγμής έχετε θεραπεύσει %d",
    population = "Θα πρέπει το %d%% του γενικού πληθυσμού να επισκέπτεται το νοσοκομείο σας",
  },
  research = {
    close = "Κλείσιμο οθόνης Έρευνας",
    cure_dec = "Μείωση ποσοστού της έρευνας για Θεραπείες",
    drugs_dec = "Μείωση ποσοστού Φαρμακευτικής έρευνας",
    specialisation_dec = "Μείωση ποσοστού Ειδικής Έρευνας",
    specialisation_inc = "Αύξηση ποσοστού Ειδικής Έρευνας",
    drugs_inc = "Αύξηση ποσοστού Φαρμακευτικής έρευνας",
    allocated_amount = "Δεσμευμένο ποσό του προϋπολογισμού",
    diagnosis_dec = "Μείωση ποσοστού διαγνωστικής Έρευνας",
    improvements_dec = "Μείωση ποσοστού φαρμακευτικών Βελτιώσεων",
    diagnosis_inc = "Αύξηση ποσοστού έρευνας Εξοπλισμού",
    cure_inc = "Αύξηση ποσοστού Έρευνας",
    improvements_inc = "Αύξηση ποσοστού Βελτιώσεων",
  },
  custom_game_window = {
    load_selected_level = "Φορτώστε και παίξτε το επιλεγμένο επίπεδο",
    free_build = "Τσεκάρετε αυτό το κουτί αν θέλετε να παίζετε χωρίς χρήματα ή συνθήκες νίκης και ήττας",
    choose_game = "Κλικάρετε ένα επίπεδο για να διαβάσετε περισσότερα σχετικά με αυτό",
  },
  new_game_window = {
    difficulty = "Επιλέξτε το επίπεδο δυσκολίας που θέλετε να παίξετε το παιχνίδι",
    medium = "Αυτή είναι ο μέσος δρόμος για να ακολουθήσετε αν δεν είστε σίγουροι το να επιλέξετε",
    player_name = "Εισάγετε το όνομα που θα θέλατε να σας αναφέρουν στο παιχνίδι",
    tutorial = "Κάντε κλικ εδώ για να ενεργοποιήσετε κάποιες συμβουλές για να σας βοηθήσουν να ξεκινήσετε στο παιχνίδι",
    hard = "Αν είστε συνηθισμένοι σε αυτό το είδος παιχνιδιού και θέλετε μια μεγαλύτερη πρόκληση, διαλέξτε αυτήν την επιλογή",
    cancel = "Ωπ, δεν ήθελα όντως να ξεκινήσω καινούριο παιχνίδι!",
    easy = "Αν είστε νέος σε παιχνίδια προσομοίωσης τότε αυτή είναι η κατάλληλη επιλογή για εσάς",
    start = "Ξεκινήστε το παιχνίδι με τις επιλεγμένες ρυθμίσεις",
  },
  message = {
    button = "Κάντε αριστερό κλικ για να ανοίξετε το μήνυμα",
    button_dismiss = "Κάντε αριστερό κλικ για να ανοίξετε το μήνυμα, δεξί κλικ για να το αποσύρετε",
  },
  graphs = {
    close = "Κλείσιμο Οθόνης Γραφήματος",
    visitors = "Εμφάνιση επισκεπτών",
    scale = "Κλίμακα γραφήματος",
    balance = "Εμφάνιση υπολοίπου",
    cures = "Εμφάνιση θεραπειών",
    wages = "Εμφάνιση μισθών",
    reputation = "Εμφάνιση Αξίας Νοσοκομείου",
    deaths = "Εμφάνιση θανάτων",
    money_out = "Εμφάνιση εξόδων",
    money_in = "Εμφάνιση εσόδων",
  },
  folders_window = {
    browse_screenshots = "Αναζητήστε άλλη τοποθεσία για φάκελο Screenshots  ( Τρέχουσα τοποθεσία: %1% ) ",
    browse_font = "Αναζητήστε άλλο αρχείο γραμματοσειράς ( τρέχουσα τοποθεσία: %1% )",
    not_specified = "Δεν έχει οριστεί τοποθεσία φακέλου!",
    savegames_location = "Από προεπιλογή, ο φάκελος των Saves βρίσκεται μαζί με το αρχείο ρυθμίσεων και θα χρησιμοποιηθεί για την διατήρηση των αποθηκευμένων παιχνιδιών σας. Αν αυτό δεν είναι επιθυμητό, τότε μπορείτε να διαλέξετε τον δικό σας φάκελο, απλώς περιηγηθείτε στην διαδρομή δίσκου που θέλετε να χρησιμοποιήσετε.",
    browse = "Αναζήτηση τοποθεσίας φακέλου",
    data_location = "Ο φάκελος της αρχικής εγκατάστασης του Theme Hospital, η οποία απαιτείται για να τρέξει το CorsixTH",
    browse_data = "Αναζητήστε άλλη τοποθεσία εγκατάστασης του Theme Hospital (τρέχουσα τοποθεσία: %1%)",
    reset_to_default = "Επαναφορά διαδρομής στην προεπιλεγμένη τοποθεσία",
    browse_saves = "Αναζητήστε άλλη τοποθεσία για τον φάκελο των Saves σας  ( Τρέχουσα τοποθεσία: %1% ) ",
    screenshots_location = "Από προεπιλογή, τα Screenshots αποθηκεύονται σε έναν φάκελο μαζί με το αρχείο ρυθμίσεων. Αν αυτό δεν είναι επιθυμητό, τότε μπορείτε να διαλέξετε τον δικό σας φάκελο, απλώς περιηγηθείτε στην διαδρομή δίσκου που θέλετε να χρησιμοποιήσετε.",
    browse_music = "Αναζητήστε άλλη τοποθεσία για τον φάκελο της μουσικής σας  ( Τρέχουσα τοποθεσία: %1% ) ",
    no_font_specified = "Δεν έχει οριστεί τοποθεσία γραμματοσειράς!",
    back = "Κλείσιμο αυτού του μενού και επιστροφή στο Μενού Ρυθμίσεων",
    default = "Προεπιλεγμένη τοποθεσία",
    music_location = "Επιλέξτε τοποθεσία για τα αρχεία μουσικής σας.  Πρέπει να έχετε δημιουργήσει ήδη τον φάκελο, και ύστερα να περιηγηθείτε σε αυτόν που μόλις δημιουργήσατε.",
    font_location = "Τοποθεσία αρχείου γραμματοσειράς που είναι ικανό να εμφανίσει χαρακτήρες Unicode οι οποίοι απαιτούνται από την γλώσσα σας. Αν αυτό δεν έχει οριστεί, τότε δεν θα μπορέσετε να επιλέξετε γλώσσες που χρειάζονται περισσότερους χαρακτήρες από όσους μπορεί να παρέχει το αρχικό παιχνίδι. Παραδείγματα τέτοιων γλωσσών: Ρωσικά και Κινέζικα",
  },
  toolbar = {
    hire = "Πρόσληψη προσωπικού",
    research = "Έρευνα",
    status = "Κατάσταση",
    bank_button = "Αριστερό Κλικ για Διαχειριστή Τραπέζης, Δεξί Κλικ για Τραπεζική Δήλωση",
    balance = "Το υπόλοιπό σας",
    edit = "Επεξεργασία δωματίων/αντικειμένων",
    charts = "Γραφήματα",
    reputation = "Η φήμη σας",
    town_map = "Χάρτης πόλης",
    rooms = "Χτίσιμο δωματίων",
    policy = "Πολιτικές",
    date = "Η ημερομηνία",
    casebook = "Αρχείο υποθέσεων (casebook) φαρμάκων",
    staff_list = "Διαχείριση προσωπικού",
    objects = "Επίπλωση διαδρόμων",
  },
  hotkey_window = {
    button_accept = "Αποδοχή της παραπάνω ανάθεσης hotkeys και αποθήκευση στον δίσκο",
    button_defaults = "Επαναφορά όλων των hotkeys στις προεπιλογές του προγράμματος",
    button_back_02 = "Επιστροφή στο βασικό παράθυρο hotkey. Εκεί μπορείτε να αποδεχτείτε τα hotkeys που άλλαξαν σε αυτό εδώ το παράθυρο",
    button_cancel = "Ακύρωση ανάθεσης και επιστροφή στο μενού επιλογών",
    button_recallPosKeys = "Άνοιγμα παραθύρου για ορισμό πλήκτρων για αποθήκευση και επανάκληση των θέσεων της κάμερας",
    caption_panels = "Άνοιγμα παραθύρου για ανάθεση πλήκτρων του panel",
  },
  town_map = {
    heat_dec = "Μείωση θερμοκρασίας",
    close = "Έξοδος από την οθόνη πόλης",
    balance = "Υπόλοιπο",
    people = "Εμφάνιση ανθρώπων",
    heating_bill = "Λογαριασμός θέρμανσης",
    heat_level = "Επίπεδο θέρμανσης",
    radiators = "Εμφάνιση καλοριφέρ",
    fire_extinguishers = "Εμφάνιση πυροσβεστήρων",
    plants = "Εμφάνιση φυτών",
    heat_inc = "Αύξηση θερμοκρασίας",
    objects = "Εμφάνιση αντικειμένων",
  },
  custom_campaign_window = {
    start_selected_campaign = "Φόρτωση του πρώτου επιπέδου αυτής της καμπάνιας",
    choose_campaign = "Επιλέξτε μια καμπάνια για να διαβάσετε περισσότερα σχετικά με αυτήν",
  },
  casebook = {
    cured = "Αριθμός αναρρώσεων",
    sent_home = "Αριθμός ασθενών που απορρίφθησαν",
    increase = "Αύξηση ποσού",
    cure_type = {
      surgery = "Αυτή η ασθένεια απαιτεί εγχείρηση",
      drug = "Αυτή η αγωγή χρησιμοποιεί φάρμακο",
      psychiatrist = "Ο Ψυχίατρος εκτελεί αυτήν την αγωγή",
      machine = "Αυτή η αγωγή απαιτεί ένα μηχάνημα",
      drug_percentage = "Αυτή η αγωγή χρησιμοποιεί φάρμακο - το δικό σας είναι %d%% αποδοτικό",
      unknown = "Δεν γνωρίζετε ακόμα πως να θεραπεύσετε αυτήν την ασθένεια",
    },
    earned_money = "Συνολικά κέρδη μέχρι τώρα",
    close = "Κλείσιμο Αρχείου Υποθέσεων",
    reputation = "Φήμη Αναφορικά με Θεραπείες ή Διαγνώσεις",
    down = "Κύλιση κάτω",
    research = "Κάντε κλικ εδώ για να χρησιμοποιήσετε μπάτζετ της Ειδικευμένης Έρευνας σε αυτήν την θεραπεία",
    deaths = "Αριθμός θανόντων",
    treatment_charge = "Κόστος",
    decrease = "Μείωση ποσού",
    cure_requirement = {
      hire_staff = "Απαιτείται να προσλάβετε προσωπικό για να χειριστείτε αυτήν την θεραπεία",
      possible = "Μπορείτε να χειριστείτε αυτήν την θεραπεία",
      hire_surgeon = "Απαιτείται να προσλάβετε έναν δεύτερο Χειρουργό για να πραγματοποιείτε εγχειρήσεις",
      hire_surgeons = "Απαιτείται να προσλάβετε δύο Χειρουργούς για να πραγματοποιείτε εγχειρήσεις",
      build_ward = "Πρέπει να χτίσετε μια Πτέρυγα για να χειριστείτε αυτήν την θεραπεία",
      research_machine = "Πρέπει να ερευνήσετε εξοπλισμό για αυτήν την θεραπεία",
      ward_hire_nurse = "Χρειάζεστε μια Νοσοκόμα να δουλέψει σε μια Πτέρυγα για να χειριστείτε αυτήν την θεραπεία",
      build_room = "Απαιτείται να χτίσετε ένα δωμάτιο για να χειριστείτε αυτήν την θεραπεία",
      hire_staff_old = "Πρέπει να προσλάβετε έναν/μία %s για να χειριστείτε αυτήν την θεραπεία",
      not_possible = "Δεν μπορείτε ακόμα να χειριστείτε αυτήν την θεραπεία",
    },
    up = "Κύλιση πάνω",
  },
  pay_rise_window = {
    accept = "Ανταποκριθείτε στην ζήτηση για αύξηση πληρωμής",
    decline = "Μην δεχτείτε την αύξηση - απολύστε τους αντί αυτού",
  },
  window_general = {
    confirm = "Επιβεβαίωση",
    cancel = "Ακύρωση",
  },
  options_window = {
    capture_mouse = "Κάντε κλικ για να ενεργοποιήσετε/απενεργοποιήσετε την δέσμευση του κέρσορα από το παράθυρο του παιχνιδιού",
    apply_zoomspeed = "Εφαρμογή της καταχωρημένης ταχύτητας zoom.",
    language_dropdown_item = "Επιλέξτε τα %s ως γλώσσα",
    select_language = "Επιλέξτε την γλώσσα του παιχνιδιού",
    cancel_zoomspeed = "Επιστροφή χωρίς αλλαγή της ταχύτητας zoom.",
    audio_button = "Ενεργοποιήστε ή απενεργοποιήστε όλους τους ήχους του παιχνιδιού",
    audio_toggle = "Ενεργοποιήστε ή απενεργοποιήστε",
    resolution = "Η ανάλυση στην οποία τρέχει το παιχνίδι",
    back = "Κλείσιμο παραθύρου Ρυθμίσεων",
    language = "Η γλώσσα στην οποία θα εμφανιστούν τα κείμενα του παιχνιδιού",
    scrollspeed = "Θέστε την ταχύτητα κύλισης μεταξύ 1 (η πιο αργή) και 10 (η ταχύτερη). Η προεπιλογή είναι 2.",
    width = "Εισάγετε επιθυμητό πλάτος οθόνης",
    fullscreen = "Επιλέξτε αν το παιχνίδι θα τρέχει σε λειτουργία πλήρης οθόνης ή παραθύρου",
    cancel_scrollspeed = "Επιστροφή χωρίς να αλλάξετε την ταχύτητα κύλισης.",
    hotkey = "Αλλαγή hotkeys πληκτρολογίου.",
    apply_shift_scrollspeed = "Εφαρμόστε την καταχωρημένη ταχύτητα κύλισης με Shift.",
    apply = "Εφαρμογή της καταχωρημένης ανάλυσης",
    change_resolution = "Αλλαγή της ανάλυσης του παραθύρου στις διαστάσεις που καταχωρήθηκαν στα αριστερά",
    apply_scrollspeed = "Εφαρμογή της καταχωρημένης ταχύτητας κύλισης.",
    fullscreen_button = "Κάντε κλικ για ενεργοποίηση/απενεργοποίηση λειτουργίας πλήρης οθόνης",
    cancel_shift_scrollspeed = "Επιστροφή χωρίς να αλλάξετε την ταχύτητα κύλισης με Shift.",
    select_resolution = "Επιλέξτε μια νέα ανάλυση",
    zoom_speed = "Θέστε την ταχύτητα zoom της κάμερας από 10 (η πιο αργή) σε 1000 (η ταχύτερη). Η προεπιλογή είναι 80.",
    customise_button = "Περισσότερες ρυθμίσεις που μπορείτε να τροποποιήσετε για να προσαρμόσετε την εμπειρία του game play",
    folder_button = "Επιλογές Φακέλων",
    cancel = "Επιστροφή χωρίς να αλλάξετε την ανάλυση",
    shift_scrollspeed = "Θέστε την ταχύτητα της κύλισης όταν πιέζετε παρατεταμένα το Shift καθώς κάνετε scrolling. 1 (η πιο αργή) με 10 (η ταχύτερη). Προεπιλογή: 4.",
    height = "Εισάγετε επιθυμητό ύψος οθόνης",
  },
  cheats_window = {
    close = "Κλείσιμο διαλόγου cheats",
    cheats = {
      all_research = "Ολοκληρώνει όλη την έρευνα.",
      emergency = "Δημιουργεί ένα επείγον περιστατικό.",
      vip = "Δημιουργεί έναν VIP.",
      win_level = "Κερδίζει το τρέχον επίπεδο.",
      epidemic = "Δημιουργεί έναν μεταδοτικό ασθενεί που μπορεί να προκαλέσει επιδημία",
      toggle_infected = "Εμφανίζει τα εικονίδια των μολυσμένων για την ενεργή επιδημία που ανακαλύφθηκε",
      end_year = "Μετάβαση στο τέλος του χρόνου.",
      lose_level = "Χάστε το τρέχον επίπεδο.",
      increase_prices = "Αύξηση όλων των τιμών κατά 50% (μέγιστο 200%)",
      earthquake = "Δημιουργεί έναν σεισμό.",
      decrease_prices = "Μείωση όλων των τιμών κατά 50% (ελάχιστο 50%)",
      create_patient = "Δημιουργεί έναν Ασθενή στα όρια του χάρτη.",
      money = "Προσθέτει 10.000 στο τραπεζικό σας υπόλοιπο.",
      end_month = "Μετάβαση στο τέλος του μήνα.",
    },
  },
  place_objects_window = {
    buy_sell = "Αγοραπώληση αντικειμένων",
    cancel = "Ακύρωση",
    pick_up = "Σήκωμα αντικειμένου",
    confirm = "Επιβεβαίωση",
  },
  staff_window = {
    face = "Το πρόσωπο αυτού του ατόμου - κλικ για άνοιγμα οθόνης διαχείρισης",
    close = "Ακύρωση αιτούντα",
    tiredness = "Επίπεδο κούρασης",
    salary = "Μηνιαίος μισθός",
    happiness = "Επίπεδο ευτυχίας",
    researcher = "Ερευνητής",
    doctor_seniority = "Βαθμός - Μαθητευόμενος, Γιατρός ή Σύμβουλος",
    skills = "Ειδικές Δεξιότητες",
    surgeon = "Χειρουργός",
    center_view = "Αριστερό κλικ για εστίαση στο προσωπικό, δεξί κλικ για εναλλαγή μεταξύ ανθρώπινου δυναμικού",
    name = "Όνομα προσωπικού",
    pick_up = "Αρπαγή",
    sack = "Απόλυση",
    psychiatrist = "Ψυχίατρος",
    ability = "Επίπεδο ικανοτήτων",
  },
  load_game_window = {
    load_game_number = "Φόρτωση παιχνιδιού %d",
    load_game = "Φόρτωση παιχνιδιού %s",
    load_autosave = "Φόρτωση autosave",
  },
  machine_window = {
    close = "Ακύρωση αιτούντα",
    status = "Κατάσταση του εξοπλισμού",
    name = "Όνομα",
    times_used = "Αριθμός φορών που έχει χρησιμοποιηθεί ο εξοπλισμός",
    repair = "Καλέστε Πολυτεχνίτη για επιδιόρθωση εξοπλισμού",
    replace = "Αντικατάσταση εξοπλισμού",
  },
  lua_console = {
    close = "Κλείσιμο κονσόλας",
    textbox = "Εισάγετε κώδικα Lua για να τρέξετε εδώ",
    execute_code = "Τρέξτε τον κώδικα που έχετε εισάγει",
  },
  menu_list_window = {
    back = "Κλείσιμο αυτού του παραθύρου",
    save_date = "Κάντε κλικ εδώ για να ταξινομήσετε την λίστα κατά τελευταία ημερομηνία τροποποίησης",
    name = "Κάντε κλικ εδώ για να ταξινομήσετε την λίστα κατά όνομα",
  },
  save_map_window = {
    map = "Αντικατάσταση χάρτη %s",
    new_map = "Εισάγετε όνομα αποθηκευμένου παιχνιδιού για τον χάρτη",
  },
  research_policy = {
    research_progress = "Πρόοδος για την επόμενη ανακάλυψη σε αυτήν την κατηγορία: %1%/%2%",
    no_research = "Δεν διεξάγεται κάποια έρευνα για την συγκεκριμένη κατηγορία αυτήν την στιγμή",
  },
  build_room_window = {
    cost = "Κόστος του τρέχοντος επιλεγμένου δωματίου",
    room_classes = {
      treatment = "Επιλέξτε δωμάτια Γενικής Θεραπείας",
      facilities = "Επιλέξτε δωμάτια Ειδικών Εγκαταστάσεων",
      diagnosis = "Επιλέξτε δωμάτια Διάγνωσης",
      clinic = "Επιλέξτε δωμάτια Ειδικευμένων Κλινικών",
    },
    close = "Ακύρωση αυτού του αιτούντα και επιστροφή στο παιχνίδι",
  },
  policy = {
    close = "Κλείσιμο οθόνης Πολιτικής",
    staff_stay = "Κάντε κλικ εδώ για να κάνετε ορισμένα άτομα του προσωπικού σας να παραμένουν στο δωμάτιο που τους τοποθετήσατε",
    diag_procedure = "Αν η διάγνωση ενός Γιατρού είναι λιγότερο ακριβής από το ποσοστό ΑΠΟΣΤΟΛΗΣ ΣΤΟ ΣΠΙΤΙ τότε ο ασθενής θα σταλθεί στο σπίτι του. Αν η διάγνωση είναι καλύτερη από το ποσοστό ΕΙΚΑΣΙΑΣ ΘΕΡΑΠΕΙΑΣ τότε ο ασθενής θα σταλθεί για την κατάλληλη θεραπεία",
    staff_rest = "Πόσο κουρασμένο πρέπει να είναι το προσωπικό σας πριν πάνε για ξεκούραση",
    diag_termination = "Η διάγνωση ενός ασθενούς θα συνεχιστεί έως ότου οι Γιατροί είναι τόσο σίγουροι όσο το ποσοστό ΔΙΑΚΟΠΗΣ ΔΙΑΔΙΚΑΣΙΑΣ ή μέχρι να εξαντληθεί όλος ο διαγνωστικός εξοπλισμός σας",
    staff_leave = "Κάντε κλικ εδώ για να καλέσετε προσωπικό το οποίο δεν είναι απασχολημένο προς βοήθεια συναδέλφων που την χρειάζονται",
  },
  buy_objects_window = {
    decrease = "Αγοράστε ένα λιγότερο από αυτό το αντικείμενο",
    total_value = "Συνολική αξία αντικειμένων που έχετε παραγγείλει",
    confirm = "Αγορά αντικειμένων",
    price = "Τιμή αντικειμένου",
    cancel = "Ακύρωση",
    increase = "Αγοράστε ένα περισσότερο από αυτό το αντικείμενο",
  },
  main_menu = {
    load_menu = {
      load_slot = "ΦΟΡΤΩΣΗ ΠΑΙΧΝΙΔΙΟΥ",
      empty_slot = "ΚΕΝΟ",
    },
    map_edit = "Δημιουργία προσαρμοσμένου χάρτη",
    custom_campaign = "Παίχτε μια καμπάνια δημιουργημένη από την κοινότητα",
    new_game = "Εκκίνηση του πρώτου επιπέδου στην καμπάνια",
    network = "Εκκίνηση παιχνιδιού δικτύου",
    exit = "Όχι, όχι, σας παρακαλώ μην φεύγετε!",
    quit = "Πρόκειται να κάνετε έξοδο από το CorsixTH.   Είστε σίγουροι πως αυτό θέλετε να κάνετε;",
    options = "Τροποποιήστε διάφορες ρυθμίσεις",
    custom_level = "Χτίστε το νοσοκομείο σας σε ένα μοναδικό σενάριο",
    load_game = "Φόρτωση αποθηκευμένου παιχνιδιού",
    continue = "Συνέχεια του πιο πρόσφατου αποθηκευμένου παιχνιδιού",
  },
  patient_window = {
    close = "Ακύρωση αιτούντα",
    queue = "Προβολή λεπτομερειών για την θέση αυτού του ατόμου στην ουρά",
    center_view = "Εστίαση οθόνης σε αυτό το άτομο",
    thirst = "Πόσο διψασμένο είναι αυτό το άτομο",
    happiness = "Ευτυχία ατόμου",
    casebook = "Προβολή λεπτομερειών ασθένειας",
    graph = "Κλικ για εναλλαγή μεταξύ του γραφήματος της υγείας του ατόμου και το ιστορικό θεραπείας",
    send_home = "Απομάκρυνση του ατόμου από το νοσοκομείο",
    abort_diagnosis = "Αποστολή ατόμου για θεραπεία τώρα αντί να περιμένετε την ολοκλήρωση της διάγνωσης",
    warmth = "Επίπεδο ζεστασιάς ατόμου",
  },
  statement = {
    close = "Κλείσιμο οθόνης Δήλωσης",
  },
}
original_credits = {
  [1] = " ",
  [2] = " ",
  [3] = " ",
  [4] = " ",
  [5] = " ",
  [6] = " ",
  [7] = " ",
  [8] = " ",
  [9] = " ",
  [10] = " ",
  [11] = " ",
  [12] = ":Σχεδιάστηκε και Δημιουργήθηκε από",
  [13] = ":Bullfrog Productions",
  [14] = " ",
  [15] = ":Ομάδα Ανάπτυξης Pluto",
  [16] = ",",
  [17] = "Mark Webley",
  [18] = "Gary Carr",
  [19] = "Matt Chilton",
  [20] = "Matt Sullivan",
  [21] = "Jo Rider",
  [22] = "Rajan Tande",
  [23] = "Wayne Imlach",
  [24] = "Andy Bass",
  [25] = "Jon Rennie",
  [26] = "Adam Coglan",
  [27] = "Natalie White",
  [28] = " ",
  [29] = " ",
  [30] = " ",
  [31] = ":Προγραμματισμός",
  [32] = ",",
  [33] = "Mark Webley",
  [34] = "Matt Chilton",
  [35] = "Matt Sullivan",
  [36] = "Rajan Tande",
  [37] = " ",
  [38] = " ",
  [39] = " ",
  [40] = ":Art",
  [41] = ",",
  [42] = "Gary Carr",
  [43] = "Jo Rider",
  [44] = "Andy Bass",
  [45] = "Adam Coglan",
  [46] = " ",
  [47] = " ",
  [48] = " ",
  [49] = ":Επιπλέον Προγραμματισμός",
  [50] = ",",
  [51] = "Ben Deane",
  [52] = "Gary Morgan",
  [53] = "Jonty Barnes",
  [54] = " ",
  [55] = " ",
  [56] = " ",
  [57] = ":Επιπλέον Art",
  [58] = ",",
  [59] = "Eoin Rogan",
  [60] = "George Svarovsky",
  [61] = "Saurev Sarkar",
  [62] = "Jason Brown",
  [63] = "John Kershaw",
  [64] = "Dee Lee",
  [65] = " ",
  [66] = " ",
  [67] = " ",
  [68] = ":Ακολουθία Intro",
  [69] = ",",
  [70] = "Stuart Black",
  [71] = " ",
  [72] = " ",
  [73] = " ",
  [74] = ":Μουσική και Ηχητικά Εφέ",
  [75] = ",",
  [76] = "Russell Shaw",
  [77] = "Adrian Moore",
  [78] = " ",
  [79] = " ",
  [80] = " ",
  [81] = ":Επιπλέον Μουσική",
  [82] = ",",
  [83] = "Jeremy Longley",
  [84] = "Andy Wood",
  [85] = " ",
  [86] = " ",
  [87] = " ",
  [88] = ":Φωνή Σχολιαστή",
  [89] = ",",
  [90] = "Rebecca Green",
  [91] = " ",
  [92] = " ",
  [93] = " ",
  [94] = ":Σχεδιασμός Επιπέδων",
  [95] = ",",
  [96] = "Wayne Imlach",
  [97] = "Natalie White",
  [98] = "Steven Jarrett",
  [99] = "Shin Kanaoya",
  [100] = " ",
  [101] = " ",
  [102] = " ",
  [103] = ":Scripting",
  [104] = ",",
  [105] = "James Leach",
  [106] = "Sean Masterson",
  [107] = "Neil Cook",
  [108] = " ",
  [109] = " ",
  [110] = " ",
  [111] = ":R&D",
  [112] = " ",
  [113] = ":Μηχανή Γραφικών",
  [114] = ",",
  [115] = "Andy Cakebread",
  [116] = "Richard Reed",
  [117] = " ",
  [118] = " ",
  [119] = " ",
  [120] = ":Υποστήριξη R&D",
  [121] = ",",
  [122] = "Glenn Corpes",
  [123] = "Martin Bell",
  [124] = "Ian Shaw",
  [125] = "Jan Svarovsky",
  [126] = " ",
  [127] = " ",
  [128] = " ",
  [129] = ":Βιβλιοθήκη και Εργαλεία",
  [130] = " ",
  [131] = "Βιβλιοθήκη Dos και Win 95",
  [132] = ",",
  [133] = "Mark Huntley",
  [134] = "Alex Peters",
  [135] = "Rik Heywood",
  [136] = " ",
  [137] = " ",
  [138] = " ",
  [139] = ":Βιβλιοθήκη Δικτύου",
  [140] = ",",
  [141] = "Ian Shippen",
  [142] = "Mark Lamport",
  [143] = " ",
  [144] = " ",
  [145] = " ",
  [146] = ":Βιβλιοθήκη Ήχου",
  [147] = ",",
  [148] = "Russell Shaw",
  [149] = "Tony Cox",
  [150] = " ",
  [151] = " ",
  [152] = " ",
  [153] = ":Προγραμματισμός Installer",
  [154] = ",",
  [155] = "Andy Nuttall",
  [156] = "Tony Cox",
  [157] = "Andy Cakebread",
  [158] = " ",
  [159] = " ",
  [160] = " ",
  [161] = ":Ηθική Υποστήριξη",
  [162] = ",",
  [163] = "Peter Molyneux",
  [164] = " ",
  [165] = " ",
  [166] = " ",
  [167] = ":Testing Manager",
  [168] = ",",
  [169] = "Andy Robson",
  [170] = " ",
  [171] = " ",
  [172] = " ",
  [173] = ":Lead Testers",
  [174] = ",",
  [175] = "Wayne Imlach",
  [176] = "Jon Rennie",
  [177] = " ",
  [178] = " ",
  [179] = " ",
  [180] = ":Play Testers",
  [181] = ",",
  [182] = "Jeff Brutus",
  [183] = "Wayne Frost",
  [184] = "Steven Lawrie",
  [185] = "Tristan Paramor",
  [186] = "Nathan Smethurst",
  [187] = " ",
  [188] = "Ryan Corkery",
  [189] = "Simon Doherty",
  [190] = "James Dormer",
  [191] = "Martin Gregory",
  [192] = "Ben Lawley",
  [193] = "Joel Lewis",
  [194] = "David Lowe",
  [195] = "Robert Monczak",
  [196] = "Dominic Mortoza",
  [197] = "Karl O'Keeffe",
  [198] = "Michael Singletary",
  [199] = "Andrew Skipper",
  [200] = "Stuart Stephen",
  [201] = "David Wallington",
  [202] = " ",
  [203] = "And all our other Work Experience Play Testers",
  [204] = " ",
  [205] = " ",
  [206] = " ",
  [207] = ":Τεχνική Υποστήριξη",
  [208] = ",",
  [209] = "Kevin Donkin",
  [210] = "Mike Burnham",
  [211] = "Simon Handby",
  [212] = " ",
  [213] = " ",
  [214] = " ",
  [215] = ":Marketing",
  [216] = ",",
  [217] = "Pete Murphy",
  [218] = "Sean Ratcliffe",
  [219] = " ",
  [220] = " ",
  [221] = " ",
  [222] = ":Με ευχαριστίες σε:",
  [223] = ",",
  [224] = "Tamara Burke",
  [225] = "Annabel Roose",
  [226] = "Chris Morgan",
  [227] = "Pete Larsen",
  [228] = " ",
  [229] = " ",
  [230] = " ",
  [231] = ":PR",
  [232] = ",",
  [233] = "Cathy Campos",
  [234] = " ",
  [235] = " ",
  [236] = " ",
  [237] = ":Documentation",
  [238] = ",",
  [239] = "Mark Casey",
  [240] = "Richard Johnston",
  [241] = "James Lenoel",
  [242] = "Jon Rennie",
  [243] = " ",
  [244] = " ",
  [245] = " ",
  [246] = ":Documentation & Σχεδιασμός Συσκευασίας",
  [247] = ",",
  [248] = "Caroline Arthur",
  [249] = "James Nolan",
  [250] = " ",
  [251] = " ",
  [252] = " ",
  [253] = ":Project Manager Μετάφρασης",
  [254] = ",",
  [255] = "Carol Aggett",
  [256] = " ",
  [257] = " ",
  [258] = " ",
  [259] = ":Μετάφραση",
  [260] = ",",
  [261] = "Sandra Picaper",
  [262] = "Sonia 'Sam' Yazmadjian",
  [263] = " ",
  [264] = "Bettina Klos",
  [265] = "Alexa Kortsch",
  [266] = "Bianca Normann",
  [267] = " ",
  [268] = "C.T.O. S.p.A. Zola Predosa (BO)",
  [269] = "Gian Maria Battistini",
  [270] = "Maria Ziino",
  [271] = "Gabriele Vegetti",
  [272] = " ",
  [273] = "Elena Ruiz de Velasco",
  [274] = "Julio Valladares",
  [275] = "Ricardo Martínez",
  [276] = " ",
  [277] = "Kia Collin",
  [278] = "CBG Consult",
  [279] = "Ulf Thor",
  [280] = " ",
  [281] = " ",
  [282] = " ",
  [283] = ":Παραγωγή",
  [284] = ",",
  [285] = "Rachel Holman",
  [286] = " ",
  [287] = " ",
  [288] = " ",
  [289] = ":Παραγωγός",
  [290] = ",",
  [291] = "Mark Webley",
  [292] = " ",
  [293] = " ",
  [294] = " ",
  [295] = ":Συμπαραγωγός",
  [296] = ",",
  [297] = "Andy Nuttall",
  [298] = " ",
  [299] = " ",
  [300] = " ",
  [301] = ":Operations",
  [302] = ",",
  [303] = "Steve Fitton",
  [304] = " ",
  [305] = " ",
  [306] = " ",
  [307] = ":Διαχείριση Εταιρίας",
  [308] = ",",
  [309] = "Audrey Adams",
  [310] = "Annette Dabb",
  [311] = "Emma Gibbs",
  [312] = "Lucia Gobbo",
  [313] = "Jo Goodwin",
  [314] = "Sian Jones",
  [315] = "Kathy McEntee",
  [316] = "Louise Ratcliffe",
  [317] = " ",
  [318] = " ",
  [319] = " ",
  [320] = ":Εταιρικό Μάνατζμεντ",
  [321] = ",",
  [322] = "Les Edgar",
  [323] = "Peter Molyneux",
  [324] = "David Byrne",
  [325] = " ",
  [326] = " ",
  [327] = ":Όλοι στην Bullfrog Productions",
  [328] = " ",
  [329] = " ",
  [330] = " ",
  [331] = ":Ειδικές Ευχαριστίες σε",
  [332] = ",",
  [333] = "Όλους στο Frimley Park Hospital",
  [334] = " ",
  [335] = ":Ειδικά",
  [336] = ",",
  [337] = "Beverley Cannell",
  [338] = "Doug Carlisle",
  [339] = " ",
  [340] = " ",
  [341] = " ",
  [342] = ":Keep On Thinking",
  [343] = " ",
  [344] = " ",
  [345] = " ",
  [346] = " ",
  [347] = " ",
  [348] = " ",
  [349] = " ",
  [350] = " ",
  [351] = " ",
  [352] = " ",
  [353] = " ",
  [354] = " ",
  [355] = " ",
  [356] = " ",
  [357] = " ",
  [358] = " ",
  [359] = " ",
  [360] = " ",
  [361] = ".",
}
dynamic_info = {
  staff = {
    actions = {
      heading_for = "Κατευθύνεται για %s",
      wandering = "Απλά περιπλανιέται",
      going_to_repair = "Πηγαίνει να επιδιορθώσει %s",
      waiting_for_patient = "Περιμένει ασθενή",
      fired = "Απολύθηκε",
    },
    psychiatrist_abbrev = "Ψυχ.",
    tiredness = "Κούραση",
    ability = "Ικανότητα",
  },
  object = {
    times_used = "Χρησιμοποιήθηκε %d Φορές",
    strength = "Δύναμη %d",
    queue_expected = "Αναμενόμενη Ουρά %d",
    queue_size = "Μέγεθος Ουράς %d",
  },
  health_inspector = "Επιθεωρητής Υγείας",
  patient = {
    diagnosis_progress = "Πρόοδος διάγνωσης",
    emergency = "Επείγον περιστατικό: %s",
    actions = {
      fed_up = "Κουράστηκε και φεύγει",
      cured = "Θεραπεύτηκε!",
      queueing_for = "Στην ουρά για %s",
      epidemic_sent_home = "Στάλθηκε σπίτι από τον Επιθεωρητή",
      waiting_for_diagnosis_rooms = "Περιμένει να χτίσετε περισσότερες διαγνωστικές εγκαταστάσεις",
      no_treatment_available = "Δεν υπάρχει διαθέσιμη θεραπεία - πάω σπίτι",
      waiting_for_treatment_rooms = "Περιμένω να χτίσετε ένα δωμάτιο θεραπείας για εμένα",
      no_diagnoses_available = "Δεν υπάρχουν διάθεσιμες περισσότερες διαγνώσεις - πάω σπίτι",
      awaiting_decision = "Περιμένω την απόφασή σας",
      on_my_way_to = "Στο δρόμο για %s",
      epidemic_vaccinated = "Δεν είμαι πλέον μεταδοτικός",
      dying = "Πεθαίνει!",
      sent_home = "Στάλθηκε Σπίτι",
      no_gp_available = "Περιμένει να χτίσετε ένα Γραφείο Γενικού Ιατρού",
      sent_to_other_hospital = "Παραπέμφθηκε αλλού",
      epidemic_contagious = "Είμαι μεταδοτικός",
      prices_too_high = "Οι τιμές σας παραείναι υψηλές - πάω σπίτι",
    },
    diagnosed = "Διαγνώσθηκε: %s ",
    guessed_diagnosis = "Διάγνωση που εικάζεται: %s ",
  },
  vip = "Επισκέπτης VIP",
}
object = {
  desk = "Γραφείο",
  toilet_sink = "Νιπτήρας",
  fire_extinguisher = "Πυροσβεστήρας",
  skeleton = "Σκελετός",
  scanner = "Σαρωτής",
  x_ray = "Ακτινογραφία",
  litter_bomb = "Βόμβα Σκουπιδιών",
  rathole = "Ποντικότρυπα",
  video_game = "Βιντεοπαιχνίδι",
  electrolyser = "Μηχάνημα Ηλεκτρόλυσης",
  litter = "Σκουπίδι",
  swing_door2 = "Περιστρεφόμενη πόρτα",
  bookcase = "Βιβλιοθήκη",
  couch = "Καναπές",
  lamp = "Λάμπα",
  x_ray_viewer = "Προβολέας ακτινογραφίας",
  bed3 = "Κρεβάτι",
  chair = "Καρέκλα",
  bench = "Πάγκος",
  operating_table = "Χειρουργικό τραπέζι",
  crash_trolley = "Κινούμενη συρταριέρα",
  entrance_right = "Δεξιά πόρτα εισόδου",
  inflator = "Φουσκωτής",
  op_sink1 = "Νιπτήρας",
  radiator = "Καλοριφέρ",
  drinks_machine = "Αναψυκτήριο",
  cast_remover = "Μηχάνημα αφαίρεσης γύψου",
  ultrascanner = "Ultrascanner",
  hair_restorer = "Μηχάνημα αποκατάστασης μαλλιών",
  dna_fixer = "Επιδιορθωτής DNA",
  plant = "Φυτός",
  slicer = "Τεμαχιστής",
  auto_autopsy = "Αυτο-αυτοψία",
  computer = "Υπολογιστής",
  projector = "Προβολέας",
  radiation_shield = "Προστασία ραδιενέργειας",
  op_sink2 = "Νιπτήρας",
  table2 = "Τραπέζι",
  jelly_moulder = "Διαμορφωτής Ζελέ",
  cardio = "Καρδιόγραμμα",
  blood_machine = "Μηχάνημα Αίματος",
  door = "Πόρτα",
  pool_table = "Τραπέζι μπιλιάρδου",
  cabinet = "Αρχειοθήκη",
  shower = "Ντους",
  surgeon_screen = "Οθόνη χειρουργείου",
  gates_of_hell = "Πύλες της Κολάσεως",
  table1 = "Τραπέζι",
  bed2 = "Κρεβάτι",
  sofa = "Καναπές",
  comfortable_chair = "Αναπαυτική καρέκλα",
  console = "Κονσόλα",
  atom_analyser = "Ατομικός Αναλυτής",
  tv = "TV",
  bed = "Κρεβάτι",
  pharmacy_cabinet = "Ντουλάπι φαρμακείου",
  toilet = "Τουαλέτα",
  lecture_chair = "Καρέκλα διάλεξης",
  screen = "Οθόνη",
  bin = "Κάδος",
  swing_door1 = "Ταλαντευόμενη πόρτα",
  entrance_left = "Αριστερή πόρτα εισόδου",
  reception_desk = "Γραφείο ρεσεψιόν",
}
staff_title = {
  researcher = "Ερευνητής",
  general = "Γενικός",
  surgeon = "Χειρουργός",
  junior = "Αρχάριος",
  consultant = "Σύμβουλος",
  receptionist = "Ρεσεψιονίστ",
  doctor = "Γιατρός",
  nurse = "Νοσοκόμα",
  psychiatrist = "Ψυχίατρος",
}
rooms_short = {
  research_room = "Τμήμα Ερευνών",
  hair_restoration = "Επανόρθωση Μαλλιών",
  blood_machine = "Μηχάνημα Αίματος",
  scanner = "Σαρωτής",
  inflation = "Κλινική Ξεφουσκώματος",
  x_ray = "Ακτινογραφία",
  reception = "Ρεσεψιόν",
  staffroom = "Δωμάτιο Προσωπικού",
  destroyed = "Κατεστραμμένο",
  psychiatric = "Ψυχιατρείο",
  pharmacy = "Φαρμακείο",
  toilets = "Τουαλέτες",
  cardiogram = "Καρδιογράφημα",
  ward = "Πτέρυγα",
  operating_theatre = "Χειρουργείο",
  general_diag = "Γενική Διάγ",
  ultrascan = "Ultrascan",
  tongue_clinic = "Κλινική Χαλαρωμένης Γλώσσας",
  corridor_objects = "Αντικείμενα Διαδρόμου",
  training_room = "Εκπαίδευση",
  dna_fixer = "Επιδιορθωτής DNA",
  gps_office = "Γραφείο Γενικού Ιατρού",
  jelly_vat = "Jelly Vat",
  decontamination = "Απολύμανση",
  fracture_clinic = "Κλινική Καταγμάτων",
  electrolysis = "Κλινική Ηλεκτρόλυσης",
}
room_descriptions = {
  hair_restoration = {
    [1] = "Κλινική Επανόρθωσης Μαλλιών//",
    [2] = "Οι Ασθενείς που υποφέρουν από την ακραία πάθηση της Φαλάκρας θα παραπεμφθούν στο Μηχάνημα Επανόρθωσης Μαλλιών σε αυτήν την Κλινική. Το μηχάνημα, το οποίο χειρίζεται ένας Γιατρός, ραγδαία θα φυτέψει φρέσκα μαλλιά στο κεφάλι του ασθενούς.//",
    [3] = "Η Κλινική Επανόρθωσης Μαλλιών απαιτεί έναν οποιονδήποτε Γιατρό. Επίσης χρήζει περιστασιακής συντήρησης. ",
  },
  blood_machine = {
    [1] = "Μηχάνημα Αίματος//",
    [2] = "Το Μηχάνημα Αίματος είναι ένα κομμάτι διαγνωστικού εξοπλισμού το οποίο ελέγχει τα κύτταρα στο αίμα ενός ασθενή για να ανακαλύψει τι πρόβλημα υγείας αντιμετωπίζει.//",
    [3] = "Το Μηχάνημα Αίματος απαιτεί έναν οποιονδήποτε Γιατρό. Χρήζει επίσης συντήρησης. ",
  },
  scanner = {
    [1] = "Σαρωτής//",
    [2] = "Οι ασθενείς λαμβάνουν μία ακριβή διάγνωση χρησιμοποιώντας αυτόν τον σύνθετο Σαρωτή. Ύστερα πηγαίνουν να δουν έναν Γιατρό στο Γραφείο του Γενικού Ιατρού για περαιτέρω θεραπεία.//",
    [3] = "Ο Σαρωτής απαιτεί έναν οποιονδήποτε Γιατρό. Χρήζει επίσης συντήρησης. ",
  },
  inflation = {
    [1] = "Κλινική Φουσκώματος//",
    [2] = "Οι ασθενείς με την οδυνηρή αλλά παράλληλα αστεία πάθηση του Φουσκωτού Κεφαλιού πρέπει να έρθουν στην Κλινική Φουσκώματος, όπου θα του σκάσουν το υπερμέγεθος κρανίο και άμεσα θα το ξαναφουσκώσουν στην κατάλληλη πίεση.//",
    [3] = "Η Κλινική Φουσκώματος απαιτεί έναν οποιονδήποτε Γιατρό. Επίσης χρήζει συχνής συντήρησης από έναν Πολυτεχνίτη. ",
  },
  x_ray = {
    [1] = "Ακτινογραφία//",
    [2] = "Το μηχάνημα Ακτινογραφίας φωτογραφίζει το εσωτερικό των ανθρώπων χρησιμοποιώντας ειδική ακτινοβολία για να δώσουν στο ιατρικό προσωπικό σας μια καλή ιδέα του τι πάει στραβά.//",
    [3] = "Το δωμάτιο Ακτινογραφίας απαιτεί έναν οποιονδήποτε Γιατρό. Χρήζει επίσης συντήρησης. ",
  },
  operating_theatre = {
    [1] = "Χειρουργείο//",
    [2] = "Αυτή η σημαντική εγκατάσταση είναι το μέρος που γιατρεύεται μια πληθώρα παθήσεων. Το Χειρουργείο πρέπει να έχει μεγάλο μέγεθος, και πρέπει να εξοπλιστεί με τον κατάλληλο εξοπλισμό. Είναι ζωτικό κομμάτι του νοσοκομείου σας.//",
    [3] = "Το Χειρουργείο απαιτεί δύο Γιατρούς με πιστοποίηση Χειρουργού. ",
  },
  jelly_vat = {
    [1] = "Κλινική Ζελέ//",
    [2] = "Οι ασθενείς που έχουν καταραστεί με την γελοία ασθένεια Ζελετίτιδα πρέπει να ταλαντευτούν προς την Κλινική Ζελέ, και να τοποθετηθούν στην Δεξαμενή Ζελέ. Η συγκεκριμένη δεξαμενή τους θεραπεύει με έναν τρόπο ο οποίος ακόμα δεν είναι πλήρως κατανοητός από το ιατρικό επάγγελμα.//",
    [3] = "Η Κλινική Ζελέ απαιτεί έναν οποιονδήποτε Γιατρό. Χρειάζεται επίσης έναν Πολυτεχνίτη για συντήρηση. ",
  },
  staff_room = {
    [1] = "Δωμάτιο Προσωπικού//",
    [2] = "Το προσωπικό σας κουράζεται καθώς κάνουν την δουλειά τους. Χρειάζονται αυτό το δωμάτιο για να χαλαρωσούν και να ανανεωθούν. Το κουρασμένο προσωπικό γίνεται αργό, ζητά μεγαλύτερες απολαβές και στο τέλος θα παραιτηθεί. Κάνουν επίσης περισσότερα λάθη. Αξίζει πολύ να χτίσετε ένα δωμάτιο προσωπικού που έχει πολλές δραστηριότητες για αυτούς. Σιγορευτείτε ότι υπάρχει χώρος για αρκετά μέλη του προσωπικού ανά φορά. ",
  },
  pharmacy = {
    [1] = "Φαρμακείο//",
    [2] = "Οι ασθενείς που έχουν διαγνωστεί και χρήζουν φαρμακευτικής θεραπείας πρέπει να επισκεφτούν το Φαρμακείο για να λάβουν τα φάρμακά τους. Καθώς περισσότερες φαρμακευτικές θεραπείες ανακαλύπτονται και γίνονται διαθέσιμες, αυτό το δωμάτιο γίνεται όλο και πιο απασχολημένο. Μπορεί να θέλετε να φτιάξετε κι άλλο Φαρμακείο αργότερα.//",
    [3] = "Το Φαρμακείο απαιτεί μια Νοσοκόμα. ",
  },
  no_room = {
    [1] = "",
  },
  cardiogram = {
    [1] = "Καρδιογράφημα//",
    [2] = "Οι ασθενείς ελέγχονται εδώ και λαμβάνουν την διάγνωσή τους, προτού επιστρέψουν σε ένα Γραφείο Γενικού Ιατρού για να τους ανατεθεί μια θεραπεία.//",
    [3] = "Το δωμάτιο Καρδιογραφήματος απαιτεί έναν οποιονδήποτε Γιατρό. Χρήζει, επιπλέον, συντήρησης. ",
  },
  gp = {
    [1] = "Γραφείο Γενικού Ιατρού//",
    [2] = "Αυτό είναι το θεμελιώδες δωμάτιο διάγνωσης στο νοσοκομείο σας. Νέοι ασθενείς στέλνονται εδώ για να ανακαλύψουν τι δεν πάει καλά μαζί τους. Ύστερα παραπέμπονται για περαιτέρω διαγνώσεις είτε σε ένα δωμάτιο στο οποίο μπορούν να λάβουν την θεραπεία τους. Μπορεί να θέλετε να χτίσετε κι άλλο Γραφείο Γενικού Ιατρού αν το πρώτο γίνει πολύ απασχολημένο. Όσο μεγαλύτερο είναι το δωμάτιο, και όσα περισσότερα αντικείμενα τοποθετήσετε μέσα του, τόσο μεγαλύτερο το κύρος για τον Γιατρό. Αυτό ισχύει επίσης και για τα υπόλοιπα δωμάτια.//",
    [3] = "Το Γραφείο Γενικού Ιατρού απαιτεί έναν οποιονδήποτε Γιατρό. ",
  },
  psych = {
    [1] = "Ψυχιατρείο//",
    [2] = "Οι ασθενείς που διαγνώσκονται με ψυχιατρικές ασθένειες πρέπει να επισκεφτούν το Ψυχιατρικό Τμήμα για να λάβουν ψυχοθεραπεία. Οι Ψυχίατροι μπορούν επίσης να πραγματοποιήσουν διαγνώσεις, ανακαλύπτοντας τον τύπο των ασθενειών που αντιμετωπίζουν, αν είναι ψυχικής φύσεως, και να τις θεραπεύσουν χρησιμοποιώντας αυτόν τον αξιόπιστο καναπέ.//",
    [3] = "Το Ψυχιατρικό Τμήμα απαιτεί έναν Γιατρό ειδικευμένο στην επιστήμη της Ψυχιατρικής. ",
  },
  ward = {
    [1] = "Πτέρυγα//",
    [2] = "Οι Ασθενείς κρατούνται εδώ για παρακολούθηση από μια Νοσοκόμα κατά την διάρκεια μιας διάγνωσης. Παραμένουν εδώ επίσης πριν να εγχειρηθούν.//",
    [3] = "Η Πτέρυγα απαιτεί μια Νοσοκόμα. ",
  },
  general_diag = {
    [1] = "Γενική Διάγνωση//",
    [2] = "Οι ασθενείς που απαιτούν περαιτέρω διάγνωση στέλνονται εδώ για να εξεταστούν. Αν το Γραφείο Γενικού Ιατρού δεν εξακριβώσει τι δεν πάει καλά, η Γενική Διάγνωση συχνά θα τα καταφέρει. Από εδώ θα σταλθούν πίσω στο Γραφείο του Γενικού Ιατρού για ανάλυση των αποτελεσμάτων που παρήχθησαν εδώ.//",
    [3] = "Το δωμάτιο Γενικής Διάγνωσης απαιτεί έναν οποιονδήποτε Γιατρό. ",
  },
  ultrascan = {
    [1] = "Ultrascan//",
    [2] = "Το Ultrascan είναι το απόλυτο εργαλείο στον διαγνωστικό εξοπλισμό. Κοστίζει αρκετά αλλά αξίζει τα χρήματα αν θέλετε οι διαγνώσεις που πραγματοποιείτε στο νοσοκομείο σας να είναι άριστες.//",
    [3] = "Το Ultrascanner απαιτεί έναν οποιονδήποτε Γιατρό. Χρήζει επίσης συντήρησης. ",
  },
  toilets = {
    [1] = "Τουαλέτα//",
    [2] = "Οι ασθενείς που νιώθουν το κάλεσμα της φύσης θα χρειαστεί να ξαλαφρώσουν υπό την άνεση των εγκαταστάσεων υγεινής σας. Μπορείτε να χτίσετε επιπλέον τουαλέτες και νιπτήρες αν αναμένετε πολλούς επισκέπτες. Σε ορισμένες περιπτώσεις, ίσως θα ήταν καλή ιδέα να χτίσετε τουαλέτες σε διαφορετικά σημεία του νοσοκομείου σας. ",
  },
  tv_room = {
    [1] = "ΤΟ ΔΩΜΑΤΙΟ ΤΗΛΕΟΡΑΣΗΣ ΔΕΝ ΧΡΗΣΙΜΟΠΟΙΕΙΤΑΙ",
  },
  fracture_clinic = {
    [1] = "Κλινική Καταγμάτων//",
    [2] = "Οι ασθενείς εκείνοι που έχουν την ατυχία να έχουν Σπασμένα Κόκκαλα έρχονται εδώ. Το Μηχάνημα Αφαίρεσης Γύψου θα χρησιμοποιήσει ισχυρά βιομηχανικά laser για να κόψε οποιονδήποτε γύψο, προκαλώντας μόνο ελάχιστη οδύνη στον ασθενή.//",
    [3] = "Η Κλινική Καταγμάτων απαιτεί μια Νοσοκόμα. Χρήζει επιπλέον τακτικής συντήρησης. ",
  },
  dna_fixer = {
    [1] = "Κλινική DNA//",
    [2] = "Ασθενείς που έχουν πειραχτεί γενετικά από εξωγήινους από έναν άλλο κόσμο πρέπει να υποβληθούν σε αντικατάσταση DNA σε αυτό εδώ το δωμάτιο. Το Μηχάνημα Επιδιόρθωσης DNA είναι ένα πολύ περίπλοκο κομμάτι εξοπλισμού και θα ήταν φρόνιμο να έχετε έναν πυροσβεστήρα στο δωμάτιο, για παν ενδεχόμενο.//",
    [3] = "Το Μηχάνημα Επιδιόρθωσης DNA απαιτεί περιστασιακή συντήρηση από έναν Πολυτεχνίτη. Απαιτεί επίσης έναν Γιατρό ειδικευμένο στην Έρευνα για να το χειριστεί. ",
  },
  slack_tongue = {
    [1] = "Κλινική Χαλαρωμένης Γλώσσας//",
    [2] = "Οι ασθενείς που διαγνώσκονται με Χαλαρωμένη Γλώσσα στο Γραφείο Γενικού Ιατρού στέλνονται σε αυτήν την κλινική για θεραπεία. Ο Γιατρός θα χρησιμοποιήσει ένα κομμάτι εξοπλισμού υψίστης τεχνολογίας για να επεκτείνει την γλώσσα και να την τεμαχίσει, επιστρέφοντας έτσι τον ασθενή σε συνθήκες άκρας υγείας.//",
    [3] = "Η Κλινική Χαλαρωμένης Γλώσσας απαιτεί έναν οποιονδήποτε Γιατρό. Χρήζει επίσης τακτικής συντήρησης. ",
  },
  research = {
    [1] = "Δωμάτιο Ερευνών//",
    [2] = "Νέα φάρμακα και θεραπείες ανακαλύπτονται ή βελτιώνονται στο Δωμάτιο Ερευνών. Είναι ζωτικό κομμάτι του νοσοκομείου σας, και θα κάνει θαύματα για τα ποσοστά επιτυχίας των θεραπειών σας.//",
    [3] = "Το Δωμάτιο Ερευνών απαιτεί έναν Γιατρό ειδικευμένο στην Έρευνα. ",
  },
  decontamination = {
    [1] = "Κλινική Απολύμανσης//",
    [2] = "Οι ασθενείς που έχουν εκτεθεί σε Ακτινοβολία παραπέμπονται γρήγορα στην Κλινική Απολύμανσης. Αυτό το δωμάτιο περιέχει ένα ντουζ το οποίο καθαρίζει τον ασθενή από όλη την φρικιαστική ραδιενέργεια και μάκα.//",
    [3] = "Το Ντουζ Απολύμανσης απαιτεί έναν οποιονδήποτε Γιατρό. Χρήζει επίσης συντήρησης από έναν Πολυτεχνίτη. ",
  },
  training = {
    [1] = "Δωμάτιο Εκπαίδευσης//",
    [2] = "Οι Μαθητευόμενοι και οι Γιατροί σας μπορούν να κερδίσουν πολύτιμες επιπλέον εξειδικεύσεις μελετώντας σε αυτό το δωμάτιο. Έναν Σύμβουλος με Χειρουργικές, Ερευνητικές ή Ψυχιατρικές δεξιότητες θα τις μεταλαμπαδεύσει σε όποιους Γιατρούς εκπαιδεύονται. Οι Γιατροί που ήδη κατέχουν αυτές τις δεξιότητες θα ανακαλύψουν ότι η ικανότητά τους να τις χρησιμοποιούν βελτιώνεται εδώ.//",
    [3] = "Το Δωμάτιο Εκπαίδευσης απαιτεί έναν Σύμβουλο. ",
  },
  electrolysis = {
    [1] = "Δωμάτιο Ηλεκτρόλυσης//",
    [2] = "Οι ασθενείς με Μαλλίτιδα παραπέμπονται σε αυτό το δωμάτιο, όπου το ειδικό Μηχάνημα Ηλεκτρόλυσης τους ξεριζώνει τα μαλλιά και κλείνει τους πόρους ηλεκτρικά χρησιμοποιώντας μια ουσία που δεν διαφέρει και πολύ από σοβά.//",
    [3] = "Το Δωμάτιο Ηλεκτρόλυσης απαιτεί έναν οποιονδήποτε Γιατρό. Χρήζει επίσης συντήρησης από έναν Πολυτεχνίτη. ",
  },
}
menu_options = {
  capture_mouse = "  ΔΕΣΜΕΥΣΗ ΚΕΡΣΟΡΑ  ",
  wage_increase = "  ΑΙΤΗΣΕΙΣ ΑΥΞΗΣΗΣ ΜΙΣΘΩΝ  ",
  game_speed = "  ΤΑΧΥΤΗΤΑ ΠΑΙΧΝΙΔΙΟΥ  ",
  twentyfour_hour_clock = "  24ΩΡΟ ΡΟΛΟΪ  ",
  lock_windows = "  ΚΛΕΙΔΩΜΑ ΠΑΡΑΘΥΡΩΝ  ",
  sound_vol = "  ΕΝΤΑΣΗ ΗΧΟΥ  ",
  edge_scrolling = "  ΚΥΛΙΣΗ ΑΚΡΩΝ  ",
  sound = "  (%1%) ΗΧΟΣ   ",
  warmth_colors = "  ΧΡΩΜΑΤΑ ΘΕΡΜΟΤΗΤΑΣ  ",
  announcements_vol = "  ΗΧΟΣ ΑΝΑΚΟΙΝΩΣΕΩΝ  ",
  music_vol = "  ΕΝΤΑΣΗ ΜΟΥΣΙΚΗΣ  ",
  jukebox = "  (%1%) JUKEBOX  ",
  announcements = "  (%1%) ΑΝΑΚΟΙΝΩΣΕΙΣ   ",
  adviser_disabled = "  (%1%) ΒΟΗΘΟΣ  ",
  music = "  (%1%) ΜΟΥΣΙΚΗ   ",
  autosave = "  AUTOSAVE  ",
}
save_map_window = {
  new_map = "Νέος Χάρτης",
  caption = "Αποθήκευση Χάρτη (%1%)",
}
diseases = {
  diag_scanner = {
    name = "Διαγνωστικός Σαρωτής",
  },
  autopsy = {
    name = "Αυτοψία",
  },
  sweaty_palms = {
    name = "Ιδρωμένα Χέρια",
    cause = "Αιτία - φόβος συνεντεύξεων για δουλειά.",
    symptoms = "Συμπτώματα - οι χειραψίες με τον ασθενή είναι σαν να σφίγγεις ένα πρόσφατα βρεγμένο σφουγγάρι.",
    cure = "Θεραπεία - Ένας Ψυχίατρος βοηθάει τον ασθενή να ξεπεράσει αυτήν την φανταστική πάθηση.",
  },
  diag_general_diag = {
    name = "Γενική Διάγνωση",
  },
  third_degree_sideburns = {
    name = "Φαβορίτες 3ου Βαθμού",
    cause = "Αιτία - νοσταλγική λαχτάρα για τα 70s (ΣτΜ: λογοπαίγνιο με τα εγκαύματα 3ου βαθμού, 3rd Degree Sideburns)",
    symptoms = "Συμπτώματα - μεγάλο μαλλί, παντελόνια καμπάνες, πλατφόρμες και μέικ-απ από γκλίτερ.",
    cure = "Θεραπεία - Ο Ψυχίατρος πρέπει, κάνοντας χρήση σύγχρονων μεθόδων και τεχνικών, να πείσει τον ασθενή ότι αυτά τα μαλλιαρά αξεσουάρ είναι για τα σκουπίδια.",
  },
  ruptured_nodules = {
    name = "Σπασμένοι Όζοι",
    cause = "Αιτία - bungee jumping με κρύο καιρό.",
    symptoms = "Συμπτώματα - ανικανότητα να κάτσουν με άνεση.",
    cure = "Θεραπεία - Δύο ειδικευμένοι Χειρουργοί πρέπει να αφαιρέσουν τους όζους με σταθερά χέρια.",
  },
  diag_x_ray = {
    name = "Διαγ Ακτινογραφίας",
  },
  diag_psych = {
    name = "Διαγ Ψυχίατρου",
  },
  slack_tongue = {
    name = "Χαλαρωμένη Γλώσσα",
    cause = "Αιτία - χρόνιες υπεραναλύσεις και συζητήσεις γύρω από σαπουνόπερες.",
    symptoms = "Συμπτώματα - η γλώσσα πρήζεται και γίνεται πέντε φορές μεγαλύτερη από το κανονικό.",
    cure = "Θεραπεία - Η γλώσσα τοποθετείται στο Μηχάνημα Τεμαχισμού, και αφαιρείται γρήγορα, αποτελεσματικά και επίπονα.",
  },
  alien_dna = {
    name = "Εξωγήινο DNA",
    cause = "Αιτία - ευφυείς εξωγήινες μορφές που προσκολλάνε σε πρόσωπα.",
    symptoms = "Συμπτώματα - σταδιακή μεταμόρφωση σε εξωγήινο και επιθυμία για καταστροφή πόλεων.",
    cure = "Θεραπεία - Το DNA αφαιρείται μηχανικά, καθαρίζεται από εξωγήινα στοιχεία και αντικαθίσταται γρήγορα.",
  },
  infectious_laughter = {
    name = "Μεταδοτικό Γέλιο",
    cause = "Αιτία - κλασική κωμωδία καταστάσεων.",
    symptoms = "Συμπτώματα - ατελείωτα πνιχτά γελάκια και επανάληψη μη αστείων φράσεων.",
    cure = "Θεραπεία - Ένας πιστοποιημένος Ψυχίατρος πρέπει να υπενθυμίσει στον ασθενή πόσο σοβαρή είναι αυτή η ασθένεια.",
  },
  gastric_ejections = {
    name = "Γαστρικές Αποβολές",
    cause = "Αιτία - πικάντικο Μεξικάνικο ή Ινδικό φαγητό.",
    symptoms = "Συμπτώματα - μισο-χωνευμένο φαγητό που αποβάλλονται από τον ασθενή σε τυχαία συμπλέγματα.",
    cure = "Θεραπεία - Ο ασθενής πίνει ένα ειδικό προσκολλητικό διάλυμα που αποτρέπει την περαιτέρω αποβολή των περιεχομένων του στομαχιού.",
  },
  bloaty_head = {
    name = "Φουσκωτό Κεφάλι",
    cause = "Αιτία - το να σνιφάρεις τυρί και να πιεις μη απιονισμένο βροχόνερο.",
    symptoms = "Συμπτώματα - πολύ άβολο για τον πάσχοντα.",
    cure = "Θεραπεία - Το πρησμένο κεφάλι σκάει σαν φούσκα, και ξαναφουσκώνει στην σωστή πίεση χρησιμοποιώντας ένα έξυπνο μηχάνημα.",
  },
  discrete_itching = {
    name = "Διακριτική Φαγούρα",
    cause = "Αιτία - μικροσκοπικά έντομα με κοφτερά δόντια.",
    symptoms = "Συμπτώματα - φαγούρα, που οδηγεί σε φλεγμονές στο σώμα.",
    cure = "Θεραπεία - Ο ασθενής πίνει ένα φαρμακευτικό κολλώδες σιρόπι για να αποτρέψει την φαγούρα.",
  },
  invisibility = {
    name = "Αορατότητα",
    cause = "Αιτία - τσίμπημα από ραδιενεργό (και αόρατο) μυρμήγκι.",
    symptoms = "Συμπτώματα - οι ασθενείς δεν αντιμετωπίζουν καμία ενόχληση. Πράγματι, κάποιοι από αυτούς χρησιμοποιούν την πάθησή τους για να κάνουν πλάκες στην οικογένειά τους.",
    cure = "Θεραπεία - Ένα χρωματιστό διάλυμα στο Φαρμακείο σύντομα αποκαθιστά τον ασθενή σε πλήρη αδιαφάνεια.",
  },
  king_complex = {
    name = "Σύμπλεγμα του Βασιλιά",
    cause = "Προκαλείται όταν το πνεύμα του Βασιλιά (ΣτΜ: Έλβις) εισέρχεται στο μυαλό του ασθενούς και τον διακατέχει.",
    symptoms = "Συμπτώματα - ο ασθενής φοράει σουέντ χρωματιστά παπούτσια και τρώει cheeseburgers.",
    cure = "Θεραπεία - Ένας Ψυχίατρος λέει στον ασθενή πόσο γελοίος δείχνει.",
  },
  iron_lungs = {
    name = "Σιδερένια Πνευμόνια",
    cause = "Αιτία - αιθαλομίχλη του κέντρου της πόλης σε συνδυασμό με κατάλοιπα κεμπάμπ.",
    symptoms = "Συμπτώματα - ικανότητα να βγάζεις φωτιές από το στόμα και να φωνάζεις δυνατά κάτω από το νερό.",
    cure = "Θεραπεία - Δύο Χειρουργοί εργάζονται για να χειρουργήσουν και να αφαιρέσουν τα, σκληρά σαν από γύψο, πνευμόνια στο Χειρουργείο.",
  },
  diag_ultrascan = {
    name = "Διαγ Ultrascan",
  },
  pregnancy = {
    name = "Έγκυος",
    cause = "Αιτία - διακοπές ρεύματος σε αστικές περιοχές.",
    symptoms = "Συμπτώματα - τάσεις κατανάλωσης μεγάλων ποσοτήτων φαγητού και επακόλουθη μπυροκοιλιά.",
    cure = "Θεραπεία - Το μωρό αφαιρείται στο Χειρουργείο, πλένεται και παρουσιάζεται στον ασθενή.",
  },
  kidney_beans = {
    name = "Νεφροφάσολα",
    cause = "Αιτία - να μασουλάς παγάκια σε ποτά.",
    symptoms = "Συμπτώματα - πόνος και τακτικές επισκέψεις στην τουαλέτα.",
    cure = "Θεραπεία - Δύο Χειρουργοί πρέπει να αφαιρέσουν τα φασόλια χωρίς να αγγίξουν τα τοιχώματα των νεφρών.",
  },
  hairyitis = {
    name = "Μαλλίτιδα",
    cause = "Αιτία - παρατεταμένη έκθεση στο φεγγάρι.",
    symptoms = "Συμπτώματα - οι πάσχοντες βιώνουν έχουν ενισχυμένη αίσθηση όσφρησης.",
    cure = "Θεραπεία - Ένα μηχάνημα ηλεκτρόλυσης αφαιρεί το τρίχωμα και κλείνει τους πόρους.",
  },
  spare_ribs = {
    name = "Περίσσια Πλευρά",
    cause = "Αιτία - το να κάθεσαι σε κρύα πατώματα.",
    symptoms = "Συμπτώματα - μη ευχάριστο αίσθημα μεγαλωμένου στήθους.",
    cure = "Θεραπεία - Τα πλευρά πρέπει να αφαιρεθούν από δύο Χειρουργούς, και να δοθούν στον ασθενή σε πακέτο για να τα φάει ο σκύλος στο σπίτι.",
  },
  broken_heart = {
    name = "Ραγισμένη Καρδιά",
    cause = "Αιτία - κάποιος πλουσιότερος, νεότερος και πιο λεπτός από τον ασθενή.",
    symptoms = "Συμπτώματα - κλάμα και κακώσεις λόγω επαναλαμβανόμενης καταπόνησης στα χέρια λόγω σκισίματος φωτογραφιών από διακοπές.",
    cure = "Θεραπεία - Δύο Χειρουργοί ανοίγουν το στήθος και απαλά και προσεκτικά μπαλώνουν την καρδιά κρατώντας την αναπνοή τους.",
  },
  sleeping_illness = {
    name = "Ασθένεια Ύπνου",
    cause = "Αιτία - υπερενεργός αδένας ύπνου στον ουρανό του στόματος.",
    symptoms = "Συμπτώματα - Ακατανίκητη επιθυμία του ασθενούς να ξαπλώσει και να ρίξει έναν υπνάκο όπου και να βρίσκεται.",
    cure = "Θεραπεία - Υψηλή δόση ένος ισχυρού διεγερτικού το οποίο χορηγείται από μία Νοσοκόμα.",
  },
  broken_wind = {
    name = "Κλανίδι",
    cause = "Αιτία - το να χρησιμοποιήσεις έναν διάδρομο γυμναστηρίου μετά από ένα μεγάλο γεύμα.",
    symptoms = "Συμπτώματα - εκνευρισμός και αναστάτωση των ανθρώπων ακριβώς πίσω από τον ασθενή.",
    cure = "Θεραπεία - μια βαριά μείξη από ειδικά υδάτινα άτομα πίνεται σβέλτα στο Φαρμακείο.",
  },
  diag_blood_machine = {
    name = "Διαγ Μηχάνημα Αίματος",
  },
  serious_radiation = {
    name = "Σοβαρή Ακτινοβολία",
    cause = "Αιτία - το να μπερδέψεις ισότοπα πλουτωνίου για τσίχλες.",
    symptoms = "Συμπτώματα - οι ασθενείς με αυτήν την πάθηση νιώθουν πολύ, πολύ άσχημα.",
    cure = "Θεραπεία - Ο ασθενής πρέπει να τοποθετηθεί σε ένα Ντουζ Απολύμανσης και να καθαριστεί καταλλήλως.",
  },
  tv_personalities = {
    name = "TV Περσόνα",
    cause = "Αιτία - πρωινάδικα και μεσημεριανάδικα.",
    symptoms = "Συμπτώματα - ψευδαισθήσεις σχετικά με την ικανότητα του ατόμου να παρουσιάσει εκπομπή μαγειρικής.",
    cure = "Θεραπεία - Ένας εκπαιδευμένος Ψυχίατρος πρέπει να πείσει τον ασθενή να πουλήσει την τηλεόρασή του και να αγοράσει ραδιόφωνο.",
  },
  fake_blood = {
    name = "Ψεύτικο Αίμα",
    cause = "Αιτία - ο ασθενής συνήθως έχει πέσει θύμα φάρσας.",
    symptoms = "Συμπτώματα - Κόκκινο υγρό στις φλέβες το οποίο εξατμίζεται σε επαφή με ρούχα.",
    cure = "Θεραπεία - Καθησυχασμός από την Ψυχίατρο είναι ο μόνος τρόπος αντιμετώπισης αυτού του προβλήματος.",
  },
  general_practice = {
    name = "Γενική Ιατρική",
  },
  chronic_nosehair = {
    name = "Χρόνιο Τρίχωμα στην Μύτη",
    cause = "Αιτία - Το να ρουφάει περιφρονητικά την μύτη του ο ασθενής σε κοινωνικά κατώτερα άτομα από αυτόν.",
    symptoms = "Συμπτώματα - τρίχωμα στην μύτη τόσο μεγάλο που ένας ασβός θα μπορούσε να κάνει φωλιά.",
    cure = "Θεραπεία - Ένα αηδιαστικό φίλτρο αφαίρεσης τριχών χορηγείται διά της στοματικής οδού, το οποίο ετοιμάζεται από μία Νοσοκόμα στο Φαρμακείο.",
  },
  diag_ward = {
    name = "Διαγ Πτέρυγα",
  },
  gut_rot = {
    name = "Σαπισμένη Κοιλιά",
    cause = "Αιτία - Mrs. O'Malley's Good Time Whisky Cough Mixture.",
    symptoms = "Συμπτώματα - δεν υπάρχει πια βήχας αλλά ούτε και στομαχικά τοιχώματα.",
    cure = "Θεραπεία - Μία Νοσοκόμα χορηγεί μια επιλογή από διαλυμμένα χυμικά για να επικαλύψει την κοιλίτσα.",
  },
  corrugated_ankles = {
    name = "Πτυχωτοί Αστράγαλοι",
    cause = "Αιτία - το να οδηγάς το αυτοκίνητό σου πάνω από τα μέτρα αποσυμφόρησης της κυκλοφορίας στον δρόμο.",
    symptoms = "Συμπτώματα - Τα παπούτσια δεν φοριούνται άνετα.",
    cure = "Θεραπεία - Ένα ελαφρώς τοξικό μείγμα βοτάνων και μυρωδικών πίνεταιγια να ισιώσουν οι αστράγαλοι.",
  },
  baldness = {
    name = "Φαλάκρα",
    cause = "Αιτία - να λες ψέματα και να επινοείς ιστορίες για να είσαι δημοφιλής.",
    symptoms = "Συμπτώματα - αστραφτερό κεφάλι και ντροπιασμός.",
    cure = "Θεραπεία - Το μαλλί συγχωνεύεται στο κεφάλι του ασθενή πολύ εύκολα χρησιμοποιώντας αυτό το οδυνηρό μηχάνημα.",
  },
  golf_stones = {
    name = "Πέτρες του Γκολφ",
    cause = "Αιτία - έκθεση σε δηλητηριώδες αέριο μέσα σε μπαλάκια του γκολφ. (ΣτΜ: λογοπαίγνιο, Golf Stones από Gallstones, δηλαδή πέτρες στην χολή)",
    symptoms = "Συμπτώματα - παραλήρημα και μεγάλη ντροπή.",
    cure = "Θεραπεία - Οι πέτρες πρέπει να αφαιρεθούν μέσω εγχείρησης η οποία απαιτεί δύο Χειρουργούς.",
  },
  heaped_piles = {
    name = "Ενοχλητικές Ζοχάδες",
    cause = "Αιτία - το να στέκεσαι κοντά σε ψύκτες.",
    symptoms = "Συμπτώματα - Ο ασθενής αισθάνεται σαν να κάθεται πάνω σε μια σακούλα από βόλους.",
    cure = "Θεραπεία - Ένα ευχάριστο αλλά πολύ όξινο ποτό διαλύει τις αιμορροΐδες εκ των έσω.",
  },
  transparency = {
    name = "Διαφάνεια",
    cause = "Αιτία - το να γλείφεις το γιαούρτι από το αλουμινένιο καπάκι μιας συσκευασίας γιαουρτιού.",
    symptoms = "Συμπτώματα - η σάρκα γίνεται διάφανη και αηδιαστική.",
    cure = "Θεραπεία - Ένα διάλυμα από το Φαρμακείο που εμπεριέχει ειδικά ψυγμένο και χρωματισμένο νερό θεραπεύει αυτήν την ασθένεια.",
  },
  uncommon_cold = {
    name = "Μη Συνηθισμένο Κρυολόγημα",
    cause = "Αιτία - μικρά σωματίδια μύξας στον αέρα. (ΣτΜ: λογοπαίγνιο, Uncommon Cold έναντι Common Cold)",
    symptoms = "Συμπτώματα - μύτη που τρέχει, φτέρνισμα και ξεθωριασμένα πνευμόνια.",
    cure = "Θεραπεία - Μια μεγάλη γουλιά από σιρόπι μη συνηθισμένου κρυολογήματος φτιαγμένο από ειδικά συστατικά στο Φαρμακείο θα θεραπεύσει αυτήν την ασθένεια.",
  },
  diag_cardiogram = {
    name = "Διαγ Καρδιογράφημα",
  },
  fractured_bones = {
    name = "Σπασμένα Κόκκαλα",
    cause = "Αιτία - πέφτοντας από μεγάλα ύψη πάνω σε τσιμέντο.",
    symptoms = "Συμπτώματα - ηχηρό κρακ και ανικανότητα χρήσης των εμπλεκόμενων άκρων.",
    cure = "Θεραπεία - Τοποθετείται ο γύψος και ύστερα αφαιρείται χρησιμοποιώντας ένα μηχάνημα αφαίρεσης με λέιζερ.",
  },
  jellyitis = {
    name = "Ζελετίτιδα",
    cause = "Αιτία - δίαιτα υψηλή σε ζελατίνη και υπερβολική άσκηση.",
    symptoms = "Συμπτώματα - υπερβολικό ταρακούνημα και γερές πτώσεις.",
    cure = "Θεραπεία - Ο ασθενής βυθίζεται για λίγο στην Δεξαμενή Ζελέ που βρίσκεται σε ένα ειδικό δωμάτιο.",
  },
  unexpected_swelling = {
    name = "Αναπάντεχο Πρήξιμο",
    cause = "Αιτία - οτιδήποτε αναπάντεχο.",
    symptoms = "Σύμπτωμα - πρήξιμο.",
    cure = "Θεραπεία - Το πρήξιμο μπορεί να μειωθεί μόνο ανοίγοντας το σημείο σε μια εγχείρηση που απαιτεί δύο Χειρουργούς.",
  },
  the_squits = {
    name = "Τσιρλιπιπί",
    cause = "Αιτία - το να τρως πίτσα που έχει πέσει κάτω από την ηλεκτρική κουζίνα.",
    symptoms = "Συμπτώματα - εεε. Σίγουρα μπορείτε να μαντέψετε.",
    cure = "Θεραπεία - Ένα γλουτενικό μιξ από ινώδη φαρμακευτικά χημικά στερεοποιεί τα μέσα του ασθενούς.",
  },
}
debug_patient_window = {
  caption = "Ασθενής Αποσφαλμάτωσης",
}
menu_display = {
  high_res = "  HIGH RES  ",
  shadows = "  ΣΚΙΕΣ  ",
  mcga_lo_res = "  MCGA LO-RES  ",
}
letter = {
  [1] = {
    [1] = "Αγαπητέ/ή %s//",
    [2] = "Θαυμάσια! Χειριστήκατε την διαχείριση αυτού του νοσοκομείου εξαιρετικά. Εμείς οι φίρμες στο Υπουργείο Υγείας αναρωτιόμαστε αν θα σας ενδιέφερε να αναλάβετε ένα μεγαλύτερο έργο. Υπάρχει μια δουλειά για την οποία θεωρούμε πως θα ήσασταν τέλειος/α. Ο μισθός είναι $%d. Σκεφτείτε το.//",
    [3] = "Θα σας ενδιέφερε μια δουλειά στο Νοσοκομείο %s ;",
  },
  [2] = {
    [1] = "Αγαπητέ/ή %s//",
    [2] = "Περίφημα! Έχετε κάνει φανταστική πρόοδο στο νοσοκομείο σας. Έχουμε βρει κάπου αλλού να αναλάβετε, αν θέλετε μια αλλαγή σκηνικού και μία φρέσκια σειρά προκλήσεων. Δεν είναι ανάγκη να δεχτείτε, αλλά ίσως αξίζει αν το κάνετε. Ο μισθός είναι $%d//",
    [3] = "Θέλετε μια δουλειά στο Νοσοκομείο %s ;",
  },
  [3] = {
    [1] = "Αγαπητέ/ή %s//",
    [2] = "Είχατε τεράστια επιτυχία κατά την διάρκεια της θητείας σας σε αυτό το νοσοκομείο. Ως εκ τούτου, προβλέπουμε μεγάλα πράγματα για εσάς, και θα θέλαμε να σας προσφέρουμε μια θέση αλλού. Ο μισθός είναι $%d, και πιστεύουμε ότι θα λατρεύατε την νέα πρόκληση που θα συνιστούσε.//",
    [3] = "Θέλετε μια θέση στο Νοσοκομείο %s ;",
  },
  [4] = {
    [1] = "Αγαπητέ/ή %s//",
    [2] = "Συγχαρητήρια! Εμείς στο Υπουργείο είμαστε πολύ εντυπωσιασμένοι από τις ικανότητες σας στην διαχείριση ενός νοσοκομείου. Είστε σίγουρα ένα χρυσό αγόρι/κορίτσι στον τομέα της Υγείας. Πιστεύουμε ότι θα προτιμούσατε μια ελαφρώς δυσκολότερη δουλειά, ωστόσο. Θα πληρωθείτε μισθό της τάξεως των $%d, αλλά η απόφαση είναι δική σας.//",
    [3] = "Ενδιαφέρεστε να δουλέψετε στο Νοσοκομείο %s ;",
  },
  [5] = {
    [1] = "Αγαπητέ/ή %s//",
    [2] = "Γεια και πάλι. Σεβόμαστε την επιθυμία σας να μην αποχωρήσετε από αυτό το μαγευτικό νοσοκομείο, αλλά θα σας παροτρύναμε να το ξανασκεφτείτε. Θα σας προσφέρουμε τον μεγάλο μισθό των $%d αν είστε έτοιμοι να εγκατασταθείτε σε άλλο νοσοκομείο και να το τρέξετε εξίσου καλά.//",
    [3] = "Θα θέλατε να εγκατασταθείτε στο Νοσοκομείο %s τώρα;",
  },
  [6] = {
    [1] = "Αγαπητέ/ή %s//",
    [2] = "Χαιρετισμοί. Γνωρίζουμε πόσοι ευχαριστημένοι ήσασταν σε αυτό το αξιαγάπητο ίδρυμα το οποίο έχετε διαχειριστεί εξαιρετικά καλά, αλλά πιστεύουμε ότι πρέπει να σκεφτείτε να προχωρήσετε την καριέρα σας τώρα. Φυσικά, θα λάβετε το ποσό των $%d αν αποφασίσετε να μετακομίσετε. Αξίζει να το σκεφτείτε.//",
    [3] = "Θα θέλατε μια θέση στο Νοσοκομείο %s ;",
  },
  [7] = {
    [1] = "Αγαπητέ/ή %s//",
    [2] = "Καλή σας μέρα! Το Υπουργείο Υγείας θα ήθελε να γνωρίζει αν θα αναθεωρούσατε την απόφασή σας να μείνετε στο τρέχον νοσοκομείο σας. Εκτιμούμε το γεγονός ότι έχετε ένα εξαιρετικό νοσοκομείο, αλλά αισθανόμαστε ότι θα ήταν πολύ καλό για εσάς αν αποδεχόσασταν ένα πιο απαιτητικό πόστο, και έναν μισθό της τάξεως των $%d.//",
    [3] = "Θα επιθυμούσατε μια θέση στο Νοσοκομείο %s ;",
  },
  [8] = {
    [1] = "Αγαπητέ/ή %s//",
    [2] = "Γεια σας και πάλι. Αρνηθήκατε την πρόταση μας στο προηγούμενο γράμμα, στο οποίο σας προσφέραμε μια πολυπόθητη θέση σε έναν νέο νοσοκομείο, και έναν μεγαλύτερο μισθό της τάξεως των $%d. Αισθανόμαστε, ωστόσο, πως πρέπει να αναθεωρήσετε αυτή σας την απόφαση. Έχουμε την τέλεια δουλειά για εσάς, βλέπετε.//",
    [3] = "Θα λάβετε το πόστο στο Νοσοκομείο %s ; Παρακαλώ πολύ;",
  },
  [9] = {
    [1] = "Αγαπητέ/ή %s//",
    [2] = "Αποδείξατε πως είστε οι πιο ικανοί διαχειριστές νοσοκομείου στην μακριά και πολυτάραρχη ιστορία της ιατρικής. Ένα τέτοιο τεράστιο κατόρθωμα δεν μπορεί να μείνει αβράβευτο, έτσι θα θέλαμε να σας προσφέρουμε το πόστο του Ανώτατου Προέδρου Όλων των Νοσοκομείων. Αυτή είναι μια τιμητική δουλειά, και έρχεται μαζί με έναν μισθό της τάξεως των $%d. Θα σας δοθεί η δικιά σας προσωπική πομπή η οποία θα είναι αντάξια των κατορθωμάτων σας και θα είναι μακριά σαν τεράστια απόδειξη σουπερμάρκετ, και επιπλέον ο κόσμος θα σας δείχνει την εκτίμησή του όπου και αν βρίσκεστε.//",
    [3] = "Ευχαριστούμε για όλα όσα έχετε κάνει. Είθε να απολαύσετε μια μακρόχρονη ημι-συνταξιοδότηση.//",
    [4] = "",
  },
  [10] = {
    [1] = "Αγαπητέ/ή %s//",
    [2] = "Συγχαρητήρια που διοικήσατε επιτυχώς όλα τα νοσοκομεία στα οποία σας αναθέσαμε. Μια τέτοια εξαιρετική επίδοση σας καθιστά άξιους για δωρεάν μετάβαση σε όλες τις πόλεις του κόσμου. Θα σας δοθεί σύνταξη της τάξεως των $%d, και επιπλέον μια λιμουζίνα, και το μόνο που ζητάμε είναι να ταξιδεύετε από πολή σε πόλη, να συναντάτε τους θαυμαστές σας και να προωθείτε την δουλειά όλων των νοσοκομείων παντού.//",
    [3] = "Είμαστε όλοι περήφανοι για εσάς. Δεν υπάρχει ούτε ένα άτομο ανάμεσα μας που δεν αισθάνεται ευγνωμοσύνη για όλη την σκληρή δουλειά που έχετε κάνει στο να σώζετε ζωές.//",
    [4] = "",
  },
  [11] = {
    [1] = "Αγαπητέ/ή %s//",
    [2] = "Η καριέρα σας είναι υποδειγματική, και αποτελείτε έμπνευση για όλους μας. Σας ευχαριστούμε για την διαχείριση τόσων πολλών νοσοκομειακών ιδρυμάτων και για το ότι τα καταφέρατε εξίσου καλά σε όλα. Θα θέλαμε να σας προσφέρουμε αποδοχές εφ' όρου ζωής της τάξεως των $%d, και ζητάμε μονάχα να ταξιδεύετε από πόλη σε πόλη με επίσημο κάμπριο αυτοκίνητο, κάνοντας διαλέξεις για το πως κατορθώσατε τόσα πολλά σε τόσο σύντομο χρονικό διάστημα.//",
    [3] = "Αποτελείτε παράδειγμα για κάθε ορθά σκεπτόμενο άτομο, και ανεξαιρέτως, όλοι στον κόσμο σας θεωρούν ως το υπέρτατο απόκτημα στον χώρο της ιατρικής.//",
    [4] = "",
  },
  [12] = {
    [1] = "Αγαπητέ/ή %s//",
    [2] = "Η επιτυχής καριέρα σας ως ο/η σπουδαιότερος/η διαχειριστής/στρια νοσοκομείου από την εποχή του Μωυσή φτάνει κοντά στο τέλος της. Ωστόσο, τέτοιος ήταν ο αντίκτυπός σας στον χώρο της ιατρικής, που το Υπουργείο επιθυμεί να σας προσφερεί αποδοχές της τάξεως των $%d απλά για να κάνετε εμφανίσεις εκ μέρους μας, ανοίγοντας πανηγύρια, να παρευρίσκεστε σε καθελκύσεις πλοίων και να πηγαίνεται σε talk shows. Όλος ο κόσμος ζητωκραυγάζει για εσάς, συν το ότι θα ήταν καλή δημοσιότητα για όλους μας!//",
    [3] = "Παρακαλούμε αποδεχτείτε αυτήν την θέση. Δεν θα είναι δύσκολη δουλειά, και θα σας προσφέρουμε αυτοκίνητο και αστυνομική συνοδεία οπουδήποτε και αν πηγαίνετε.//",
    [4] = "",
  },
  custom_level_completed = "Συγχαρητήρια! Έχετε ολοκληρώσει όλους τους στόχους σε αυτό το προσαρμοσμένο επίπεδο!",
  campaign_completed = "Απίστευτο! Καταφέρατε να ολοκληρώσετε όλα τα επίπεδα. Τώρα μπορείτε να χαλαρώσετε και να απολαύσετε τον χρόνο σας γράφοντας σε forums στο Ίντερνετ για τα κατορθώματά σας. Καλή τύχη!",
  campaign_level_missing = "Συγγνώμη, αλλά το επόμενο επίπεδο αυτής της καμπάνιας φαίνεται να λείπει. (Όνομα: %s)",
  return_to_main_menu = "Θα θέλατε να επιστρέψετε πίσω στο κύριο μενού ή να συνεχίσετε να παίζετε;",
  dear_player = "Αγαπητέ/ή %s",
  campaign_level_completed = "Καλή δουλειά! Κερδίσατε το επίπεδο. Αλλά δεν τελείωσε ακόμα! Θα επιθυμούσατε μια θέση στο Νοσοκομείο %s;",
}
room_classes = {
  clinics = "Κλινικές",
  treatment = "Θεραπεία",
  diagnosis = "Διάγνωση",
  facilities = "Μονάδες",
}
graphs = {
  money_in = "Έσοδα",
  visitors = "Επισκέπτες",
  balance = "Προϋπολογισμός",
  cures = "Θεραπείες",
  wages = "Μισθοί",
  reputation = "Φήμη",
  time_spans = {
    [1] = "1 Χρόνος",
    [2] = "12 Χρόνια",
    [3] = "48 Χρόνια",
  },
  deaths = "Θάνατοι",
  money_out = "Έξοδα",
}
staff_list = {
  skill = "ΔΕΞΙΟΤΗΤΑ",
  total_wages = "ΣΥΝΟΛΙΚΟΙ ΜΙΣΘΟΙ",
  tiredness = "ΚΟΥΡΑΣΗ",
  morale = "ΗΘΙΚΟ",
}
